@@ -42,7 +42,7 @@ void audiomanager::initsound()
             copystring(d, "Audio devices: ");
 
             // null separated device string
-            for(const ALchar *c = devices; c[strlen(c)+1]; c += strlen(c)+1)
+            for(const ALchar *c = devices; *c; c += strlen(c)+1)
             {
                 if(c!=devices) concatstring(d, ", ");
                 concatstring(d, c);
@@ -124,15 +124,15 @@ void audiomanager::musicpreload(int id)
 {
     if(nosound) return;
     stopsound();
-    if(musicvol && (id>=M_FLAGGRAB && id<=M_LASTMINUTE2))
+    if(musicvol && (id>=M_FLAGGRAB && id<=M_LASTMINUTE2) && musics.inrange(id))
     {
         char *name = musics[id];
         conoutf("preloading music #%d : %s", id, name);
         if(gamemusic->open(name))
         {
-            defformatstring(whendone)("musicvol %d", musicvol);
+            /*defformatstring(whendone)("musicvol %d", musicvol);
             musicdonecmd = newstring(whendone);
-            //conoutf("when done: %s", musicdonecmd);
+            conoutf("when done: %s", musicdonecmd);*/
             const int preloadfadetime = 3;
             gamemusic->fadein(lastmillis, preloadfadetime);
             gamemusic->fadeout(lastmillis+2*preloadfadetime, preloadfadetime);
@@ -181,9 +181,17 @@ void audiomanager::setmusicvol(int musicvol)
     if(gamemusic) gamemusic->setvolume(musicvol > 0 ? musicvol/255.0f : 0);
 }
 
+void reloadmapsoundconfig();
+
 void audiomanager::setlistenervol(int vol)
 {
-    if(!nosound) alListenerf(AL_GAIN, vol/255.0f);
+    static int oldvol = 0;
+    if(!nosound)
+    {
+        alListenerf(AL_GAIN, vol/255.0f);
+        if(vol && !oldvol && !mapsounds.length()) reloadmapsoundconfig();
+    }
+    oldvol = vol;
 }
 
 void audiomanager::registermusic(char *name)
@@ -193,7 +201,7 @@ void audiomanager::registermusic(char *name)
     musics.add(newstring(name));
 }
 
-int audiomanager::findsound(char *name, int vol, vector<soundconfig> &sounds)
+int audiomanager::findsound(const char *name, int vol, vector<soundconfig> &sounds)
 {
     loopv(sounds)
     {
@@ -202,7 +210,7 @@ int audiomanager::findsound(char *name, int vol, vector<soundconfig> &sounds)
     return -1;
 }
 
-int audiomanager::addsound(char *name, int vol, int maxuses, bool loop, vector<soundconfig> &sounds, bool load, int audibleradius)
+int audiomanager::addsound(const char *name, int vol, int maxuses, bool loop, vector<soundconfig> &sounds, bool load, int audibleradius)
 {
     if(nosound) return -1;
     if(!soundvol) return -1;
@@ -225,20 +233,23 @@ int audiomanager::addsound(char *name, int vol, int maxuses, bool loop, vector<s
     return sounds.length()-1;
 }
 
-void audiomanager::preloadmapsound(entity &e, bool trydl)
+bool audiomanager::preloadmapsound(entity &e, bool trydl)
 {
-    if(e.type!=SOUND || !mapsounds.inrange(e.attr1)) return;
+    if(e.type!=SOUND || !mapsounds.inrange(e.attr1)) return false;
     sbuffer *buf = mapsounds[e.attr1].buf;
-    if(!buf->load(trydl) && !trydl) conoutf("\f3failed to load sample %s", buf->name);
+    if(!buf->load(trydl) && !trydl) { conoutf("\f3failed to load sample %s", buf->name); return false; }
+    return true;
 }
 
-void audiomanager::preloadmapsounds(bool trydl)
+bool audiomanager::preloadmapsounds(bool trydl)
 {
+    int missing = 0;
     loopv(ents)
     {
         entity &e = ents[i];
-        if(e.type==SOUND) preloadmapsound(e, trydl);
+        if(e.type == SOUND && !preloadmapsound(e, trydl)) missing++;
     }
+    return !missing;
 }
 
 void audiomanager::applymapsoundchanges() // during map editing, drop all mapsounds so they can be re-added
@@ -268,11 +279,12 @@ void audiomanager::soundcleanup()
     mapsounds.shrink(0);
     locations.deletecontents();
     gamesounds.shrink(0);
-    bufferpool.clear();
 
     // kill scheduler
     sourcescheduler::instance().reset();
 
+    bufferpool.clear();
+
     // shutdown openal
     alcMakeContextCurrent(NULL);
     if(context) alcDestroyContext(context);
@@ -330,15 +342,10 @@ void audiomanager::updateplayerfootsteps(playerent *p)
     {
         // play footsteps
 
-        int grounddist;
-        if( ((int)p->o.x) >= 0 && ((int)p->o.y) >= 0 && ((int)p->o.x) < ssize && ((int)p->o.y) < ssize) { // sam's fix to the sound crash
-            grounddist = hdr.waterlevel-S((int)p->o.x, (int)p->o.y)->floor;
-        }else{
-            grounddist = 0;
-        }
-//        int grounddist = hdr.waterlevel-S((int)p->o.x, (int)p->o.y)->floor;
-        bool water = p->o.z-p->eyeheight+0.25f<hdr.waterlevel;
-        if(water && grounddist>p->eyeheight) return; // don't play step sound when jumping into water
+        float grounddist = 0;
+        if(!((int(p->o.x) | int(p->o.y)) & ~(ssize - 1))) grounddist = waterlevel - S((int)p->o.x, (int)p->o.y)->floor;
+        bool water = p->o.z - p->eyeheight+0.25f < waterlevel;
+        if(water && grounddist > p->eyeheight) return; // don't play step sound when jumping into water
 
         int stepsound;
         if(p->crouching) stepsound = water ? S_WATERFOOTSTEPSCROUCH : S_FOOTSTEPSCROUCH; // crouch
@@ -358,7 +365,7 @@ void audiomanager::updateplayerfootsteps(playerent *p)
         {
             // play
             float rndoffset = float(rnd(500))/500.0f;
-            audiomgr._playsound(stepsound, ref, local ? SP_HIGH : SP_LOW, rndoffset);
+            _playsound(stepsound, ref, local ? SP_HIGH : SP_LOW, rndoffset);
         }
     }
 }
@@ -420,12 +427,11 @@ void voicecom(char *sound, char *text)
     static int last = 0;
     if(!last || lastmillis-last > 2000)
     {
-        extern int voicecomsounds;
         defformatstring(soundpath)("voicecom/%s", sound);
         int s = audiomgr.findsound(soundpath, 0, gamesounds);
-        if(s < 0 || s < S_AFFIRMATIVE || s >= S_NULL) return;
+        if(!gamesound_isvoicecom(s)) return;
         if(voicecomsounds>0) audiomgr.playsound(s, SP_HIGH);
-        if(s >= S_NICESHOT) // public
+        if(gamesound_ispublicvoicecom(s)) // public
         {
             addmsg(SV_VOICECOM, "ri", s);
             toserver(text);
@@ -559,7 +565,7 @@ void audiomanager::updateaudio()
 
     alcSuspendContext(context); // don't process sounds while we mess around
 
-    bool alive = player1->state!=CS_DEAD;
+    bool alive = player1->state==CS_ALIVE;
     bool firstperson = camera1==player1 || (player1->isspectating() && player1->spectatemode==SM_DEATHCAM);
 
     // footsteps
@@ -572,7 +578,7 @@ void audiomanager::updateaudio()
     }
 
     // water
-    bool underwater = /*alive &&*/ firstperson && hdr.waterlevel>player1->o.z+player1->aboveeye;
+    bool underwater = /*alive &&*/ firstperson && waterlevel > player1->o.z + player1->aboveeye;
     updateloopsound(S_UNDERWATER, underwater);
 
     // tinnitus
@@ -618,7 +624,7 @@ void audiomanager::updateaudio()
 
             if(hearable && !loc) // play
             {
-                _playsound(sound, entref, SP_HIGH, 0.0f, true);
+                _playsound(sound, entref, SP_LOW, 0.0f, true);
             }
             else if(!hearable && loc) // stop
             {
@@ -641,7 +647,9 @@ void audiomanager::updateaudio()
             {
                 char *cmd = musicdonecmd;
                 musicdonecmd = NULL;
+                setcontext("hook", "musicdonecmd");
                 execute(cmd);
+                resetcontext();
                 delete[] cmd;
             }
         }
@@ -766,13 +774,16 @@ bool staticreference::operator==(const worldobjreference &other)
 
 audiomanager audiomgr;
 
-COMMANDF(sound, "i", (int *n)
+const char *soundprioritynames[] = {"LOW", "NORMAL", "HIGH", "HIGHEST", ""}; // keep in sync with enum { SP_LOW = 0, SP_NORMAL, SP_HIGH, SP_HIGHEST }
+
+COMMANDF(sound, "is", (int *n, char *priorityname)
 {
-    audiomgr.playsound(*n);
+    int priority = getlistindex(priorityname, soundprioritynames, true, SP_NORMAL);
+    audiomgr.playsound(*n, priority);
 });
 
 COMMANDF(applymapsoundchanges, "", (){
-    audiomgr.applymapsoundchanges();
+    if(m_coop || !multiplayer("applymapsoundchanges")) audiomgr.applymapsoundchanges();
 });
 
 COMMANDF(unmuteallsounds, "", () {
@@ -789,23 +800,84 @@ COMMANDF(soundmuted, "i", (int *n)
     intret(audiomgr.soundmuted(*n));
 });
 
+VAR(mapsoundchanged, 0, 0, 1);
+
 COMMANDF(mapsoundreset, "", ()
 {
     audiomgr.mapsoundreset();
+    mapconfigdata.mapsoundlines.shrink(0);
+    mapsoundchanged = 1;
+    flagmapconfigchange();
 });
 
-VARF(soundchannels, 4, 32, 1024, audiomgr.setchannels(soundchannels); );
+VARF(soundchannels, 4, 128, 1024, audiomgr.setchannels(soundchannels));
 
-VARFP(soundvol, 0, 128, 255, audiomgr.setlistenervol(soundvol); );
+VARFP(soundvol, 0, 128, 255, audiomgr.setlistenervol(soundvol));
 
 COMMANDF(registersound, "siii", (char *name, int *vol, int *loop, int *audibleradius)
 {
     intret(audiomgr.addsound(name, *vol, -1, *loop != 0, gamesounds, true, *audibleradius));
 });
 
+void registerdefaultsounds()
+{
+    loopi(S_NULL)
+    {
+        ASSERT(soundcfg[i].key == i);
+        audiomgr.addsound(soundcfg[i].name, soundcfg[i].vol, -1, soundcfg[i].loop != 0, gamesounds, true, soundcfg[i].audibleradius);
+    }
+}
+
+const char *soundcategories[SC_NUM + 1] = { "PAIN", "OWNPAIN", "WEAPON", "PICKUP", "MOVEMENT", "BULLET", "OTHER", "VOICECOM", "TEAM", "PUBLIC", "FFA", "FLAGONLY", "" };
+
+void enumsounds(char *what)
+{
+    vector<char *> w;
+    explodelist(what, w);
+    int flags = 0, nflags = 0;
+    loopv(w)
+    {
+        if(w[i][0] == '!') nflags |= 1 << getlistindex(w[i] + 1, soundcategories, false, SC_NUM);
+        else flags |= 1 << getlistindex(w[i], soundcategories, false, SC_NUM);
+        delstring(w[i]);
+    }
+    vector<char> res;
+    loopi(S_NULL) if((soundcfg[i].flags & flags) && !(soundcfg[i].flags & nflags)) cvecprintf(res, "%d %s\n", i, escapestring(soundcfg[i].desc));
+    resultcharvector(res, -1);
+}
+COMMAND(enumsounds, "s");
+
+const char *mapsoundbasepath = "packages/audio/", *mapsoundfinalpath = "ambience/";
+const int mapsoundbasepath_n = strlen(mapsoundbasepath), mapsoundfinalpath_n = strlen(mapsoundfinalpath);
+
 COMMANDF(mapsound, "si", (char *name, int *maxuses)
 {
-    audiomgr.addsound(name, 255, *maxuses, true, mapsounds, false, 0);
+    filtertext(name, name, FTXT__MEDIAFILEPATH);
+    defformatstring(stripped)("%s%s", mapsoundbasepath, name[0] == '.' && name[1] == '/' ? name + 2 : name);
+    unixpath(path(stripped));
+    if(mapconfigdata.mapsoundlines.length() > 254)
+    {
+        conoutf("\f3error: too many mapsounds");
+        flagmapconfigerror(LWW_CONFIGERR);
+        scripterr();
+    }
+    else if(!strncmp(stripped, mapsoundbasepath, mapsoundbasepath_n))
+    {
+        name = stripped + mapsoundbasepath_n;
+        audiomgr.addsound(name, 255, *maxuses, true, mapsounds, false, 0);
+        intret(mapconfigdata.mapsoundlines.length());
+        if(!strncmp(name, mapsoundfinalpath, mapsoundfinalpath_n)) name += mapsoundfinalpath_n; // base path for mapsoundlines is "packages/audio/ambience"
+        copystring(mapconfigdata.mapsoundlines.add().name, name);
+        mapconfigdata.mapsoundlines.last().maxuses = *maxuses;
+        mapsoundchanged = 1;
+        flagmapconfigchange();
+    }
+    else
+    {
+        conoutf("\f3error: mapsound \"%s\" outside packages/audio/", stripped);
+        flagmapconfigerror(LWW_CONFIGERR);
+        scripterr();
+    }
 });
 
 COMMANDF(registermusic, "s", (char *name)
@@ -823,3 +895,210 @@ COMMANDF(music, "sis", (char *name, int *millis, char *cmd)
     audiomgr.music(name, *millis, cmd);
 });
 
+void mapsoundslotusage(int *n) // returns all entity indices that use a mapsound of slot n
+{
+    string res = "";
+    loopv(ents) if(ents[i].type == SOUND && ents[i].attr1 == *n) concatformatstring(res, "%s%d", i ? " " : "", i);
+    result(res);
+}
+COMMAND(mapsoundslotusage, "i");
+
+void reloadmapsoundconfig()
+{
+    vector<char> sc;
+    getcurrentmapconfig(sc, true); // only soundconfig
+    execute(sc.getbuf()); // cheap and easy way ;)
+}
+
+void deletemapsoundslot(int *n, char *opt) // delete mapsound slot - only if unused or "purge" is specified
+{
+    EDITMP("deletemapsoundslot");
+    if(!mapconfigdata.mapsoundlines.inrange(*n)) return;
+    bool purgeall = !strcmp(opt, "purge"), slotused = false;
+    loopv(ents) if(ents[i].type == SOUND && ents[i].attr1 == *n) slotused = true;
+    if(!purgeall && slotused) { conoutf("mapsound slot #%d is in use: can't delete", *n); return; }
+    audiomgr.mapsoundreset();
+    int deld = 0;
+    loopv(ents) if(ents[i].type == SOUND)
+    {
+        entity &e = ents[i];
+        if(e.attr1 == *n)
+        { // delete entity
+            deleted_ents.add(e);
+            deletesoundentity(e);
+            memset(&e, 0, sizeof(persistent_entity));
+            e.type = NOTUSED;
+            deld++;
+        }
+        else if(e.attr1 > *n) e.attr1--; // adjust models in higher slots
+    }
+    mapconfigdata.mapsoundlines.remove(*n);
+    reloadmapsoundconfig();
+    defformatstring(s)(" (%d mapsounds purged)", deld);
+    conoutf("mapsound slot #%d deleted%s", *n, deld ? s : "");
+    mapsoundchanged = 1;
+    unsavededits++;
+    hdr.flags |= MHF_AUTOMAPCONFIG; // requires automapcfg
+}
+COMMAND(deletemapsoundslot, "is");
+
+void editmapsoundslot(int *n, char *name, char *maxuses) // edit slot parameters != ""
+{
+    string res = "";
+    if(mapconfigdata.mapsoundlines.inrange(*n))
+    {
+        mapsoundline &msl = mapconfigdata.mapsoundlines[*n];
+        if((*name || *maxuses) && !noteditmode("editmapsoundslot") && !multiplayer("editmapsoundslot"))
+        { // change attributes
+            if(*maxuses) msl.maxuses = strtol(maxuses, NULL, 0);
+            if(*name) copystring(msl.name, strncmp(name, mapsoundfinalpath, mapsoundfinalpath_n) ? name : name + mapsoundfinalpath_n);
+            reloadmapsoundconfig();
+            mapsoundchanged = 1;
+            unsavededits++;
+            hdr.flags |= MHF_AUTOMAPCONFIG; // requires automapcfg
+        }
+        formatstring(res)("\"%s\" %d", msl.name, msl.maxuses); // give back all current attributes
+    }
+    result(res);
+}
+COMMAND(editmapsoundslot, "iss");
+
+void getmapsoundorigin(char *fname)
+{
+    if(!*fname) return;
+    defformatstring(s)("packages/audio/ambience/%s", fname);
+    findfile(path(s), "r");
+    const char *res = s;
+    switch(findfilelocation)
+    {
+        case FFL_ZIP:     res = "zip";                                                break;
+        case FFL_WORKDIR: res = fileexists(s, "r") ? "official" : "<file not found>"; break;
+        case FFL_HOME:    res = "custom";                                             break;
+        default:          formatstring(s)("package dir #%d", findfilelocation);       break;
+    }
+    result(res);
+}
+COMMAND(getmapsoundorigin, "s");
+
+void mapsoundslotbyname(char *name) // returns the slot(s) that a certain mapsound file is used in
+{
+    string res = "";
+    loopv(mapconfigdata.mapsoundlines) if(!strcmp(name, mapconfigdata.mapsoundlines[i].name)) concatformatstring(res, "%s%d", *res ? " " : "", i);
+    result(res);
+}
+COMMAND(mapsoundslotbyname, "s");
+
+void getmapsoundlist() // create a list of mapsound filenames
+{
+    vector<char *> files;
+    listfilesrecursive("packages/audio/ambience", files);
+    files.sort(stringsort);
+    loopvrev(files) if(files.inrange(i + 1) && !strcmp(files[i], files[i + 1])) delstring(files.remove(i + 1)); // remove doubles
+    int pn = strlen("packages/audio/ambience/");
+    vector<char> res;
+    loopv(files)
+    {
+        if(!strncmp(files[i], "packages/audio/ambience/", pn))
+        {
+            const char *s = files[i] + pn;
+            int sn = strlen(s);
+            if(sn > 4)
+            {
+                const char *e = s + sn - 4;
+                if(!strcmp(e, ".ogg") || !strcmp(e, ".wav")) cvecprintf(res, "\"%s\"\n", s);
+            }
+        }
+    }
+    files.deletearrays();
+    resultcharvector(res, -1);
+}
+COMMAND(getmapsoundlist, "");
+
+struct tempmsslot { mapsoundline m; vector<int> oldslot; bool used; };
+
+int tempmscmp(tempmsslot *a, tempmsslot *b)
+{
+    int n = strcmp(a->m.name, b->m.name);
+    if(n) return n;
+    return a->m.maxuses - b->m.maxuses;
+}
+
+int tempmssort(tempmsslot *a, tempmsslot *b)
+{
+    int n = tempmscmp(a, b);
+    return n ? n : a->oldslot[0] - b->oldslot[0];
+}
+
+int tempmsunsort(tempmsslot *a, tempmsslot *b)
+{
+    return a->oldslot[0] - b->oldslot[0];
+}
+
+void sortmapsoundslots(char **args, int numargs)
+{
+    bool nomerge = false, mergeused = false, nosort = false, unknownarg = false;
+    loopi(numargs) if(args[i][0])
+    {
+        if(!strcasecmp(args[i], "nomerge")) nomerge = true;
+        else if(!strcasecmp(args[i], "nosort")) nosort = true;
+        else if(!strcasecmp(args[i], "mergeused")) mergeused = true;
+        else { conoutf("sortmapsoundslots: unknown argument \"%s\"", args[i]); unknownarg = true; }
+    }
+
+    EDITMP("sortmapsoundslots");
+    if(unknownarg || mapconfigdata.mapsoundlines.length() < 3) return;
+
+    vector<tempmsslot> tempslots;
+    loopv(mapconfigdata.mapsoundlines)
+    {
+        tempslots.add().m = mapconfigdata.mapsoundlines[i];
+        tempslots.last().oldslot.add(i);
+        tempslots.last().used = false;
+    }
+    loopv(ents) if(ents[i].type == SOUND && tempslots.inrange(ents[i].attr1)) tempslots[ents[i].attr1].used = true;
+    tempslots.sort(tempmssort);
+
+    // remove double entries
+    if(!nomerge) loopvrev(tempslots) if(i > 0)
+    {
+        tempmsslot &s1 = tempslots[i], &s0 = tempslots[i - 1];
+        if(!tempmscmp(&s0, &s1) && (mergeused || !s0.used || !s1.used))
+        {
+            if(s1.used) s0.used = true;
+            loopvj(s1.oldslot) s0.oldslot.add(s1.oldslot[j]);
+            tempslots.remove(i);
+        }
+    }
+    if(nosort) tempslots.sort(tempmsunsort);
+
+    // create translation table
+    uchar newslot[256];
+    loopk(256) newslot[k] = k;
+    loopv(tempslots)
+    {
+        tempmsslot &t = tempslots[i];
+        loopvj(t.oldslot)
+        {
+            if(t.oldslot[j] < 256) newslot[t.oldslot[j]] = i;
+        }
+    }
+
+    // translate all mapsound entities
+    loopv(ents) if(ents[i].type == SOUND) ents[i].attr1 = newslot[ents[i].attr1 & 255];
+
+    conoutf("%d mapsound slots%s, %d %sslots merged", tempslots.length(), nosort ? "" : " sorted", mapconfigdata.mapsoundlines.length() - tempslots.length(), mergeused ? "" : "unused ");
+
+    // rewrite mapmodel slot list
+    mapconfigdata.mapsoundlines.shrink(tempslots.length());
+    loopv(tempslots)
+    {
+        mapconfigdata.mapsoundlines[i] = tempslots[i].m;
+    }
+    reloadmapsoundconfig();
+    unsavededits++;
+    mapsoundchanged = 1;
+    hdr.flags |= MHF_AUTOMAPCONFIG; // requires automapcfg
+}
+COMMAND(sortmapsoundslots, "v");
+
+
