@@ -9,6 +9,9 @@ VARP(networkdebug, 0, 0, 1);
 extern bool watchingdemo;
 extern string clientpassword;
 
+void *downloaddemomenu = NULL;
+static vector<mline> demo_mlines;
+
 packetqueue pktlogger;
 
 void neterr(const char *s)
@@ -24,6 +27,10 @@ void neterr(const char *s)
 
 VARP(autogetmap, 0, 1, 1); // only if the client doesn't have that map
 VARP(autogetnewmaprevisions, 0, 1, 1);
+VAR(disconnectonmaperror, 0, 1, 1);
+
+extern bool incompatiblemap;
+extern void trydisconnect();
 
 bool localwrongmap = false;
 int MA = 0, Hhits = 0; // flowtron: moved here
@@ -34,17 +41,18 @@ bool changemapserv(char *name, int mode, int download, int revision)        // f
     if(m_demo) return true;
     if(m_coop)
     {
-        if(!name[0] || !load_world(name)) empty_world(0, true);
+        if(!name[0] || load_world(name) < 0) empty_world(0, true);
         return true;
     }
     else if(player1->state==CS_EDITING) { /*conoutf("SANITY drop from EDITING");*/ toggleedit(true); } // fix stuck-in-editmode bug
-    bool loaded = load_world(name);
+    bool loaded = load_world(name) >= 0;
+    if(disconnectonmaperror && incompatiblemap && multiplayer(NULL)) trydisconnect();
     if(download > 0)
     {
         bool revmatch = hdr.maprevision == revision || revision == 0;
         if(watchingdemo)
         {
-            if(!revmatch) conoutf(_("%c3demo was recorded on map revision %d, you have map revision %d"), CC, revision, hdr.maprevision);
+            if(loaded && !revmatch) conoutf("\f3demo was recorded on map revision %d, you have map revision %d", revision, hdr.maprevision);
         }
         else
         {
@@ -58,15 +66,15 @@ bool changemapserv(char *name, int mode, int download, int revision)        // f
                 else
                 {
                     defformatstring(msg)("map '%s' revision: local %d, provided by server %d", name, hdr.maprevision, revision);
-                    alias("__getmaprevisions", msg);
+                    alias("__getmaprevisions", msg, true);
                     showmenu("getmap");
                 }
             }
             else
             {
-                if(!loaded || download < 10) conoutf(_("\"getmap\" to download the current map from the server"));
-                else conoutf(_("\"getmap\" to download a %s version of the current map from the server"),
-                         revision == 0 ? _("different") : (revision > hdr.maprevision ? _("newer") : _("older")));
+                if(!loaded || download < 10) conoutf("\"getmap\" to download the current map from the server");
+                else conoutf("\"getmap\" to download a %s version of the current map from the server",
+                         revision == 0 ? "different" : (revision > hdr.maprevision ? "newer" : "older"));
             }
         }
     }
@@ -78,7 +86,7 @@ bool changemapserv(char *name, int mode, int download, int revision)        // f
 // don't care if he's in the scenery or other players,
 // just don't overlap with our client
 
-void updatepos(playerent *d)
+void updateplayerpos(playerent *d)
 {
     const float r = player1->radius+d->radius;
     const float dx = player1->o.x-d->o.x;
@@ -105,8 +113,6 @@ void updatelagtime(playerent *d)
 
 extern void trydisconnect();
 
-VARP(maxrollremote, 0, 0, 20); // bound remote "roll" values by our maxroll?!
-
 void parsepositions(ucharbuf &p)
 {
     int type;
@@ -117,7 +123,7 @@ void parsepositions(ucharbuf &p)
         {
             int cn, f, g;
             vec o, vel;
-            float yaw, pitch, roll = 0;
+            float yaw, pitch = 0;
             bool scoping;//, shoot;
             if(type == SV_POSC)
             {
@@ -128,7 +134,7 @@ void parsepositions(ucharbuf &p)
                 o.y = q.getbits(usefactor + 4) / DMF;
                 yaw = q.getbits(9) * 360.0f / 512;
                 pitch = (q.getbits(8) - 128) * 90.0f / 127;
-                roll = !q.getbits(1) ? (q.getbits(6) - 32) * 20.0f / 31 : 0.0f;
+                if(!q.getbits(1)) q.getbits(6);
                 if(!q.getbits(1))
                 {
                     vel.x = (q.getbits(4) - 8) / DVELF;
@@ -157,7 +163,7 @@ void parsepositions(ucharbuf &p)
                 yaw   = (float)getuint(p);
                 pitch = (float)getint(p);
                 g = getuint(p);
-                if ((g>>3) & 1) roll  = (float)(getint(p)*20.0f/125.0f);
+                if ((g>>3) & 1) getint(p);
                 if (g & 1) vel.x = getint(p)/DVELF; else vel.x = 0;
                 if ((g>>1) & 1) vel.y = getint(p)/DVELF; else vel.y = 0;
                 if ((g>>2) & 1) vel.z = getint(p)/DVELF; else vel.z = 0;
@@ -187,7 +193,6 @@ void parsepositions(ucharbuf &p)
                 sniperrifle *sr = (sniperrifle *)d->weaponsel;
                 sr->scoped = d->scoping = scoping;
             }
-            d->roll = roll;
             d->strafe = (f&3)==3 ? -1 : f&3;
             f >>= 2;
             d->move = (f&3)==3 ? -1 : f&3;
@@ -198,7 +203,7 @@ void parsepositions(ucharbuf &p)
             f >>= 2;
             d->last_pos = totalmillis;
             updatecrouch(d, f&1);
-            updatepos(d);
+            updateplayerpos(d);
             updatelagtime(d);
             extern int smoothmove, smoothdist;
             if(d->state==CS_DEAD)
@@ -227,8 +232,6 @@ void parsepositions(ucharbuf &p)
             if(d->state==CS_LAGGED || d->state==CS_SPAWNING) d->state = CS_ALIVE;
             // when playing a demo spectate first player we know about
             if(player1->isspectating() && player1->spectatemode==SM_NONE) togglespect();
-            extern void clamproll(physent *pl);
-            if(maxrollremote) clamproll((physent *) d);
             break;
         }
 
@@ -243,7 +246,6 @@ char *mlayout = NULL;
 int Mv = 0, Ma = 0, F2F = 1000 * MINFF; // moved up:, MA = 0;
 float Mh = 0;
 extern int connected;
-extern int lastpm;
 extern bool noflags;
 bool item_fail = false;
 int map_quality = MAP_IS_EDITABLE;
@@ -251,15 +253,16 @@ int map_quality = MAP_IS_EDITABLE;
 /// TODO: many functions and variables are redundant between client and server... someone should redo the entire server code and unify client and server.
 bool good_map() // call this function only at startmap
 {
+    return true;
     if (mlayout) MA = checkarea(sfactor, mlayout);
 
     F2F = 1000 * MINFF;
     if(m_flags)
     {
-        flaginfo &f0 = flaginfos[0];
-        flaginfo &f1 = flaginfos[1];
+//        flaginfo &f0 = flaginfos[0];
+  //      flaginfo &f1 = flaginfos[1];
 #define DIST(x) (f0.pos.x - f1.pos.x)
-        F2F = (!numflagspawn[0] || !numflagspawn[1]) ? 1000 * MINFF : DIST(x)*DIST(x)+DIST(y)*DIST(y);
+        F2F = 1000;//(!numflagspawn[0] || !numflagspawn[1]) ? 1000 * MINFF : DIST(x)*DIST(x)+DIST(y)*DIST(y);
 #undef DIST
     }
 
@@ -273,8 +276,9 @@ bool good_map() // call this function only at startmap
         {
             entity &e2 = ents[j];
             if (e2.type < I_CLIPS || e2.type > I_AKIMBO || i == j) continue;
+            // only I_CLIPS, I_AMMO, I_GRENADE, I_HEALTH, I_HELMET, I_ARMOUR, I_AKIMBO
 #define DIST(x) (e1.x - e2.x)
-#define DIST_ATT ((e1.z + e1.attr1) - (e2.z + e2.attr1))
+#define DIST_ATT ((e1.z + float(e1.attr1) / entscale[e1.type][0]) - (e2.z + float(e2.attr1) / entscale[e2.type][0]))
             float r2 = DIST(x)*DIST(x) + DIST(y)*DIST(y) + DIST_ATT*DIST_ATT;
 #undef DIST_ATT
 #undef DIST
@@ -313,6 +317,16 @@ VARP(hudextras, 0, 0, 3);
 
 int teamworkid = -1;
 
+char *strcaps(const char *s, bool on)
+{
+    static string r;
+    char *o = r;
+    if(on) while(*s && o < &r[sizeof(r)-1]) *o++ = toupper(*s++);
+    else while(*s && o < &r[sizeof(r)-1]) *o++ = tolower(*s++);
+    *o = '\0';
+    return r;
+}
+
 void showhudextras(char hudextras, char value){
     void (*outf)(const char *s, ...) = (hudextras > 1 ? hudoutf : conoutf);
     bool caps = hudextras < 3 ? false : true;
@@ -363,20 +377,12 @@ int lastspawn = 0;
 
 void onCallVote(int type, int vcn, char *text, char *a)
 {
-    if(identexists("onCallVote"))
-    {
-        defformatstring(runas)("%s %d %d [%s] [%s]", "onCallVote", type, vcn, text, a);
-        execute(runas);
-    }
+    exechook(HOOK_SP_MP, "onCallVote", "%d %d [%s] [%s]", type, vcn, text, a);
 }
 
 void onChangeVote(int mod, int id)
 {
-    if(identexists("onChangeVote"))
-    {
-        defformatstring(runas)("%s %d %d", "onChangeVote", mod, id);
-        execute(runas);
-    }
+    exechook(HOOK_SP_MP, "onChangeVote", "%d %d", mod, id);
 }
 
 VARP(voicecomsounds, 0, 1, 2);
@@ -386,7 +392,6 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
 {
     static char text[MAXTRANS];
     int type, joining = 0;
-    bool demoplayback = false;
 
     while(p.remaining())
     {
@@ -410,7 +415,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
             DEBUGVAR(d);
             ASSERT(type>=0 && type<SV_NUM);
             DEBUGVAR(messagenames[type]);
-            protocoldebug(true);
+            protocoldebug(DEBUGCOND);
         }
         else protocoldebug(false);
         #endif
@@ -422,7 +427,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 int mycn = getint(p), prot = getint(p);
                 if(prot!=CUR_PROTOCOL_VERSION && !(watchingdemo && prot == -PROTOCOL_VERSION))
                 {
-                    conoutf(_("%c3incompatible game protocol (local protocol: %d :: server protocol: %d)"), CC, CUR_PROTOCOL_VERSION, prot);
+                    conoutf("\f3incompatible game protocol (local protocol: %d :: server protocol: %d)", CUR_PROTOCOL_VERSION, prot);
                     conoutf("\f3if this occurs a lot, obtain an upgrade from \f1http://assault.cubers.net");
                     if(watchingdemo) conoutf("breaking loop : \f3this demo is using a different protocol\f5 : end it now!"); // SVN-WiP-bug: causes endless retry loop else!
                     else disconnect();
@@ -430,7 +435,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 }
                 sessionid = getint(p);
                 player1->clientnum = mycn;
-                if(getint(p) > 0) conoutf(_("INFO: this server is password protected"));
+                if(getint(p) > 0) conoutf("INFO: this server is password protected");
                 sendintro();
                 break;
             }
@@ -480,13 +485,13 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
             {
                 int cn = getint(p);
                 getstring(text, p);
-                filtertext(text, text);
+                filtertext(text, text, FTXT__CHAT);
                 playerent *d = getclient(cn);
                 if(!d) break;
                 if(d->ignored) clientlogf("ignored: %s%s %s", colorname(d), type == SV_TEAMTEXT ? ":" : "", text);
                 else
                 {
-                    if(m_teammode) conoutf(type == SV_TEAMTEXTME ? "\f1%s %s" : "%s:\f1 %s", colorname(d), highlight(text));
+                    if(m_teammode || team_isspect(player1->team)) conoutf(type == SV_TEAMTEXTME ? "\f1%s %s" : "%s:\f1 %s", colorname(d), highlight(text));
                     else conoutf(type == SV_TEAMTEXTME ? "\f0%s %s" : "%s:\f0 %s", colorname(d), highlight(text));
                 }
                 break;
@@ -503,7 +508,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 else if(d)
                 {
                     getstring(text, p);
-                    filtertext(text, text);
+                    filtertext(text, text, FTXT__CHAT);
                     if(d->ignored && d->clientrole != CR_ADMIN) clientlogf("ignored: %s%s %s", colorname(d), type == SV_TEXT ? ":" : "", text);
                     else conoutf(type == SV_TEXTME ? "\f0%s %s" : "%s:\f0 %s", colorname(d), highlight(text));
                 }
@@ -514,7 +519,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
             {
                 int cn = getint(p);
                 getstring(text, p);
-                filtertext(text, text);
+                filtertext(text, text, FTXT__CHAT);
                 playerent *d = getclient(cn);
                 if(!d) break;
                 if(d->ignored) clientlogf("ignored: pm %s %s", colorname(d), text);
@@ -522,11 +527,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 {
                     conoutf("%s (PM):\f9 %s", colorname(d), highlight(text));
                     lastpm = d->clientnum;
-                    if(identexists("onPM"))
-                    {
-                        defformatstring(onpm)("onPM %d [%s]", d->clientnum, text);
-                        execute(onpm);
-                    }
+                    exechook(HOOK_SP_MP, "onPM", "%d [%s]", d->clientnum, text);
                 }
                 break;
             }
@@ -540,15 +541,15 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 int downloadable = getint(p);
                 int revision = getint(p);
                 localwrongmap = !changemapserv(text, mode, downloadable, revision);
-                if(m_arena && joining>2) deathstate(player1);
+                if(m_arena && joining > 1 && !watchingdemo) deathstate(player1);
                 break;
             }
 
             case SV_ITEMLIST:
             {
                 int n;
-                resetspawns();
-                while((n = getint(p))!=-1) setspawn(n, true);
+                resetpickups();
+                while((n = getint(p))!=-1) setpickupspawn(n, true);
                 break;
             }
 
@@ -560,17 +561,12 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
 
             case SV_SWITCHNAME:
                 getstring(text, p);
-                filtertext(text, text, 0, MAXNAMELEN);
+                filtertext(text, text, FTXT__PLAYERNAME, MAXNAMELEN);
                 if(!text[0]) copystring(text, "unarmed");
                 if(d)
                 {
-                    if(strcmp(d->name, text))
-                        conoutf(_("%s is now known as %s"), colorname(d), colorname(d, text));
-                    if(identexists("onNameChange"))
-                    {
-                        defformatstring(onnamechange)("onNameChange %d \"%s\"", d->clientnum, text);
-                        execute(onnamechange);
-                    }
+                    if(strcmp(d->name, text)) conoutf("%s is now known as %s", colorname(d), colorname(d, text));
+                    exechook(HOOK_SP, "onNameChange", "%d \"%s\"", d->clientnum, text);
                     copystring(d->name, text, MAXNAMELEN+1);
                     updateclientname(d);
                 }
@@ -601,26 +597,25 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                     break;
                 }
                 getstring(text, p);
-                filtertext(text, text, 0, MAXNAMELEN);
+                filtertext(text, text, FTXT__PLAYERNAME, MAXNAMELEN);
                 if(!text[0]) copystring(text, "unarmed");
                 if(d->name[0])          // already connected
                 {
                     if(strcmp(d->name, text))
-                        conoutf(_("%s is now known as %s"), colorname(d), colorname(d, text));
+                        conoutf("%s is now known as %s", colorname(d), colorname(d, text));
                 }
                 else                    // new client
                 {
-                    conoutf(_("connected: %s"), colorname(d, text));
+                    conoutf("connected: %s", colorname(d, text));
                 }
                 copystring(d->name, text, MAXNAMELEN+1);
-                if(identexists("onConnect"))
-                {
-                    defformatstring(onconnect)("onConnect %d", d->clientnum);
-                    execute(onconnect);
-                }
+                exechook(HOOK_SP_MP, "onConnect", "%d", d->clientnum);
                 loopi(2) d->setskin(i, getint(p));
                 d->team = getint(p);
 
+//                d->maxroll = (float)clamp(getint(p), 0, ROLLMOVMAX);          FIXME: uncomment on protocol bump + etc.
+//                d->maxrolleffect = (float)clamp(getint(p), 0, ROLLEFFMAX);    FIXME: uncomment on protocol bump
+
                 if(!demo || !watchingdemo || demoprotocol > 1132) d->address = getint(p); // partial IP address
 
                 if(m_flags) loopi(2)
@@ -637,13 +632,9 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 int cn = getint(p);
                 playerent *d = getclient(cn);
                 if(!d) break;
-                if(d->name[0]) conoutf(_("player %s disconnected"), colorname(d));
+                if(d->name[0]) conoutf("player %s disconnected", colorname(d));
                 zapplayer(players[cn]);
-                if(identexists("onDisconnect"))
-                {
-                    defformatstring(ondisconnect)("onDisconnect %d", d->clientnum);
-                    execute(ondisconnect);
-                }
+                exechook(HOOK_SP_MP, "onDisconnect", "%d", d->clientnum);
                 break;
             }
 
@@ -652,12 +643,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 int val = getint(p);
                 if(!d) break;
                 if(val) d->state = CS_EDITING;
-                else
-                {
-                    //2011oct16:flowtron:keep spectator state
-                    //specators shouldn't be allowed to toggle editmode for themselves. they're ghosts!
-                    d->state = d->state==CS_SPECTATE?CS_SPECTATE:CS_ALIVE;
-                }
+                else d->state = CS_ALIVE;
                 break;
             }
 
@@ -675,6 +661,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 loopi(NUMGUNS) s->ammo[i] = getint(p);
                 loopi(NUMGUNS) s->mag[i] = getint(p);
                 s->state = CS_SPAWNING;
+                arenaintermission = 0;
                 if(s->lifesequence==0) s->resetstats(); //NEW
                 break;
             }
@@ -684,7 +671,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 if ( map_quality == MAP_IS_BAD )
                 {
                     loopi(6+2*NUMGUNS) getint(p);
-                    conoutf(_("map deemed unplayable - fix it before you can spawn"));
+                    conoutf("map deemed unplayable - fix it before you can spawn");
                     break;
                 }
 
@@ -707,8 +694,8 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 arenaintermission = 0;
                 if(m_arena && !localwrongmap)
                 {
-                    closemenu(NULL);
-                    conoutf(_("new round starting... fight!"));
+                    if(connected) closemenu(NULL);
+                    conoutf("new round starting... fight!");
                     hudeditf(HUDMSG_TIMER, "FIGHT!");
                     if(m_botmode) BotManager.RespawnBots();
                 }
@@ -725,7 +712,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 vec from, to;
                 loopk(3) to[k] = getint(p)/DMF;
                 playerent *s = getclient(scn);
-                if(!s || !weapon::valid(gun)) break;
+                if(!s || !valid_weapon(gun)) break;
                 loopk(3) from[k] = s->o.v[k];
                 if(gun==GUN_SHOTGUN) createrays(from, to);
                 s->lastaction = lastmillis;
@@ -773,24 +760,24 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
 
             case SV_AUTHREQ:
             {
-                extern int autoauth;
+//                extern int autoauth;
                 getstring(text, p);
-                if(autoauth && text[0] && tryauth(text)) conoutf("server requested authkey \"%s\"", text);
+            //    if(autoauth && text[0] && tryauth(text)) conoutf("server requested authkey \"%s\"", text);
                 break;
             }
 
             case SV_AUTHCHAL:
             {
                 getstring(text, p);
-                authkey *a = findauthkey(text);
-                uint id = (uint)getint(p);
+           //     authkey *a = findauthkey(text);
+           //     uint id = (uint)getint(p);
                 getstring(text, p);
-                if(a && a->lastauth && lastmillis - a->lastauth < 60*1000)
+           //     if(a && a->lastauth && lastmillis - a->lastauth < 60*1000)
                 {
-                    vector<char> buf;
-                    answerchallenge(a->key, text, buf);
+             //       vector<char> buf;
+           //         answerchallenge(a->key, text, buf);
                     //conoutf("answering %u, challenge %s with %s", id, text, buf.getbuf());
-                    addmsg(SV_AUTHANS, "rsis", a->desc, id, buf.getbuf());
+           //         addmsg(SV_AUTHANS, "rsis", a->desc, id, buf.getbuf());
                 }
                 break;
             }
@@ -912,7 +899,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                     discscore &ds = discscores.add();
                     ds.team = team;
                     getstring(text, p);
-                    filtertext(ds.name, text, 0, MAXNAMELEN);
+                    filtertext(ds.name, text, FTXT__PLAYERNAME, MAXNAMELEN);
                     ds.flags = getint(p);
                     ds.frags = getint(p);
                     ds.deaths = getint(p);
@@ -923,7 +910,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
             case SV_ITEMSPAWN:
             {
                 int i = getint(p);
-                setspawn(i, true);
+                setpickupspawn(i, true);
                 break;
             }
 
@@ -935,41 +922,78 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 break;
             }
 
-            case SV_EDITH:              // coop editing messages, should be extended to include all possible editing ops
-            case SV_EDITT:
-            case SV_EDITS:
-            case SV_EDITD:
-            case SV_EDITE:
+            case SV_EDITXY:              // coop editing messages, should be extended to include all possible editing ops
             {
+                int cmd = getint(p);
                 int x  = getint(p);
                 int y  = getint(p);
                 int xs = getint(p);
                 int ys = getint(p);
-                int v  = getint(p);
-                block b = { x, y, xs, ys };
-                switch(type)
+                int a1 = getint(p);
+                int a2 = getint(p);
+                if(m_coop && !OUTBORD(x,y)  && xs > 0 && ys > 0 && !OUTBORD(x + xs-1, y + ys - 1))
                 {
-                    case SV_EDITH: editheightxy(v!=0, getint(p), b); break;
-                    case SV_EDITT: edittexxy(v, getint(p), b); break;
-                    case SV_EDITS: edittypexy(v, b); break;
-                    case SV_EDITD: setvdeltaxy(v, b); break;
-                    case SV_EDITE: editequalisexy(v!=0, b); break;
+                    block b = { x, y, xs, ys };
+                    switch(cmd)
+                    {
+                        case EDITXY_HEIGHT:   editheightxy(a1 != 0, a2, b); break;
+                        case EDITXY_TEX:      edittexxy(a1, a2, b);         break;
+                        case EDITXY_TYPE:     edittypexy(a1, b);            break;
+                        case EDITXY_VDELTA:   setvdeltaxy(a1, b);           break;
+                        case EDITXY_EQUALISE: editequalisexy(a1 != 0, b);   break;
+                        case EDITXY_TAG:      edittagxy(a1, a2, b);         break;
+                        case EDITXY_SLOPE:    slopexy(a1, a2, b);           break;
+                        case EDITXY_STAIRS:   stairsxy(a1, a2, b);          break;
+                        case EDITXY_FLIPROT:  selfliprotate(b, a1);         break;
+                    }
                 }
                 break;
             }
 
+            case SV_EDITARCH:
+            {
+                int av[50];  // MAXARCHVERT, hardcoded
+                int x  = getint(p);
+                int y  = getint(p);
+                int xs = getint(p);
+                int ys = getint(p);
+                int a1 = getint(p);
+                loopi(50) av[i] = getint(p);
+                if(m_coop && !OUTBORD(x,y)  && xs > 0 && ys > 0 && !OUTBORD(x + xs-1, y + ys - 1))
+                {
+                    block b = { x, y, xs, ys };
+                    archxy(a1, av, b);
+                }
+                break;
+            }
+
+            case SV_EDITBLOCK:
+            {
+                int bx = getuint(p), by = getuint(p), bxs = getuint(p), bys = getuint(p), light = getuint(p);
+                ucharbuf *pp = getgzbuf(p);
+                if(m_coop) netblockpastexy(pp, bx, by, bxs, bys, light);
+                freegzbuf(pp);
+                break;
+            }
+
             case SV_NEWMAP:
             {
                 int size = getint(p);
-                if(size>=0) empty_world(size, true);
-                else empty_world(-1, true);
+                if(!m_coop) break;
+                if(size < 0 && sfactor > 9) break; // don't enlarge over 10 in MP
+                empty_world(size, true);
                 if(d && d!=player1)
-                    conoutf(size>=0 ? _("%s started a new map of size %d") : _("%s enlarged the map to size %d"), colorname(d), sfactor);
+                    conoutf(size>=0 ? "%s started a new map of size %d" : "%s enlarged the map to size %d", colorname(d), sfactor);
                 break;
             }
 
             case SV_EDITENT:            // coop edit of ent
             {
+                if(!m_coop)
+                {
+                    loopi(12) getint(p);
+                    break;
+                }
                 uint i = getint(p);
                 while((uint)ents.length()<=i) ents.add().type = NOTUSED;
                 int to = ents[i].type;
@@ -992,6 +1016,9 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 ents[i].attr2 = getint(p);
                 ents[i].attr3 = getint(p);
                 ents[i].attr4 = getint(p);
+                ents[i].attr5 = getint(p);
+                ents[i].attr6 = getint(p);
+                ents[i].attr7 = getint(p);
                 ents[i].spawned = false;
                 if(ents[i].type==LIGHT || to==LIGHT) calclight();
                 if(ents[i].type==SOUND) audiomgr.preloadmapsound(ents[i]);
@@ -1087,12 +1114,12 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
             {
                 int acn = getint(p);
                 playerent *alive = getclient(acn);
-                conoutf(_("the round is over! next round in 5 seconds..."));
-                if(m_botmode && acn==-2) hudoutf(_("the bots have won the round!"));
-                else if(!alive) hudoutf(_("everyone died!"));
-                else if(m_teammode) hudoutf(_("team %s has won the round!"), team_string(alive->team));
-                else if(alive==player1) hudoutf(_("you are the survivor!"));
-                else hudoutf(_("%s is the survivor!"), colorname(alive));
+                conoutf("the round is over! next round in 5 seconds...");
+                if(m_botmode && acn==-2) hudoutf("the bots have won the round!");
+                else if(acn==-1) hudoutf("everyone died!");
+                else if(m_teammode) hudoutf("team %s has won the round!", team_string(alive->team));
+                else if(alive==player1) hudoutf("you are the survivor!");
+                else hudoutf("%s is the survivor!", colorname(alive));
                 arenaintermission = lastmillis;
                 break;
             }
@@ -1101,7 +1128,7 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
             {
                 extern int spawnpermission;
                 spawnpermission = getint(p);
-                if(spawnpermission == SP_REFILLMATCH) hudoutf("\f3You can now spawn to refill your team.");
+                if(spawnpermission == SP_REFILLMATCH) hudoutf("\f3you can spawn now to refill your team");
                 break;
             }
             case SV_FORCEDEATH:
@@ -1128,8 +1155,8 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                         if(pl->name[0])
                         {
                             // two messages required to allow for proper german translation - is there a better way to do it?
-                            if(pl==player1) conoutf(_("you claimed %s status"), r == CR_ADMIN ? "admin" : "master");
-                            else conoutf(_("%s claimed %s status"), colorname(pl), r == CR_ADMIN ? "admin" : "master");
+                            if(pl==player1) conoutf("you claimed %s status", r == CR_ADMIN ? "admin" : "master");
+                            else conoutf("%s claimed %s status", colorname(pl), r == CR_ADMIN ? "admin" : "master");
                         }
                     }
                 }
@@ -1141,11 +1168,13 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 int t = getint(p);
                 if(m_teammode)
                 {
-                    if(team_isvalid(t)) conoutf(_("you can't change to team %s"), team_string(t));
+                    if(team_isvalid(t)) conoutf("you can't change to team %s", team_string(t));
                 }
                 else
                 {
-                    conoutf(_("you can't change to %s mode"), team_isspect(t) ? _("spectate") : _("active"));
+                    if(team_isspect(t)) conoutf("you can't change to spectate mode");
+                    else if (player1->state!=CS_ALIVE) conoutf("you can't change to active mode");
+                    else conoutf("you can't switch teams while being alive");
                 }
                 break;
             }
@@ -1172,10 +1201,10 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                                 switch(ftr)
                                 {
                                     case FTR_PLAYERWISH:
-                                        conoutf(_("you're now in team %s"), nts);
+                                        conoutf("you're now in team %s", nts);
                                         break;
                                     case FTR_AUTOTEAM:
-                                        hudoutf(_("the server forced you to team %s"), nts);
+                                        hudoutf("the server forced you to team %s", nts);
                                         break;
                                 }
                             }
@@ -1186,18 +1215,18 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                                 switch(ftr)
                                 {
                                     case FTR_PLAYERWISH:
-                                        conoutf(_("player %s switched to team %s"), pls, nts); // new message
+                                        conoutf("player %s switched to team %s", pls, nts); // new message
                                         break;
                                     case FTR_AUTOTEAM:
-                                        if(watchingdemo) conoutf(_("the server forced %s to team %s"), colorname(d), nts);
-                                        else hudoutf(_("the server forced %s to %s team"), colorname(d), et ? _("the enemy") : _("your"));
+                                        if(watchingdemo || team_isspect(player1->team)) conoutf("the server forced %s to team %s", colorname(d), nts);
+                                        else hudoutf("the server forced %s to %s team", colorname(d), et ? "the enemy" : "your");
                                         break;
                                 }
                             }
                             if(you && !team_isspect(d->team) && team_isspect(fnt) && d->state == CS_DEAD) spectatemode(SM_FLY);
                         }
                     }
-                    else if(d->team != fnt && ftr == FTR_PLAYERWISH) conoutf(_("%s changed to active play"), you ? _("you") : colorname(d));
+                    else if(d->team != fnt && ftr == FTR_PLAYERWISH && !team_isactive(d->team)) conoutf("%s changed to active play", you ? "you" : colorname(d));
                     d->team = fnt;
                     if(team_isspect(d->team)) d->state = CS_SPECTATE;
                 }
@@ -1232,24 +1261,28 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
                 switch(type)
                 {
                     case SA_MAP:
+                    {
                         getstring(text, p);
-                        filtertext(text, text);
-                        itoa(a1, getint(p));
+                        int mode = getint(p);
+                        if(m_isdemo(mode)) filtertext(text, text, FTXT__DEMONAME);
+                        else filtertext(text, behindpath(text), FTXT__MAPNAME);
+                        itoa(a1, mode);
                         defformatstring(t)("%d", getint(p));
                         v = newvotedisplayinfo(d, type, text, a1, t);
                         break;
+                    }
                     case SA_KICK:
                     case SA_BAN:
                     {
                         itoa(a1, getint(p));
                         getstring(text, p);
-                        filtertext(text, text);
+                        filtertext(text, text, FTXT__CHAT);
                         v = newvotedisplayinfo(d, type, a1, text);
                         break;
                     }
                     case SA_SERVERDESC:
                         getstring(text, p);
-                        filtertext(text, text);
+                        filtertext(text, text, FTXT__SERVDESC);
                         v = newvotedisplayinfo(d, type, text, NULL);
                         break;
                     case SA_STOPDEMO:
@@ -1328,17 +1361,34 @@ void parsemessages(int cn, playerent *d, ucharbuf &p, bool demo = false)
             case SV_SENDDEMOLIST:
             {
                 int demos = getint(p);
-                if(!demos) conoutf(_("no demos available"));
-                else loopi(demos)
+                menureset(downloaddemomenu);
+                demo_mlines.shrink(0);
+                if(!demos)
                 {
-                    getstring(text, p);
-                    conoutf("%d. %s", i+1, text);
+                    conoutf("no demos available");
+                    mline &m = demo_mlines.add();
+                    copystring(m.name, "no demos available");
+                    menuitemmanual(downloaddemomenu,m.name);
+                }
+                else
+                {
+                    demo_mlines.reserve(demos);
+                    loopi(demos)
+                    {
+                        getstring(text, p);
+                        conoutf("%d. %s", i+1, text);
+                        mline &m = demo_mlines.add();
+                        formatstring(m.name)("%d. %s", i+1, text);
+                        formatstring(m.cmd)("getdemo %d", i+1);
+                        menuitemmanual(downloaddemomenu, m.name, m.cmd);
+                    }
                 }
                 break;
             }
 
             case SV_DEMOPLAYBACK:
             {
+                bool demoplayback = false;
                 string demofile;
                 extern char *curdemofile;
                 if(demo && watchingdemo && demoprotocol == 1132)
@@ -1423,8 +1473,8 @@ const char *parseDemoFilename(char *srvfinfo)
     {
         int fip = 0;
         char sep[] = ":";
-        char *pch;
-        pch = strtok (srvfinfo,sep);
+        char *pch, *b;
+        pch = strtok_r(srvfinfo, sep, &b);
         while (pch != NULL && fip < 4)
         {
             fip++;
@@ -1436,7 +1486,7 @@ const char *parseDemoFilename(char *srvfinfo)
                 case 4: stamp = atoi(pch); break;
                 default: break;
             }
-            pch = strtok (NULL, sep);
+            pch = strtok_r(NULL, sep, &b);
         }
         copystring(srvmap, pch);
     }
@@ -1468,13 +1518,13 @@ void receivefile(uchar *data, int len)
                 break;
             }
             path(fname);
-            stream *demo = openrawfile(fname, "wb");
+            stream *demo = openfile(fname, "wb");
             if(!demo)
             {
-                conoutf(_("failed writing to \"%s\""), fname);
+                conoutf("failed writing to \"%s\"", fname);
                 return;
             }
-            conoutf(_("received demo \"%s\""), fname);
+            conoutf("received demo \"%s\"", fname);
             demo->write(&p.buf[p.len], demosize);
             delete demo;
             break;
@@ -1483,14 +1533,14 @@ void receivefile(uchar *data, int len)
         case SV_RECVMAP:
         {
             getstring(text, p);
-            conoutf(_("received map \"%s\" from server, reloading.."), text);
+            conoutf("received map \"%s\" from server, reloading..", text);
             int mapsize = getint(p);
             int cfgsize = getint(p);
             int cfgsizegz = getint(p);
             /* int revision = */ getint(p);
             int size = mapsize + cfgsizegz;
             if(MAXMAPSENDSIZE < mapsize + cfgsizegz || cfgsize > MAXCFGFILESIZE) { // sam's suggestion
-                conoutf(_("map %s is too large to receive"), text);
+                conoutf("map %s is too large to receive", text);
             } else {
                 if(p.remaining() < size)
                 {
