@@ -3,57 +3,67 @@
 #include "cube.h"
 
 VAR(showclips, 0, 1, 1);
-VAR(showmodelclipping, 0, 0, 1);
+VARP(showmodelclipping, 0, 0, 1);
+VARP(showladderentities, 0, 0, 1);
+VARP(showplayerstarts, 0, 0, 1);
+VAR(edithideentmask, 0, 0, INT_MAX);
 
 vector<entity> ents;
-vector<int> eh_ents; // edithide entities
+
 const char *entmdlnames[] =
- {
-     "pistolclips", "ammobox", "nade", "health", "helmet", "kevlar", "akimbo", "nades", //FIXME
- };
-
- void renderent(entity &e)
- {
-     /* FIXME: if the item list change, this hack will be messed */
-
-     defformatstring(widn)("modmdlpickup%d", e.type-3);
-     defformatstring(mdlname)("pickups/%s", identexists(widn)?getalias(widn):
-
-     entmdlnames[e.type-I_CLIPS+(m_lss && e.type==I_GRENADE ? 5:0)]);
-
-     float z = (float)(1+sinf(lastmillis/100.0f+e.x+e.y)/20), yaw = lastmillis/10.0f;
-     rendermodel(mdlname, ANIM_MAPMODEL|ANIM_LOOP|ANIM_DYNALLOC, 0, 0, vec(e.x, e.y, z+S(e.x, e.y)->floor+e.attr1), yaw, 0);
- }
-
-void renderclip(entity &e)
 {
-    float xradius = max(float(e.attr2), 0.1f), yradius = max(float(e.attr3), 0.1f);
-    vec bbmin(e.x - xradius, e.y - yradius, float(S(e.x, e.y)->floor+e.attr1)),
-        bbmax(e.x + xradius, e.y + yradius, bbmin.z + max(float(e.attr4), 0.1f));
+    "pickups/nades", "pickups/pistolclips", "pickups/ammobox", "pickups/nade", "pickups/health", "pickups/helmet", "pickups/kevlar", "pickups/akimbo", ""   // doublenades + I_CLIPS..I_AKIMBO
+};
+
+void renderent(entity &e)
+{
+    const char *mdlname = entmdlnames[isitem(e.type) && !(m_lss && e.type == I_GRENADE) ? e.type - I_CLIPS + 1 : 0];  // render double nades in lss
+    float z = (float)(1+sinf(lastmillis/100.0f+e.x+e.y)/20), yaw = lastmillis/10.0f;
+    rendermodel(mdlname, ANIM_MAPMODEL|ANIM_LOOP|ANIM_DYNALLOC, 0, 0, vec(e.x, e.y, z+S(e.x, e.y)->floor + float(e.attr1) / ENTSCALE10), 0, yaw, 0);
+}
+
+void renderclip(int type, int x, int y, float xs, float ys, float h, float elev, float tilt, int shape)
+{
+    if(xs < 0.05f) xs = 0.05f;
+    if(ys < 0.05f) ys = 0.05f;
+    if(h < 0.1f) h = 0.1f;
+    vec bbmin(x - xs, y - ys, S(x, y)->floor + elev),
+        bbmax(x + xs, y + ys, bbmin.z + h);
+    vec bb[4];
+    loopi(4) bb[i] = bbmin;
+    bb[2].x = bb[1].x = bbmax.x;
+    bb[2].y = bb[3].y = bbmax.y;
+    vec o(x, y, bbmin.z);
+
+    float tx = 0, ty = 0, angle = 0;
+    switch(shape & 3)
+    {
+        case 1: tx = tilt; break; // tilt x
+        case 2: ty = tilt; break; // tilt y
+        case 3: angle = PI/4; break; // rotate 45°
+    }
+    loopi(4)
+    {
+        bb[i].sub(o).rotate_around_z(angle); // rotate
+        bb[i].z += bb[i].x * tx + bb[i].y * ty; // tilt
+        bb[i].add(o);
+    }
 
     glDisable(GL_TEXTURE_2D);
-    switch(e.type)
+    switch(type)
     {
         case CLIP:     linestyle(1, 0xFF, 0xFF, 0); break;  // yellow
         case MAPMODEL: linestyle(1, 0, 0xFF, 0);    break;  // green
         case PLCLIP:   linestyle(1, 0xFF, 0, 0xFF); break;  // magenta
+        case LADDER:   linestyle(1, 0, 0, 0xFF);    break;  // blue
     }
     glBegin(GL_LINES);
-
-    glVertex3f(bbmin.x, bbmin.y, bbmin.z);
-    loopi(2) glVertex3f(bbmax.x, bbmin.y, bbmin.z);
-    loopi(2) glVertex3f(bbmax.x, bbmax.y, bbmin.z);
-    loopi(2) glVertex3f(bbmin.x, bbmax.y, bbmin.z);
-    glVertex3f(bbmin.x, bbmin.y, bbmin.z);
-
-    glVertex3f(bbmin.x, bbmin.y, bbmax.z);
-    loopi(2) glVertex3f(bbmax.x, bbmin.y, bbmax.z);
-    loopi(2) glVertex3f(bbmax.x, bbmax.y, bbmax.z);
-    loopi(2) glVertex3f(bbmin.x, bbmax.y, bbmax.z);
-    glVertex3f(bbmin.x, bbmin.y, bbmax.z);
-
-    loopi(8) glVertex3f(i&2 ? bbmax.x : bbmin.x, i&4 ? bbmax.y : bbmin.y, i&1 ? bbmax.z : bbmin.z);
-
+    loopi(16)
+    {
+        int j = ((i + 1) % 8) / 2;
+        glVertex3f(bb[j].x, bb[j].y, bb[j].z + (i > 7 ? h : 0));
+    }
+    loopi(8) glVertex3f(bb[i / 2].x, bb[i / 2].y, bb[i / 2].z + (i & 1 ? h : 0));
     glEnd();
     glEnable(GL_TEXTURE_2D);
 }
@@ -65,64 +75,13 @@ void rendermapmodels()
         entity &e = ents[i];
         if(e.type==MAPMODEL)
         {
-            mapmodelinfo &mmi = getmminfo(e.attr2);
-            if(!&mmi) continue;
-            rendermodel(mmi.name, ANIM_MAPMODEL|ANIM_LOOP, e.attr4, 0, vec(e.x, e.y, (float)S(e.x, e.y)->floor+mmi.zoff+e.attr3), (float)((e.attr1+7)-(e.attr1+7)%15), 0, 10.0f);
+            mapmodelinfo *mmi = getmminfo(e.attr2);
+            if(!mmi) continue;
+            rendermodel(mmi->name, ANIM_MAPMODEL|ANIM_LOOP, e.attr4, 0, vec(e.x, e.y, S(e.x, e.y)->floor + mmi->zoff + float(e.attr3) / ENTSCALE5), e.attr6, float(e.attr1) / ENTSCALE10, float(e.attr5) / ENTSCALE10, 10.0f, 0, NULL, NULL, mmi->scale);
         }
     }
 }
 
-void showedithide()
-{
-    loopv(eh_ents)
-    {
-        if(eh_ents[i]>0 && eh_ents[i]<MAXENTTYPES) { conoutf("#%02d: %d : %s", i, eh_ents[i], entnames[eh_ents[i]]); }
-        else { conoutf("#%02d: %d : -n/a-", i, eh_ents[i]);  }
-    }
-}
-COMMAND(showedithide, "");
-
-void setedithide(char *text) // FIXME: human indexing inside
-{
-    eh_ents.setsize(0);
-    if(text && text[0] != '\0')
-    {
-        const char *s = strtok(text, " ");
-        do
-        {
-            bool k = false;
-            int sn = -1;
-            int tn = atoi(s);
-            loopi(MAXENTTYPES) if(!strcmp(entnames[i], s)) sn = i;
-            if(sn!=-1) { loopv(eh_ents) { if(eh_ents[i]==sn) { k = true; } } }
-            else sn = tn;
-            if(!k) { if(sn>0 && sn<MAXENTTYPES) eh_ents.add(sn); }
-            s = strtok(NULL, " ");
-        }
-        while(s);
-    }
-}
-COMMAND(setedithide, "c");
-
-void seteditshow(char *just)
-{
-    eh_ents.setsize(0);
-    if(just && just[0] != '\0')
-    {
-        const char *s = strtok(just, " ");
-        int sn = -1;
-        int tn = atoi(s);
-        loopi(MAXENTTYPES) if(!strcmp(entnames[i], s)) sn = i;
-        if(sn==-1) sn = tn;
-        loopi(MAXENTTYPES-1)
-        {
-            int j = i+1;
-            if(j!=sn) eh_ents.add(j);
-        }
-    }
-}
-COMMAND(seteditshow, "s");
-
 void renderentarrow(const entity &e, const vec &dir, float radius)
 {
     if(radius <= 0) return;
@@ -144,7 +103,7 @@ void renderentarrow(const entity &e, const vec &dir, float radius)
     loopi(5)
     {
         vec p(spoke);
-        p.rotate(2*M_PI*i/4.0f, dir);
+        p.rotate(PI2 * i / 4.0f, dir);
         p.add(arrowbase);
         glVertex3fv(p.v);
     }
@@ -163,43 +122,38 @@ void renderentities()
         if(lastmillis - lastsparkle >= 20)
         {
             lastsparkle = lastmillis - (lastmillis%20);
-            // closest see above
             loopv(ents)
             {
                 entity &e = ents[i];
                 if(e.type==NOTUSED) continue;
-                bool ice = false;
-                loopk(eh_ents.length()) if(eh_ents[k]==e.type) ice = true;
-                if(ice) continue;
+                if(edithideentmask & (1 << (e.type - 1))) continue;
                 vec v(e.x, e.y, e.z);
                 if(vec(v).sub(camera1->o).dot(camdir) < 0) continue;
-                //particle_splash(i == closest ? PART_ELIGHT : PART_ECLOSEST, 2, 40, v);
-                int sc = PART_ECARROT; // "carrot" (orange) - entity slot currently unused, possibly "reserved"
-                if(i==closest)
+                int sc = PART_ECARROT; // use "carrot" for unknown types
+                if(i == closest)
                 {
                     sc = PART_ECLOSEST; // blue
                 }
                 else switch(e.type)
                 {
-                    case LIGHT : sc = PART_ELIGHT; break; // white
-                    case PLAYERSTART: sc = PART_ESPAWN; break; // green
+                    case LIGHT:       sc = PART_ELIGHT;  break; // white
+                    case PLAYERSTART: sc = PART_ESPAWN;  break; // green
                     case I_CLIPS:
                     case I_AMMO:
-                    case I_GRENADE: sc = PART_EAMMO; break; // red
+                    case I_GRENADE:   sc = PART_EAMMO;   break; // red
                     case I_HEALTH:
                     case I_HELMET:
                     case I_ARMOUR:
-                    case I_AKIMBO: sc = PART_EPICKUP; break; // yellow
+                    case I_AKIMBO:    sc = PART_EPICKUP; break; // yellow
                     case MAPMODEL:
-                    case SOUND: sc = PART_EMODEL; break; // magenta
+                    case SOUND:       sc = PART_EMODEL;  break; // magenta
                     case LADDER:
                     case CLIP:
-                    case PLCLIP: sc = PART_ELADDER; break; // grey
-                    case CTF_FLAG: sc = PART_EFLAG; break; // turquoise
+                    case PLCLIP:      sc = PART_ELADDER; break; // grey
+                    case CTF_FLAG:    sc = PART_EFLAG;   break; // turquoise
                     default: break;
                 }
-                //particle_splash(sc, i==closest?6:2, i==closest?120:40, v);
-                particle_splash(sc, 2, 40, v);
+                particle_splash(sc, i == closest ? 14 : 2, i == closest ? 50 : 40, v);
             }
         }
     }
@@ -218,32 +172,48 @@ void renderentities()
             if(e.type==CTF_FLAG)
             {
                 defformatstring(path)("pickups/flags/%s", team_basestring(e.attr2));
-                rendermodel(path, ANIM_FLAG|ANIM_LOOP, 0, 0, vec(e.x, e.y, (float)S(e.x, e.y)->floor), (float)((e.attr1+7)-(e.attr1+7)%15), 0, 120.0f);
+                rendermodel(path, ANIM_FLAG|ANIM_LOOP, 0, 0, vec(e.x, e.y, (float)S(e.x, e.y)->floor), 0, float(e.attr1) / ENTSCALE10, 0, 120.0f);
+            }
+            else if((e.type == CLIP || e.type == PLCLIP) && showclips && !stenciling)
+            {
+                renderclip(e.type, e.x, e.y, float(e.attr2) / ENTSCALE5, float(e.attr3) / ENTSCALE5, float(e.attr4) / ENTSCALE5, float(e.attr1) / ENTSCALE10, float(e.attr6) / (4 * ENTSCALE10), e.attr7);
             }
-            else if((e.type == CLIP || e.type == PLCLIP) && showclips && !stenciling) renderclip(e);
             else if(e.type == MAPMODEL && showclips && showmodelclipping && !stenciling)
             {
-                mapmodelinfo &mmi = getmminfo(e.attr2);
-                if(&mmi && mmi.h)
+                mapmodelinfo *mmi = getmminfo(e.attr2);
+                if(mmi && mmi->h)
                 {
-                    entity ce = e;
-                    ce.type = MAPMODEL;
-                    ce.attr1 = mmi.zoff+e.attr3;
-                    ce.attr2 = ce.attr3 = mmi.rad;
-                    ce.attr4 = mmi.h;
-                    renderclip(ce);
+                    renderclip(e.type, e.x, e.y, mmi->rad, mmi->rad, mmi->h, mmi->zoff + float(e.attr3) / ENTSCALE5, 0, 0);
                 }
             }
+            else if(e.type == LADDER && showladderentities && !stenciling)
+            {
+                renderclip(e.type, e.x, e.y, 0, 0, e.attr1, 0, 0, 3);
+            }
+            else if(e.type == PLAYERSTART && showplayerstarts)
+            {
+                vec o(e.x, e.y, 0);
+                if(!OUTBORD(e.x, e.y)) o.z += S(e.x, e.y)->floor;
+                const char *skin;
+                switch(e.attr2)
+                {
+                    case 0: skin = "packages/models/playermodels/CLA/red.jpg"; break;
+                    case 1: skin = "packages/models/playermodels/RVSF/blue.jpg"; break;
+                    case 100: skin = "packages/models/playermodels/ffaspawn.jpg"; break;
+                    default: skin = "packages/models/playermodels/unknownspawn.jpg"; break;
+                }
+                rendermodel("playermodels", ANIM_IDLE, -(int)textureload(skin)->id, 1.5f, o, 0, e.attr1 / ENTSCALE10 + 90, 0);
+            }
         }
-        if(editmode && i==closest && !stenciling)//closest see above
+        if(editmode && i == closest && !stenciling)
         {
             switch(e.type)
             {
                 case PLAYERSTART:
                 {
                     glColor3f(0, 1, 1);
-                    vec dir;
-                    vecfromyawpitch(e.attr1, 0, -1, 0, dir);
+                    vec dir(0, -1, 0);
+                    dir.rotate_around_z(float(e.attr1) / ENTSCALE10 * RAD);
                     renderentarrow(e, dir, 4);
                     glColor3f(1, 1, 1);
                 }
@@ -261,17 +231,17 @@ void renderentities()
                 {
                     if(OUTBORD(f.actor->o.x, f.actor->o.y)) break;
                     defformatstring(path)("pickups/flags/small_%s%s", m_ktf ? "" : team_basestring(i), m_htf ? "_htf" : m_ktf ? "ktf" : "");
-                    rendermodel(path, ANIM_FLAG|ANIM_START|ANIM_DYNALLOC, 0, 0, vec(f.actor->o).add(vec(0, 0, 0.3f+(sinf(lastmillis/100.0f)+1)/10)), lastmillis/2.5f, 0, 120.0f);
+                    rendermodel(path, ANIM_FLAG|ANIM_START|ANIM_DYNALLOC, 0, 0, vec(f.actor->o).add(vec(0, 0, 0.3f+(sinf(lastmillis/100.0f)+1)/10)), 0, lastmillis/2.5f, 0, 120.0f);
                 }
                 break;
             case CTFF_INBASE:
-                if(!numflagspawn[i]) break;
+                if(!clentstats.flags[i]) break;
             case CTFF_DROPPED:
             {
                 if(OUTBORD(f.pos.x, f.pos.y)) break;
                 entity &e = *f.flagent;
                 defformatstring(path)("pickups/flags/%s%s", m_ktf ? "" : team_basestring(i),  m_htf ? "_htf" : m_ktf ? "ktf" : "");
-                if(f.flagent->spawned) rendermodel(path, ANIM_FLAG|ANIM_LOOP, 0, 0, vec(f.pos.x, f.pos.y, f.state==CTFF_INBASE ? (float)S(int(f.pos.x), int(f.pos.y))->floor : f.pos.z), (float)((e.attr1+7)-(e.attr1+7)%15), 0, 120.0f);
+                if(f.flagent->spawned) rendermodel(path, ANIM_FLAG|ANIM_LOOP, 0, 0, vec(f.pos.x, f.pos.y, f.state==CTFF_INBASE ? (float)S(int(f.pos.x), int(f.pos.y))->floor : f.pos.z), 0, float(e.attr1) / ENTSCALE10, 0, 120.0f);
                 break;
             }
             case CTFF_IDLE:
@@ -291,13 +261,13 @@ void pickupeffects(int n, playerent *d)
     if(!d) return;
     d->pickup(e.type);
     if (m_lss && e.type == I_GRENADE) d->pickup(e.type); // get 2
-    itemstat &is = d->itemstats(e.type);
+    itemstat *is = d->itemstats(e.type);
     if(d!=player1 && d->type!=ENT_BOT) return;
-    if(&is)
+    if(is)
     {
         if(d==player1)
         {
-            audiomgr.playsoundc(is.sound);
+            audiomgr.playsoundc(is->sound);
 
             /*
                 onPickup arg1 legend:
@@ -311,7 +281,6 @@ void pickupeffects(int n, playerent *d)
             */
             if(identexists("onPickup"))
             {
-                string o;
                 itemstat *tmp = NULL;
                 switch(e.type)
                 {
@@ -324,14 +293,10 @@ void pickupeffects(int n, playerent *d)
                     case I_ARMOUR:  tmp = &powerupstats[e.type-I_HEALTH]; break;
                     default: break;
                 }
-                if(tmp)
-                {
-                    formatstring(o)("onPickup %d %d", e.type - 3, m_lss && e.type == I_GRENADE ? 2 : tmp->add);
-                    execute(o);
-                }
+                if(tmp) exechook(HOOK_SP, "onPickup", "%d %d", e.type - 3, m_lss && e.type == I_GRENADE ? 2 : tmp->add);
             }
         }
-        else audiomgr.playsound(is.sound, d);
+        else audiomgr.playsound(is->sound, d);
     }
 
     weapon *w = NULL;
@@ -431,14 +396,14 @@ void checkitems(playerent *d)
         if(e.type==CTF_FLAG) continue;
         // simple 2d collision
         vec v(e.x, e.y, S(e.x, e.y)->floor+eyeheight);
-        if(isitem(e.type)) v.z += e.attr1;
+        if(isitem(e.type)) v.z += float(e.attr1) / ENTSCALE10;
         if(d->o.dist(v)<2.5f) trypickup(i, d);
     }
     if(m_flags) loopi(2)
     {
         flaginfo &f = flaginfos[i];
         entity &e = *f.flagent;
-        if(!e.spawned || !f.ack || (f.state == CTFF_INBASE && !numflagspawn[i])) continue;
+        if(!e.spawned || !f.ack || (f.state == CTFF_INBASE && !clentstats.flags[i])) continue;
         if(OUTBORD(f.pos.x, f.pos.y)) continue;
         if(f.state==CTFF_DROPPED) // 3d collision for dropped ctf flags
         {
@@ -453,16 +418,7 @@ void checkitems(playerent *d)
     }
 }
 
-void spawnallitems()            // spawns items locally
-{
-    loopv(ents) if(ents[i].fitsmode(gamemode) || (multiplayer(false) && gamespeed!=100 && (i=-1)))
-    {
-        ents[i].spawned = true;
-        ents[i].lastmillis = lastmillis;
-    }
-}
-
-void resetspawns(int type)
+void resetpickups(int type)
 {
     loopv(ents) if(type < 0 || type == ents[i].type) ents[i].spawned = false;
     if(m_noitemsnade || m_pistol)
@@ -471,7 +427,7 @@ void resetspawns(int type)
     }
 }
 
-void setspawn(int i, bool on)
+void setpickupspawn(int i, bool on)
 {
     if(ents.inrange(i))
     {
@@ -480,29 +436,24 @@ void setspawn(int i, bool on)
     }
 }
 
-bool selectnextprimary(int num)
+SVARFP(nextprimary, guns[GUN_ASSAULT].modelname,
 {
-    switch(num)
+    int n = getlistindex(nextprimary, gunnames, true, -1);
+    switch(n)
     {
-//         case GUN_CPISTOL:
+        default:
+            conoutf("\"%s\" is not a valid primary weapon", nextprimary);
+            n = GUN_ASSAULT;
         case GUN_CARBINE:
         case GUN_SHOTGUN:
         case GUN_SUBGUN:
         case GUN_SNIPER:
         case GUN_ASSAULT:
-            player1->setnextprimary(num);
+            player1->setnextprimary(n);
             addmsg(SV_PRIMARYWEAP, "ri", player1->nextprimweap->type);
-            return true;
-
-        default:
-            conoutf("this is not a valid primary weapon");
-            return false;
+            nextprimary = exchangestr(nextprimary, gunnames[player1->nextprimweap->type]);
+            break;
     }
-}
-
-VARFP(nextprimary, 0, GUN_ASSAULT, NUMGUNS,
-{
-    if(!selectnextprimary(nextprimary)) selectnextprimary((nextprimary = GUN_ASSAULT));
 });
 
 // flag ent actions done by the local player
@@ -531,6 +482,9 @@ void tryflagdrop(bool manual)
         {
             f.flagent->spawned = false;
             f.state = CTFF_DROPPED;
+            f.pos.x = floor(player1->o.x + 0.5f);
+            f.pos.y = floor(player1->o.y + 0.5f);
+            f.pos.z = floor(player1->o.z + 0.5f);
             f.ack = false;
             flagdropmillis = lastmillis+3000;
             addmsg(SV_FLAGACTION, "rii", manual ? FA_DROP : FA_LOST, f.team);
@@ -576,7 +530,8 @@ void flagdropped(int flag, float x, float y, float z)
     p.o.y = y;
     p.o.z = z;
     p.vel.z = -0.8f;
-    p.aboveeye = p.eyeheight = p.maxeyeheight = 0.4f;
+    p.aboveeye = 1.0f;
+    p.eyeheight = p.maxeyeheight = 0.1f;
     p.radius = 0.1f;
 
     bool oldcancollide = false;
@@ -590,12 +545,22 @@ void flagdropped(int flag, float x, float y, float z)
         moveplayer(&p, 10, true, 50);
         if(p.stuck) break;
     }
+    if(p.o.z < waterlevel)
+    {
+        p.aboveeye = 6.7f;
+        loopirev((waterlevel - p.o.z) / 0.2f + 1)
+        {
+            p.o.z += 0.2f;
+            if(p.o.z > z) p.radius = 1.0f; // don't float through small holes in the ceiling
+            if(collide(&p)) break;
+        }
+        if(p.o.z > waterlevel) p.o.z = waterlevel;
+    }
     if(f.actor) f.actor->cancollide = oldcancollide; // restore settings
 
-    f.pos.x = round(p.o.x);
-    f.pos.y = round(p.o.y);
-    f.pos.z = round(p.o.z);
-    if(f.pos.z < hdr.waterlevel) f.pos.z = (short) hdr.waterlevel;
+    f.pos.x = floor(p.o.x + 0.5f);
+    f.pos.y = floor(p.o.y + 0.5f);
+    f.pos.z = floor(p.o.z * 5 + 0.5f) / 5;
     f.flagent->spawned = true;
     f.ack = true;
 }
@@ -615,45 +580,54 @@ void flagidle(int flag)
     flaginfos[flag].flagent->spawned = false;
 }
 
-void entstats(void)
+void entstats_(void)
 {
-    int entcnt[MAXENTTYPES] = {0}, clipents = 0, spawncnt[5] = {0};
+    entitystats_s es;
+    calcentitystats(es, NULL, 0);
+    int clipents = 0, xmodels = 0, xsounds = 0;
     loopv(ents)
     {
         entity &e = ents[i];
-        if(e.type >= MAXENTTYPES) continue;
-        entcnt[e.type]++;
-        switch(e.type)
+        if(e.type == MAPMODEL)
         {
-            case MAPMODEL:
-            {
-                mapmodelinfo &mmi = getmminfo(e.attr2);
-                if(&mmi && mmi.h) clipents++;
-                break;
-            }
-            case PLAYERSTART:
-                if(e.attr2 < 2) spawncnt[e.attr2]++;
-                if(e.attr2 == 100) spawncnt[2]++;
-                break;
-            case CTF_FLAG:
-                if(e.attr2 < 2) spawncnt[e.attr2 + 3]++;
-                break;
+            mapmodelinfo *mmi = getmminfo(e.attr2);
+            if(!mmi) xmodels++;
+            if(mmi && mmi->h) clipents++;
+        }
+        else if(e.type == SOUND)
+        {
+            if(!mapconfigdata.mapsoundlines.inrange(e.attr1)) xsounds++;
         }
     }
-    loopi(MAXENTTYPES)
+    string txt = "", clips = "";
+    loopi(MAXENTTYPES) if(es.entcnt[i]) switch(i)
     {
-        if(entcnt[i]) switch(i)
-        {
-            case MAPMODEL:      conoutf(" %d %s, %d clipped", entcnt[i], entnames[i], clipents); break;
-            case PLAYERSTART:   conoutf(" %d %s, %d CLA, %d RVSF, %d FFA", entcnt[i], entnames[i], spawncnt[0], spawncnt[1], spawncnt[2]); break;
-            case CTF_FLAG:      conoutf(" %d %s, %d CLA, %d RVSF", entcnt[i], entnames[i], spawncnt[3], spawncnt[4]); break;
-            default:            conoutf(" %d %s", entcnt[i], entnames[i]); break;
-        }
+        case MAPMODEL:      conoutf(" %d %s, %d clipped, %s%d unconfigured", es.entcnt[i], entnames[i], clipents, xmodels ? "\f3" : "", xmodels); break;
+        case SOUND:         conoutf(" %d %s, %s%d unconfigured", es.entcnt[i], entnames[i], xsounds ? "\f3" : "", xsounds); break;
+        case PLAYERSTART:   conoutf(" %d %s, %d CLA, %d RVSF, %d FFA%c \f3unknown %d", es.entcnt[i], entnames[i], es.spawns[0], es.spawns[1], es.spawns[2], es.unknownspawns ? ',' : '\0', es.unknownspawns); break;
+        case CTF_FLAG:      conoutf(" %d %s, %d CLA, %d RVSF%c \f3unknown %d", es.entcnt[i], entnames[i], es.flags[0], es.flags[1], es.unknownflags ? ',' : '\0', es.unknownflags); break;
+        case CLIP:
+        case PLCLIP:        concatformatstring(clips, ", %d %s", es.entcnt[i], entnames[i]); break;
+        case NOTUSED:       conoutf(" %d deleted", es.entcnt[i]); break;
+        default:            if(isitem(i)) concatformatstring(txt, ", %d %s", es.entcnt[i], entnames[i]);
+                            else conoutf(" %d %s", es.entcnt[i], entnames[i]);
+                            break;
     }
+    if(*clips) conoutf(" %s", clips + 2);
+    if(es.pickups)
+    {
+        conoutf(" %d pickups:%s", es.pickups, txt + 1);
+        *txt = '\0';
+        loopi(LARGEST_FACTOR + 1) concatformatstring(txt, " %d", es.pickupdistance[i]);
+        conoutf(" pickupdistance:%s", txt);
+    }
+    if(es.entcnt[CTF_FLAG]) conoutf(" flag distance: %d", es.flagentdistance);
+    conoutf(" map capabilities: has ffa spawns %d, has team spawns %d, has flags %d", es.hasffaspawns ? 1 : 0, es.hasteamspawns ? 1 : 0, es.hasflags ? 1 : 0);
     conoutf("total entities: %d", ents.length());
+    intret(xmodels + xsounds);
 }
 
-COMMAND(entstats, "");
+COMMANDN(entstats, entstats_, "");
 
 vector<int> changedents;
 int lastentsync = 0;
@@ -664,8 +638,38 @@ void syncentchanges(bool force)
     loopv(changedents) if(ents.inrange(changedents[i]))
     {
         entity &e = ents[changedents[i]];
-        addmsg(SV_EDITENT, "ri9", changedents[i], e.type, e.x, e.y, e.z, e.attr1, e.attr2, e.attr3, e.attr4);
+        addmsg(SV_EDITENT, "ri9i3", changedents[i], e.type, e.x, e.y, e.z, e.attr1, e.attr2, e.attr3, e.attr4, e.attr5, e.attr6, e.attr7);
     }
     changedents.setsize(0);
     lastentsync = lastmillis;
 }
+
+void clampentityattributes(persistent_entity &e)
+{
+    if(e.type < MAXENTTYPES)
+    {
+        int c;
+        #define CLAMPATTR(x) \
+            c = entwraparound[e.type][x - 1]; \
+            if(c > 0) e.attr##x = (e.attr##x % c + c) % c;  /* fold value into range 0..c */ \
+            else if(c < 0) e.attr##x = e.attr##x % (-c)     /* fold value into range -c..c */
+        CLAMPATTR(1);
+        CLAMPATTR(2);
+        CLAMPATTR(3);
+        CLAMPATTR(4);
+        CLAMPATTR(5);
+        CLAMPATTR(6);
+        CLAMPATTR(7);
+    }
+}
+
+const char *formatentityattributes(const persistent_entity &e, bool withcomma)
+{
+    static string res;
+    int t = e.type < MAXENTTYPES ? e.type : 0;
+    const char *c = withcomma ? "," : "";
+    #define AA(x) floatstr(float(e.attr##x) / entscale[t][x - 1], true)
+    formatstring(res)("%s%s %s%s %s%s %s%s %s%s %s%s %s", AA(1), c, AA(2), c, AA(3), c, AA(4), c, AA(5), c, AA(6), c, AA(7));
+    #undef AA
+    return res;
+}
