@@ -91,54 +91,102 @@ bool plcollide(physent *d, physent *o, float &headspace, float &hi, float &lo)
     return false;
 }
 
-bool cornertest(int mip, int x, int y, int dx, int dy, int &bx, int &by, int &bs)    // recursively collide with a mipmapped corner cube
+int cornertest(int x, int y, int &bx, int &by, int &bs, sqr *&s, sqr *&h)    // iteratively collide with a mipmapped corner cube
 {
-    sqr *w = wmip[mip];
+    int mip = 1, res = -1;
+    while(SWS(wmip[mip], x>>mip, y>>mip, sfactor-mip)->type==CORNER) mip++;
+    mip--;
+    x >>= mip;
+    y >>= mip;
     int mfactor = sfactor - mip;
-    bool stest = SOLID(SWS(w, x+dx, y, mfactor)) && SOLID(SWS(w, x, y+dy, mfactor));
-    mip++;
-    x /= 2;
-    y /= 2;
-    if(SWS(wmip[mip], x, y, mfactor-1)->type==CORNER)
+    bx = x<<mip;                     // bx, by and bs are the real-world coordinates and size of the corner mip
+    by = y<<mip;                     // s is the corner mip and h is the companion mip of a corner between non-solids (to get floor and ceil from)
+    bs = 1<<mip;
+    sqr *z = SWS(wmip[mip],x - 1, y,mfactor);
+    sqr *t = SWS(z,2,0,mfactor);     //   w
+    sqr *w = SWS(z,1,-1,mfactor);    //  zst
+    sqr *v = SWS(z,1,1,mfactor);     //   v
+    s = SWS(z,1,0,mfactor);
+
+    // now, this is _exactly_ how the renderer interprets map geometry...
+    if(SOLID(z))
     {
-        bx = x<<mip;
-        by = y<<mip;
-        bs = 1<<mip;
-        return cornertest(mip, x, y, dx, dy, bx, by, bs);
+        if(SOLID(w)) res = 2; // corners between solids are solid behind the wall or SPACE in front of it
+        else if(SOLID(v)) res = 3;
     }
-    return stest;
+    else if(SOLID(t))
+    {
+        if(SOLID(w)) res = 1;
+        else if(SOLID(v)) res = 0;
+    }
+    else
+    { // not a corner between solids
+        bool wv = w->ceil-w->floor < v->ceil-v->floor;
+        h = wv ? v : w;  // in front of the corner, use floor and ceil from h
+        if(z->ceil-z->floor < t->ceil-t->floor) res = wv ? 2 : 3;
+        else res = wv ? 1 : 0;
+    }            //  03
+    return res;  //  12
 }
 
+static int cornersurface = 0;
+
 bool mmcollide(physent *d, float &hi, float &lo)           // collide with a mapmodel
 {
-    const float eyeheight = d->eyeheight;
+    const float eyeheight = d->eyeheight, SQRT2HALF = SQRT2 / 2.0f;
     const float playerheight = eyeheight + d->aboveeye;
-    loopv(ents)
+    if(editmode) clentstats.firstclip = 0;
+    for(int i = clentstats.firstclip; i < ents.length(); i++)
     {
         entity &e = ents[i];
         // if(e.type==CLIP || (e.type == PLCLIP && d->type == ENT_PLAYER))
         if (e.type==CLIP || (e.type == PLCLIP && (d->type == ENT_BOT || d->type == ENT_PLAYER || (d->type == ENT_BOUNCE && ((bounceent *)d)->plclipped)))) // don't allow bots to hack themselves into plclips - Bukz 2011/04/14
         {
-            if(fabs(e.x-d->o.x) < e.attr2 + d->radius && fabs(e.y-d->o.y) < e.attr3 + d->radius)
+            bool hitarea = false;
+            switch(e.attr7 & 3)
             {
-                const float cz = float(S(e.x, e.y)->floor+e.attr1), ch = float(e.attr4);
-                const float dz = d->o.z-d->eyeheight;
-                if(dz < cz - 0.001) { if(cz<hi) hi = cz; }
+                default: // classic unrotated clip, possibly tilted
+                    hitarea = fabs(e.x - d->o.x) < float(e.attr2) / ENTSCALE5 + d->radius && fabs(e.y - d->o.y) < float(e.attr3) / ENTSCALE5 + d->radius;
+                    break;
+                case 3: // clip rotated 45°
+                {
+                    float rx = (e.x - d->o.x) * SQRT2HALF, ry = (e.y - d->o.y) * SQRT2HALF, rr = d->radius * SQRT2; // rotate player instead of clip (adjust player radius to compensate)
+                    float a1 = fabs(rx - ry) - float(e.attr3) / ENTSCALE5 - rr, a2 = fabs(rx + ry) - float(e.attr2) / ENTSCALE5 - rr;
+                    if(a1 < 0 && a2 < 0)
+                    {
+                        float a3 =  a1 + a2 + rr;
+                        if(a3 < 0) hitarea = true;
+                        if(a3 < -1e-2 && (a3 < -rr || a1 * a2 < 0.42f)) cornersurface = a1 > a2 ? 1 : 2;
+                    }
+                    break;
+                }
+            }
+            if(hitarea)
+            {
+                float cz = float(S(e.x, e.y)->floor + float(e.attr1) / ENTSCALE10), ch = float(e.attr4) / ENTSCALE5;
+                if(e.attr6) switch(e.attr7 & 3)
+                { // incredibly ugly solution - but it only applies on the one tilted clip we stand on
+                    case 1: cz += (floor(0.5f + clamp(d->o.x - e.x + d->radius * (e.attr6 > 0 ? 1 : -1), -float(e.attr2) / ENTSCALE5, float(e.attr2) / ENTSCALE5))) * float(e.attr6) / (4 * ENTSCALE10); break; // tilt x
+                    case 2: cz += (floor(0.5f + clamp(d->o.y - e.y + d->radius * (e.attr6 > 0 ? 1 : -1), -float(e.attr3) / ENTSCALE5, float(e.attr3) / ENTSCALE5))) * float(e.attr6) / (4 * ENTSCALE10); break; // tilt y
+                }
+                const float dz = d->o.z - d->eyeheight;
+                if(dz < cz - 0.42) { if(cz<hi) hi = cz; }
                 else if(cz+ch>lo) lo = cz+ch;
                 if(hi-lo < playerheight) return true;
+                if(dz + (d->type != ENT_BOUNCE ? 1.26 : 0) > cz + ch || dz + playerheight < cz) cornersurface = 0;
             }
         }
         else if(e.type==MAPMODEL)
         {
-            mapmodelinfo &mmi = getmminfo(e.attr2);
-            if(!&mmi || !mmi.h) continue;
-            const float r = mmi.rad+d->radius;
+            mapmodelinfo *mmi = getmminfo(e.attr2);
+            if(!mmi || !mmi->h) continue;
+            const float r = mmi->rad + d->radius;
             if(fabs(e.x-d->o.x)<r && fabs(e.y-d->o.y)<r)
             {
-                const float mmz = float(S(e.x, e.y)->floor+mmi.zoff+e.attr3);
+                const float mmz = float(S(e.x, e.y)->floor + mmi->zoff + float(e.attr3) / ENTSCALE5);
                 const float dz = d->o.z-eyeheight;
-                if(dz<mmz) { if(mmz<hi) hi = mmz; }
-                else if(mmz+mmi.h>lo) lo = mmz+mmi.h;
+                if(dz < mmz - 0.42) { if(mmz < hi) hi = mmz; }
+                else if(mmz + mmi->h > lo) lo = mmz + mmi->h;
                 if(hi-lo < playerheight) return true;
             }
         }
@@ -162,9 +210,8 @@ bool objcollide(physent *d, const vec &objpos, float objrad, float objheight) //
 // all collision happens here
 // spawn is a dirty side effect used in spawning
 // drop & rise are supplied by the physics below to indicate gravity/push for current mini-timestep
-static int cornersurface = 0;
 
-bool collide(physent *d, bool spawn, float drop, float rise, int level) // levels 1 = map, 2 = players, 4 = models, default: 1+2+4
+bool collide(physent *d, bool spawn, float drop, float rise)
 {
     cornersurface = 0;
     const float fx1 = d->o.x-d->radius;     // figure out integer cube rectangle this entity covers in map
@@ -178,11 +225,15 @@ bool collide(physent *d, bool spawn, float drop, float rise, int level) // level
     float hi = 127, lo = -128;
     const float eyeheight = d->eyeheight;
     const float playerheight = eyeheight + d->aboveeye;
+    float z1 = d->o.z-eyeheight, z2 = z1 + playerheight;
+    if(d->type != ENT_BOUNCE) z1 += 1.26;
+    const int applyclip = d->type == ENT_BOT || d->type == ENT_PLAYER || (d->type == ENT_BOUNCE && ((bounceent *)d)->plclipped) ? TAGANYCLIP : TAGCLIP;
 
-    if(level&1) for(int y = y1; y<=y2; y++) for(int x = x1; x<=x2; x++)     // collide with map
+    for(int y = y1; y<=y2; y++) for(int x = x1; x<=x2; x++)     // collide with map
     {
         if(OUTBORD(x,y)) return true;
         sqr *s = S(x,y);
+        bool tagclipped = (s->tag & applyclip) != 0;
         float ceil = s->ceil;
         float floor = s->floor;
         switch(s->type)
@@ -192,13 +243,43 @@ bool collide(physent *d, bool spawn, float drop, float rise, int level) // level
 
             case CORNER:
             {
-                int bx = x, by = y, bs = 1;
-                cornersurface = 1;
-                if((x==x1 && y==y2 && cornertest(0, x, y, -1,  1, bx, by, bs) && fx1-bx<=fy2-by)
-                || (x==x2 && y==y1 && cornertest(0, x, y,  1, -1, bx, by, bs) && fx2-bx>=fy1-by) || !(++cornersurface)
-                || (x==x1 && y==y1 && cornertest(0, x, y, -1, -1, bx, by, bs) && fx1-bx+fy1-by<=bs)
-                || (x==x2 && y==y2 && cornertest(0, x, y,  1,  1, bx, by, bs) && fx2-bx+fy2-by>=bs))
-                    return true;
+                sqr *ns, *h = NULL;
+                int bx, by, bs;
+                int q = cornertest(x, y, bx, by, bs, ns, h);
+                bool matter = false, match = false;
+                switch(q)                          //  0XX3
+                {                                  //  XXXX
+                    case 0:                        //  1XX2
+                        match = x==x2 && y==y2;
+                        matter = fx2-bx+fy2-by>=bs;
+                        break;
+                    case 1:
+                        match = x==x2 && y==y1;
+                        matter = fx2-bx>=fy1-by;
+                        break;
+                    case 2:
+                        match = x==x1 && y==y1;
+                        matter = fx1-bx+fy1-by<=bs;
+                        break;
+                    case 3:
+                        match = x==x1 && y==y2;
+                        matter = fx1-bx<=fy2-by;
+                        break;
+                    default:
+                        return true; // mapper's fault: corner with unsufficient solids: renderer can't handle those anyway: treat as solid
+                }
+                cornersurface = (q & 1) ? 5 : 6;
+                sqr *n = h && !matter ? h : ns;
+                ceil = n->ceil;  // use floor & ceil from higher mips (like the renderer)
+                floor = n->floor;
+                if(!match && d->type == ENT_BOUNCE) match = q == (d->vel.x < 0) * 2 + (d->vel.y * d->vel.x < 0); // when coming towards corner surface: prefer corner bounce
+                if(match && matter)
+                {
+                    if(!h) return true; // we hit a corner between solids...
+                    else if(z1 < ns->floor || z2 > ns->ceil || tagclipped) return true; // corner is not between solids, but we hit it...
+                }
+                if(h) tagclipped = false; // for corners between non-solids, tagclips extend the corner (visualisation does not reflect that, yet)
+                                          // tagclipped corners between solids are fully clipped
                 cornersurface = 0;
                 break;
             }
@@ -211,15 +292,16 @@ bool collide(physent *d, bool spawn, float drop, float rise, int level) // level
                 ceil += (s->vdelta+S(x+1,y)->vdelta+S(x,y+1)->vdelta+S(x+1,y+1)->vdelta)/16.0f;
 
         }
+        if(tagclipped) return true; // tagged clips feel like solids
         if(ceil<hi) hi = ceil;
         if(floor>lo) lo = floor;
     }
 
-    if(level&1 && hi-lo < playerheight) return true;
+    if(hi - lo < playerheight) return true;
 
     float headspace = 10.0f;
 
-    if( level&2 && d->type!=ENT_CAMERA)
+    if(d->type!=ENT_CAMERA)
     {
         loopv(players)       // collide with other players
         {
@@ -231,7 +313,7 @@ bool collide(physent *d, bool spawn, float drop, float rise, int level) // level
     }
 
     headspace -= 0.01f;
-    if( level&4 && mmcollide(d, hi, lo)) return true;    // collide with map models
+    if(mmcollide(d, hi, lo)) return true;    // collide with map models
 
     if(spawn)
     {
@@ -243,7 +325,7 @@ bool collide(physent *d, bool spawn, float drop, float rise, int level) // level
         const float spacelo = d->o.z-eyeheight-lo;
         if(spacelo<0)
         {
-            if(spacelo>-0.01)
+            if(spacelo>-0.01 || (spacelo>-0.1 && d==player1 && player1->spectatemode==SM_FLY))
             {
                 d->o.z = lo+eyeheight;   // stick on step
             }
@@ -263,20 +345,22 @@ bool collide(physent *d, bool spawn, float drop, float rise, int level) // level
             d->vel.z = 0;                     // cancel out jumping velocity
         }
 
-        const float floorclamp = d->crouching ? 0.1f : 0.01f;
+        const float floorclamp = d->crouching && (!d->lastjump || lastmillis - d->lastjump > 250) ? 0.1f : 0.01f;
         d->onfloor = d->o.z-eyeheight-lo < floorclamp;
     }
     return false;
 }
 
-VARP(maxroll, 0, 0, 20); // note: when changing max value, fix network transmission
+VARFP(maxroll, 0, ROLLMOVDEF, ROLLMOVMAX, player1->maxroll = maxroll);
+VARFP(maxrolleffect, 0, ROLLEFFDEF, ROLLEFFMAX, player1->maxrolleffect = maxrolleffect);
+VARP(maxrollremote, 0, ROLLMOVDEF + ROLLEFFDEF, ROLLMOVMAX + ROLLEFFMAX);
 //VAR(recoilbackfade, 0, 100, 1000);
 
 void resizephysent(physent *pl, int moveres, int curtime, float min, float max)
 {
     if(pl->eyeheightvel==0.0f) return;
 
-    const bool water = hdr.waterlevel>pl->o.z;
+    const bool water = waterlevel > pl->o.z;
     const float speed = curtime*pl->maxspeed/(water ? 2000.0f : 1000.0f);
     float h = pl->eyeheightvel * speed / moveres;
 
@@ -284,7 +368,7 @@ void resizephysent(physent *pl, int moveres, int curtime, float min, float max)
     {
         pl->eyeheight += h;
         pl->o.z += h;
-        if(collide(pl))
+        if(collide(pl) && !(pl != player1 && player1->isspectating() && player1->spectatemode==SM_FOLLOW1ST)) // don't check collision during spectating in 1st person view
         {
             pl->eyeheight -= h; // collided, revert mini-step
             pl->o.z -= h;
@@ -311,18 +395,6 @@ void resizephysent(physent *pl, int moveres, int curtime, float min, float max)
 // moveres indicated the physics precision (which is lower for monsters and multiplayer prediction)
 // local is false for multiplayer prediction
 
-void clamproll(physent *pl)
-{
-    extern int maxrollremote;
-    int mroll = pl == player1 ? maxroll : maxrollremote;
-    if(pl->roll > mroll) pl->roll = mroll;
-    else if(pl->roll < -mroll) pl->roll = -mroll;
-}
-
-float var_f = 0;
-int var_i = 0;
-bool var_b = true;
-
 FVARP(flyspeed, 1.0, 2.0, 5.0);
 
 void moveplayer(physent *pl, int moveres, bool local, int curtime)
@@ -339,7 +411,7 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
     if(pl->type==ENT_BOUNCE)
     {
         bounceent* bounce = (bounceent *) pl;
-        water = hdr.waterlevel>pl->o.z;
+        water = waterlevel > pl->o.z;
 
         const float speed = curtime*pl->maxspeed/(water ? 2000.0f : 1000.0f);
         const float friction = water ? 20.0f : (pl->onfloor || isfly ? 6.0f : 30.0f);
@@ -348,7 +420,7 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
         if(pl->onfloor) // apply friction
         {
             pl->vel.mul(fpsfric-1);
-        pl->vel.div(fpsfric);
+            pl->vel.div(fpsfric);
         }
         else // apply gravity
         {
@@ -371,12 +443,11 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
     {
         const int timeinair = pl->timeinair;
         int move = pl->onladder && !pl->onfloor && pl->move == -1 ? 0 : pl->move; // movement on ladder
-        water = hdr.waterlevel>pl->o.z-0.5f;
+        if(!editfly) water = waterlevel > pl->o.z - 0.5f;
 
-        float chspeed = 0.4f;
-        if(!(pl->onfloor || pl->onladder)) chspeed = 1.0f;
+        float chspeed = (pl->onfloor || pl->onladder || !pl->crouchedinair) ? 0.4f : 1.0f;
 
-        const bool crouching = pl->crouching || pl->eyeheight < pl->maxeyeheight;
+        const bool crouching = pl->crouching || (pl->eyeheight < pl->maxeyeheight && pl->eyeheight > 1.1f);
         const float speed = curtime/(water ? 2000.0f : 1000.0f)*pl->maxspeed*(crouching && pl->state != CS_EDITING ? chspeed : 1.0f)*(pl==player1 && isfly ? flyspeed : 1.0f);
         const float friction = water ? 20.0f : (pl->onfloor || isfly ? 6.0f : (pl->onladder ? 1.5f : 30.0f));
         const float fpsfric = max(friction/curtime*20.0f, 1.0f);
@@ -395,6 +466,8 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
         d.x += (float)(pl->strafe*cosf(RAD*(pl->yaw-180)));
         d.y += (float)(pl->strafe*sinf(RAD*(pl->yaw-180)));
 
+        float curfullspeed = d.magnitudexy();
+
         pl->vel.mul(fpsfric-1.0f);   // slowly apply friction and direction to velocity, gives a smooth movement
         pl->vel.add(d);
         pl->vel.div(fpsfric);
@@ -406,8 +479,7 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
             pl->o.add(d);
             if(pl->jumpnext && !pl->trycrouch)
             {
-                pl->jumpnext = true; // fly directly upwards while holding jump keybinds
-                pl->vel.z = 0.5f;
+                pl->vel.z = 0.5f; // fly directly upwards while holding jump keybinds
             }
             else if (pl->trycrouch && !pl->jumpnext)
             {
@@ -432,10 +504,10 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
                     const float climbspeed = 1.0f;
 
                     if(pl->type==ENT_BOT && pl->state == CS_ALIVE) pl->vel.z = climbspeed; // bots climb upwards only
-                    else if(pl->type==ENT_PLAYER)
+                    else if(pl->type==ENT_PLAYER && pl->state == CS_ALIVE)
                     {
-                        if(((playerent *)pl)->k_up) pl->vel.z = climbspeed;
-                        else if(((playerent *)pl)->k_down) pl->vel.z = -climbspeed;
+                        if(((playerent *)pl)->k_up && pl->move > 0) pl->vel.z = climbspeed;
+                        else if(((playerent *)pl)->k_down && pl->move < 0) pl->vel.z = -climbspeed;
                     }
                     pl->timeinair = 0;
                 }
@@ -446,12 +518,12 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
                         if(pl->jumpnext)
                         {
                             pl->jumpnext = false;
-                            bool doublejump = pl->lastjump && lastmillis-pl->lastjump < 250 && pl->strafe != 0 && pl->lastjumpheight != 0 && pl->lastjumpheight != pl->o.z;
-                            pl->lastjumpheight = pl->o.z;
+                            bool doublejump = pl->lastjump && lastmillis - pl->lastjump < 250 && pl->strafe != 0 && pl->o.z - pl->eyeheight - pl->lastjumpheight > 0.2f;
+                            pl->lastjumpheight = pl->o.z - pl->eyeheight;
                             pl->vel.z = 2.0f; // physics impulse upwards
-                            if(doublejump) // more velocity on double jump
+                            if(doublejump && curfullspeed > 0.1f) // more velocity on double jump
                             {
-                                pl->vel.mul(1.25f);
+                                pl->vel.mul(1.25f / max(pl->vel.magnitudexy() / curfullspeed, 1.0f));
                             }
                             if(water) // dampen velocity change even harder, gives correct water feel
                             {
@@ -474,7 +546,7 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
                     }
                 }
 
-                if(timeinair > 200 && !pl->timeinair)
+                if(timeinair > 200 && !pl->timeinair && pl->inwater == water)
                 {
                     int sound = timeinair > 800 ? S_HARDLAND : S_SOFTLAND;
                     if(pl->state!=CS_DEAD)
@@ -503,16 +575,50 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
         const float f = 1.0f/moveres;
 
         // try move forward
+        vec o_null = pl->o;
         pl->o.x += f*d.x;
         pl->o.y += f*d.y;
         pl->o.z += f*d.z;
+        volatile vec gcco3(pl->o.x, pl->o.y, pl->o.z);  // force capping the o-values to float-representables (avoid player stuck conditions on 32-bit g++ builds)
+        pl->o = vec(gcco3.x, gcco3.y, gcco3.z);
         hitplayer = NULL;
-        if(!collide(pl, false, drop, rise)) continue;
-        else collided = true;
-        if(pl->type==ENT_BOUNCE && cornersurface)
+        bool spect3rd = player1->spectatemode > SM_FOLLOW1ST && player1->spectatemode <= SM_FOLLOW3RD_TRANSPARENT;
+        if((pl->type == ENT_CAMERA && pl == camera1 && spect3rd) || !collide(pl, false, drop, rise)) continue;
+        int cornersurface1 = cornersurface;
+        vec o_trying = pl->o;  // o_trying = o_null + f * d  (= one microstep in desired direction)
+        if(!cornersurface1)
+        { // brute-force check, if it is a corner hit after all
+            if(pl->type != ENT_BOUNCE)
+            {
+                int collx = 0, colly = 0;
+                pl->o.x = o_null.x;
+                if(collide(pl, false, drop, rise)) collx = cornersurface;
+                pl->o = o_trying;
+                pl->o.y = o_null.y;
+                if(collide(pl, false, drop, rise)) colly = cornersurface;
+                cornersurface1 = collx | colly;
+                if((cornersurface1 & 3) == 3) cornersurface1 = 0;
+            }
+            else
+            { // try really, really hard to detect corners for bounces (to compensate for the not-so-micro bounceent-microsteps)
+                vec vd(d.x, d.y, 0);
+                static const float a = 3.0f*PI2/10.0f, ca = cosf(a), sa = sinf(a); // 10 probe points, stretched over three turns
+                loopj(10)
+                {
+                    if(j) vd = vec(ca * vd.x - sa * vd.y, ca * vd.y + sa *vd.x, 0);
+                    pl->o = o_trying;
+                    pl->o.add(vd);
+                    if(collide(pl, false, drop, rise) && cornersurface) break;
+                }
+                cornersurface1 = cornersurface;
+            }
+            pl->o = o_trying;
+        }
+        collided = true;
+        if(pl->type==ENT_BOUNCE && cornersurface1)
         { // try corner bounce
-            float ct2f = cornersurface == 2 ? -1.0 : 1.0;
-            vec oo = pl->o, xd = d;
+            float ct2f = cornersurface1 & 2 ? -1.0 : 1.0;
+            vec xd = d;
             xd.x = d.y * ct2f;
             xd.y = d.x * ct2f;
             pl->o.x += f * (-d.x + xd.x);
@@ -526,19 +632,17 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
                 pl->vel.mul(0.7f);
                 continue;
             }
-            pl->o = oo;
+            pl->o = o_trying;
         }
         if(pl->type==ENT_CAMERA || (pl->type==ENT_PLAYER && pl->state==CS_DEAD && ((playerent *)pl)->spectatemode != SM_FLY))
         {
-            pl->o.x -= f*d.x;
-            pl->o.y -= f*d.y;
-            pl->o.z -= f*d.z;
+            pl->o = o_null;
             break;
         }
         if(pl->type!=ENT_BOUNCE && hitplayer)
         {
             vec dr(hitplayer->o.x-pl->o.x,hitplayer->o.y-pl->o.y,0);
-            float invdist = ufInvSqrt(dr.sqrxy()),
+            float invdist = 1.0f / dr.magnitudexy(),
                   push = (invdist < 10.0f ? dr.dotxy(d)*1.1f*invdist : dr.dotxy(d) * 11.0f);
 
             pl->o.x -= f*d.x*push;
@@ -548,49 +652,54 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
             pl->o.x += f*d.x*push;
             pl->o.y += f*d.y*push;
         }
-        if (cornersurface)
-        {
-            float ct2f = (cornersurface == 2 ? -1.0 : 1.0);
-            float diag = f*d.magnitudexy()*2;
-            vec vd = vec((d.y*ct2f+d.x >= 0.0f ? diag : -diag), (d.x*ct2f+d.y >= 0.0f ? diag : -diag), 0);
-            pl->o.x -= f*d.x;
-            pl->o.y -= f*d.y;
 
-            pl->o.x += vd.x;
-            pl->o.y += vd.y;
+        // the desired direction didn't work
+        pl->o.x = o_null.x;
+        pl->o.y = o_null.y;
+        vec o_nullxy = pl->o; // x and y from o_null but z from o_trying (possibly altered by collide() as well)
+
+        // try sliding
+        if(cornersurface1)
+        {
+            // along a corner wall
+            float ct2f = cornersurface1 & 2 ? -1.0 : 1.0;
+            float diag = fabs(d.x + ct2f * d.y) * 0.5f;
+            vec vd = vec((d.y*ct2f+d.x >= 0.0f ? diag : -diag), (d.x*ct2f+d.y >= 0.0f ? diag : -diag), 0);
+            float ff = f / (cornersurface1 & 4 ? 42.0f : 333.0f);
+            pl->o.x += f*vd.x - ff*d.x;
+            pl->o.y += f*vd.y - ff*d.y;
             if(!collide(pl, false, drop, rise))
             {
                 d.x = vd.x; d.y = vd.y;
                 continue;
             }
-            pl->o.x -= vd.x;
-            pl->o.y -= vd.y;
+            pl->o = o_nullxy;
         }
-        else
+        // try slide along y axis
+        pl->o.y = o_trying.y;
+        if(!collide(pl, false, drop, rise))
         {
-#define WALKALONGAXIS(x,y) \
-            pl->o.x -= f*d.x; \
-            if(!collide(pl, false, drop, rise)) \
-            { \
-                d.x = 0; \
-                if(pl->type==ENT_BOUNCE) { pl->vel.x = -pl->vel.x; pl->vel.mul(0.7f); } \
-                continue; \
-            } \
-            pl->o.x += f*d.x;
-            // player stuck, try slide along y axis
-            WALKALONGAXIS(x,y);
-            // still stuck, try x axis
-            WALKALONGAXIS(y,x);
+            d.x = 0;
+            if(pl->type==ENT_BOUNCE) { pl->vel.x = -pl->vel.x; pl->vel.mul(0.7f); }
+            continue;
         }
-//         try just dropping down
-        pl->o.x -= f*d.x;
-        pl->o.y -= f*d.y;
+        pl->o = o_nullxy;
+        // try x axis
+        pl->o.x = o_trying.x;
+        if(!collide(pl, false, drop, rise))
+        {
+            d.y = 0;
+            if(pl->type==ENT_BOUNCE) { pl->vel.y = -pl->vel.y; pl->vel.mul(0.7f); }
+            continue;
+        }
+        pl->o = o_nullxy;
+        // try just dropping down
         if(!collide(pl, false, drop, rise))
         {
             d.y = d.x = 0;
             continue;
         }
-        pl->o.z -= f*d.z;
+        pl->o = o_null;
         if(pl->type==ENT_BOUNCE) { pl->vel.z = -pl->vel.z; pl->vel.mul(0.5f); }
         break;
     }
@@ -601,19 +710,25 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
 
     if(pl->type==ENT_CAMERA) return;
 
-    if(pl->type!=ENT_BOUNCE && pl==player1)
+    if(pl->type!=ENT_BOUNCE)
     {
-        // automatically apply smooth roll when strafing
-        if(pl->strafe==0)
+        if(pl->type == ENT_PLAYER)
         {
-            pl->roll = pl->roll/(1+(float)sqrt((float)curtime)/25);
+            // automatically apply smooth roll when strafing
+            playerent *p = (playerent *)pl;
+            float iir = 1.0f + sqrtf((float)curtime) / 25.0f;
+            if(pl->strafe==0)
+            {
+                p->movroll /= iir;
+            }
+            else
+            {
+                p->movroll = clamp(p->movroll + pl->strafe * curtime / -30.0f, -p->maxroll, p->maxroll);
+            }
+            p->effroll /= iir; // fade damage roll
+            pl->roll = p->movroll + p->effroll;
+            if(pl != player1) pl->roll = clamp(pl->roll, (float)-maxrollremote, (float)maxrollremote);
         }
-        else
-        {
-            pl->roll += pl->strafe*curtime/-30.0f;
-            clamproll(pl);
-        }
-
         // smooth pitch
         const float fric = 6.0f/curtime*20.0f;
         pl->pitch += pl->pitchvel*(curtime/1000.0f)*pl->maxspeed*(pl->crouching ? 0.75f : 1.0f);
@@ -635,12 +750,16 @@ void moveplayer(physent *pl, int moveres, bool local, int curtime)
         {
             if(!pl->lastsplash || lastmillis-pl->lastsplash>500)
             {
-                audiomgr.playsound(S_SPLASH2, pl);
+                audiomgr.playsound(S_SPLASH2, &pl->o);
                 pl->lastsplash = lastmillis;
             }
             if(pl==player1) pl->vel.z = 0;
         }
-        else if(pl->inwater && !water) audiomgr.playsound(S_SPLASH1, &pl->o);
+        else if(pl->inwater && !water)
+        {
+            audiomgr.playsound(S_SPLASH1, &pl->o);
+            if(pl->type == ENT_BOUNCE) pl->maxspeed /= 8; // prevent nades from jumping out of water
+        }
         pl->inwater = water;
     }
 
@@ -674,6 +793,7 @@ void physicsframe()          // optimally schedule physics frames inside the gra
         physframetime = clamp((PHYSFRAMETIME*gamespeed)/100, 1, PHYSFRAMETIME);
         physsteps = (diff + physframetime - 1)/physframetime;
         lastphysframe += physsteps * physframetime;
+        if(!multiplayer(NULL) && physsteps > 1000) physsteps = 1000;
     }
 }
 
@@ -744,13 +864,13 @@ void attack(bool on)
 
 void jumpn(bool on)
 {
-    if(intermission) return;
+    static bool wason = false;
+    player1->jumpnext = on && !wason && !player1->crouching && !intermission && !player1->isspectating();
+    wason = on;
     if(player1->isspectating())
     {
-        if(lastmillis - player1->respawnoffset > 1000 && on) togglespect();
+        if(lastmillis - player1->lastdeath > 1000 && on) togglespect();
     }
-    else if(player1->crouching) return;
-    else player1->jumpnext = on;
 }
 
 void updatecrouch(playerent *p, bool on)
@@ -765,16 +885,10 @@ void updatecrouch(playerent *p, bool on)
 
 void crouch(bool on)
 {
-    if(player1->isspectating()) return;
+    if(player1->isspectating() && on) return;
     player1->trycrouch = on;
 }
 
-int inWater(int *type)
-{
-    if(hdr.waterlevel > (*type ? player1->o.z : (player1->o.z - player1->eyeheight))) return 1;
-    else return 0;
-}
-
 COMMAND(backward, "d");
 COMMAND(forward, "d");
 COMMAND(left, "d");
@@ -782,7 +896,6 @@ COMMAND(right, "d");
 COMMANDN(jump, jumpn, "d");
 COMMAND(attack, "d");
 COMMAND(crouch, "d");
-COMMAND(inWater, "i");
 
 void fixcamerarange(physent *cam)
 {
@@ -793,178 +906,41 @@ void fixcamerarange(physent *cam)
     while(cam->yaw>=360.0f) cam->yaw -= 360.0f;
 }
 
-FVARP(sensitivity, 1e-3f, 3.0f, 1000.0f);
-FVARP(scopesensscale, 1e-3f, 0.5f, 1000.0f);
-FVARP(sensitivityscale, 1e-3f, 1, 1000);
-FVARP(scopesens, 0, 0, 1000);
-VARP(scopesensfeel, 0, 0, 1);
-VARP(invmouse, 0, 0, 1);
-FVARP(mouseaccel, 0, 0, 1000);
-FVARP(mfilter, 0.0f, 0.0f, 6.0f);
-VARP(autoscopesens, 0, 0, 1);
+FVARP(sensitivity, 1e-3f, 3.0f, 1000.0f);       // general mouse sensitivity ("unscoped")
+FVARP(scopesens, 0, 0, 1000);                   // scoped mouse sensitivity (if zero, autoscopesens determines, how sensitivity is changed during scoping)
+FVARP(sensitivityscale, 1e-3f, 1, 1000);        // scale all sensitivity values (if unsure, keep at default value "1"- this parameter achieves cosmetic value changes only)
 
-float testsens=0;
-bool senst=0;
-int tsens(int x)
+VARP(autoscopesens, 0, 0, 1);                   // switches between scopesensscale and autoscopesensscale to calculate a scoped sensitivity (if scopesens is 0)
+FVARP(scopesensscale, 1e-3f, 0.5f, 1000.0f);    // if scoped, sens = sensitivity * scopesensscale (roughly)
+float autoscopesensscale = 0.4663077f;          // roughly scopefov/fov, recalculated if fov or scopefov changes (init value fits defaults fov 90 and scopefov 50, update, if defaults change)
+
+VARP(invmouse, 0, 0, 1);                        // invert y-axis movement (if "1")
+FVARP(mouseaccel, 0, 0, 1000);                  // make fast movement even faster (zero deactivates the feature)
+FVARP(mfilter, 0.0f, 0.0f, 6.0f);               // simple lowpass filtering (zero deactivates the feature)
+
+void mousemove(int idx, int idy)
 {
-    static bool highlock=0,lowlock=0;
-    static bool hightry=0,lowtry=0;
-    static float sensn=0,sensl=0,sensh=0;
-    static int nstep=1;
-    if (x==-2000) {  // RENDERING PART!!!
-        if(senst) {
-        draw_textf(
-        "\fJSensitivity Training (hotkeys):\n\fE1. try High Sens. %s\n2. try Low Sens. %s\n\fJ%s :"
-        "\fE\n3. choose: High Sens.\n4. choose: Low Sens.\n\fIrepeat the steps above until the training stops.\n\f35. Stop Training.",
-        VIRTW/4  , VIRTH/3,
-        hightry?"(TRIED)":"" , lowtry?"(TRIED)":"",
-        hightry&&lowtry?"after trying both choose the one you liked most":"now you can choose the sensitivity you preferred");
-        glPushMatrix(); glScalef(2,2,2);
-        draw_textf("step: \f0%d",VIRTW/2  , VIRTH/3*2,nstep);
-        glPopMatrix();
-        }
-        return 0;
-    }
-    if (x == SDLK_3 || x == SDLK_4)
-    {
-        if(!hightry || !lowtry) {
-            if(!hightry && !lowtry) {
-                conoutf("--- \f3ERROR:\f0before choosing a sensitivity, first try both higher and lower sens.");
-            } else {
-                conoutf("--- \f3ERROR:\f0try the %s%ser sensitivity too.",hightry?"":"high",lowtry?"":"low");
-            }
-        } else {
-            if (x == SDLK_3) //high sens
-            {
-                lowlock=1;
-                if (highlock)
-                {
-                    sensl=sensn;
-                    sensn=(sensh+sensl)/2.0f;
-                }
-                else
-                {
-                    sensl=sensn;
-                    sensn=sensh;
-                    sensh=sensn*2.0f;
-                }
-            }
-            if (x == SDLK_4) //low sens
-            {
-                highlock=1;
-                if (lowlock)
-                {
-                    sensh=sensn;
-                    sensn=(sensh+sensl)/2.0f;
-                }
-                else
-                {
-                    sensh=sensn;
-                    sensn=sensl;
-                    sensl=sensn/2.0f;
-                }
-            }
-            if(sensh/sensn > 1.04f) {
-            conoutf("--- \f0you chose the %ser sensitivity.",x==SDLK_3?"higher":"lower");
-            conoutf("--- \f0repeat previous steps by trying both higher and lower sens and then by choosing the one you like most.");
-            hudoutf("next step!");
-            nstep++;
-            }
-            testsens=sensn;
-            hightry=lowtry=0;
-        }
-
-    }
-    if (x == SDLK_2)
-    {
-        testsens=sensl;
-        conoutf("--- \f0You are now %strying the lower sensitivity.",lowtry?"re-":""); lowtry=1;
-    }
-    if (x == SDLK_1)
-    {
-        testsens=sensh;
-        conoutf("--- \f0You are now %strying the higher sensitivity.",hightry?"re-":""); hightry=1;
-    }
-    if (x==-1000)
-    {
-        float factor=rnd(800)+600;
-        factor/=1000;
-        sensh=sensitivity*factor*2.0f;
-        sensl=sensitivity*factor/2.0f;
-        sensn=sensitivity*factor;
-    }
-    if (sensh/sensn <= 1.04f || x == SDLK_5)
-    {
-        senst=0;
-        if(sensh/sensn <= 1.04f) {
-            conoutf("--- \f0Sensitivity Training Ended. happy fragging.");
-            sensitivity=sensn;
-        } else {
-            conoutf("--- \f0Sensitivity Training Stopped.");
-        }
-        hightry=lowtry=highlock=lowlock=0;
-        sensn=sensl=sensh=0;
-        testsens=0;
-        nstep=1;
-    }
-    return 0;
-}
-
-void findsens()
-{
-    if(!watchingdemo) {
-        senst=1;
-        tsens(-1000);
-        testsens=sensitivity;
-        conoutf("--- \f0Sensitivity Training Started.");
-        return;
-    }
-}
-COMMAND(findsens, "");
-
-inline bool zooming(playerent *plx) { return (plx->weaponsel->type == GUN_SNIPER && ((sniperrifle *)plx->weaponsel)->scoped); }
-
-void mousemove(int odx, int ody)
-{
-    static float fdx = 0, fdy = 0;
     if(intermission || (player1->isspectating() && player1->spectatemode==SM_FOLLOW1ST)) return;
-    float dx = odx, dy = ody;
-    if(mfilter > 0.0f)
-    {
+    bool zooming = player1->weaponsel->type == GUN_SNIPER && ((sniperrifle *)player1->weaponsel)->scoped;               // check if player uses scope
+    float dx = idx, dy = idy;
+    if(mfilter > 0.0001f)
+    { // simple IIR-like filter (1st order lowpass)
+        static float fdx = 0, fdy = 0;
         float k = mfilter * 0.1f;
         dx = fdx = dx * ( 1.0f - k ) + fdx * k;
         dy = fdy = dy * ( 1.0f - k ) + fdy * k;
     }
-    extern float scopesensfunc;
-    float cursens = sensitivity;
-    if(senst) {cursens=testsens;}
-    if(mouseaccel && curtime && (dx || dy)) cursens += 0.02f * mouseaccel * sqrtf(dx*dx + dy*dy)/curtime;
-    if(scopesens==0 || !zooming(player1))
-    {
-        if(scopesensfeel)
-        {
-            // AC 1.1
-            cursens /= 33.0f*sensitivityscale;
-            if( zooming(player1) ) { cursens *= autoscopesens ? scopesensfunc : scopesensscale; }
-            camera1->yaw += dx*cursens;
-            camera1->pitch -= dy*cursens*(invmouse ? -1 : 1);
-        }
-        else
-        {
-            // AC 1.0
-            if( zooming(player1) ) { cursens *= autoscopesens ? scopesensfunc : scopesensscale; }
-            float sensfactor = 33.0f*sensitivityscale;
-            camera1->yaw += dx*cursens/sensfactor;
-            camera1->pitch -= dy*cursens*(invmouse ? -1 : 1)/sensfactor;
-        }
-    }
-    else
-    {
-        // user provided value
-        float sensfactor = 33.0f*sensitivityscale;
-        camera1->yaw += dx*scopesens/sensfactor;
-        camera1->pitch -= dy*scopesens*(invmouse ? -1 : 1)/sensfactor;
+    double cursens = sensitivity;                                                                                       // basic unscoped sensitivity
+    if(mouseaccel > 0.0001f && curtime && (idx || idy)) cursens += 0.02f * mouseaccel * sqrtf(dx*dx + dy*dy)/curtime;   // optionally accelerated
+    if(zooming)
+    {                                                                                                                   //      when scoped:
+        if(scopesens > 0.0001f) cursens = scopesens;                                                                    //          if specified, use dedicated (fixed) scope sensitivity
+        else cursens *= autoscopesens ? autoscopesensscale : scopesensscale;                                            //          or adjust sensitivity by given (fixed) factor or based on scopefov/fov
     }
+    cursens /= 33.0f * sensitivityscale;                                                                                // final scaling
+
+    camera1->yaw += (float) (dx * cursens);
+    camera1->pitch -= (float) (dy * cursens * (invmouse ? -1 : 1));
 
     fixcamerarange();
     if(camera1!=player1 && player1->spectatemode!=SM_DEATHCAM)
@@ -992,35 +968,6 @@ void entinmap(physent *d)    // brute force but effective way to find a free spa
     }
     // leave ent at original pos, possibly stuck
     d->resetinterp();
-    conoutf(_("can't find entity spawn spot! (%d, %d)"), d->o.x, d->o.y);
+    conoutf("can't find entity spawn spot! (%d, %d)", int(d->o.x), int(d->o.y));
 }
 
-void vecfromyawpitch(float yaw, float pitch, int move, int strafe, vec &m)
-{
-    if(move)
-    {
-        m.x = move*-sinf(RAD*yaw);
-        m.y = move*cosf(RAD*yaw);
-    }
-    else m.x = m.y = 0;
-
-    if(pitch)
-    {
-        m.x *= cosf(RAD*pitch);
-        m.y *= cosf(RAD*pitch);
-        m.z = move*sinf(RAD*pitch);
-    }
-    else m.z = 0;
-
-    if(strafe)
-    {
-        m.x += strafe*cosf(RAD*yaw);
-        m.y += strafe*sinf(RAD*yaw);
-    }
-}
-
-void vectoyawpitch(const vec &v, float &yaw, float &pitch)
-{
-    yaw = -atan2(v.x, v.y)/RAD;
-    pitch = asin(v.z/v.magnitude())/RAD;
-}
