@@ -33,7 +33,7 @@ void *getprocaddress(const char *name)
 bool hasext(const char *exts, const char *ext)
 {
     int len = strlen(ext);
-    for(const char *cur = exts; (cur = strstr(cur, ext)); cur += len)
+    if(len) for(const char *cur = exts; (cur = strstr(cur, ext)); cur += len)
     {
         if((cur == exts || cur[-1] == ' ') && (cur[len] == ' ' || !cur[len])) return true;
     }
@@ -125,7 +125,7 @@ void gl_checkextensions()
 #endif
 }
 
-void gl_init(int w, int h, int bpp, int depth, int fsaa)
+void gl_init(int w, int h, int depth, int fsaa)
 {
     //#define fogvalues 0.5f, 0.6f, 0.7f, 1.0f
 
@@ -235,6 +235,17 @@ void box(block &b, float z1, float z2, float z3, float z4)
     xtraverts += 4;
 }
 
+void box2d(int x1, int y1, int x2, int y2, int gray)
+{
+    glColor3ub(gray, gray, gray);
+    glBegin(GL_LINE_LOOP);
+    glVertex2f(x1, y1);
+    glVertex2f(x2, y1);
+    glVertex2f(x2, y2);
+    glVertex2f(x1, y2);
+    glEnd();
+}
+
 void quad(GLuint tex, float x, float y, float s, float tx, float ty, float tsx, float tsy)
 {
     if(!tsy) tsy = tsx;
@@ -269,7 +280,7 @@ void circle(GLuint tex, float x, float y, float r, float tx, float ty, float tr,
     glVertex2f(x, y);
     loopi(subdiv+1)
     {
-        float c = cosf(2*M_PI*i/float(subdiv)), s = sinf(2*M_PI*i/float(subdiv));
+        float c = cosf(PI2 * i / float(subdiv)), s = sinf(PI2 * i / float(subdiv));
         glTexCoord2f(tx + tr*c, ty + tr*s);
         glVertex2f(x + r*c, y + r*s);
     }
@@ -371,13 +382,7 @@ void blendbox(int x1, int y1, int x2, int y2, bool border, int tex, color *c)
     {
         glDisable(GL_BLEND);
         if(tex>=0) glDisable(GL_TEXTURE_2D);
-        glColor3f(0.6f, 0.6f, 0.6f);
-        glBegin(GL_LINE_LOOP);
-        glVertex2f(x1, y1);
-        glVertex2f(x2, y1);
-        glVertex2f(x2, y2);
-        glVertex2f(x1, y2);
-        glEnd();
+        box2d(x1, y1, x2, y2, 155);
         glEnable(GL_BLEND);
     }
 
@@ -385,6 +390,24 @@ void blendbox(int x1, int y1, int x2, int y2, bool border, int tex, color *c)
     glDepthMask(GL_TRUE);
 }
 
+void framedquadtexture(GLuint tex, int x, int y, int xs, int ys, int border, int backlight, bool blend)
+{
+    if(border) blendbox(x - border, y - border, x + xs + border, y + ys + border, false);
+    glBindTexture(GL_TEXTURE_2D, tex);
+    if(blend) glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+    else glDisable(GL_BLEND);
+    glColor3ub(backlight, backlight, backlight);
+    glBegin(GL_TRIANGLE_STRIP);
+    glTexCoord2f(0, 0); glVertex2f(x,      y);
+    glTexCoord2f(1, 0); glVertex2f(x + xs, y);
+    glTexCoord2f(0, 1); glVertex2f(x,      y + ys);
+    glTexCoord2f(1, 1); glVertex2f(x + xs, y + ys);
+    glEnd();
+    xtraverts += 4;
+    dimeditinfopanel = 99;
+    if(!blend) glEnable(GL_BLEND);
+}
+
 VARP(aboveheadiconsize, 0, 50, 1000);
 VARP(aboveheadiconfadetime, 1, 2000, 10000);
 
@@ -442,25 +465,16 @@ void fixresizedscreen()
 #endif
 }
 
-float scopesensfunc = 0.4663077f;
-void scopefunc();
-FVARFP(fov, 75, 90, 120, scopefunc());
-VARFP(scopefov, 5, 50, 60, scopefunc());
+void fovchanged();
+FVARFP(fov, 75, 90, 120, fovchanged());
+VARFP(scopefov, 5, 50, 60, fovchanged());
 VARP(spectfov, 5, 110, 120);
-void scopefunc()
+void fovchanged()
 {
-    scopesensfunc = tan(((float)scopefov)*0.5f*RAD)/tan(fov*0.5f*RAD);
+    extern float autoscopesensscale;
+    autoscopesensscale = tan(((float)scopefov)*0.5f*RAD)/tan(fov*0.5f*RAD);
 }
 
-// map old fov values to new ones
-void fovcompat(int *oldfov)
-{
-    extern float aspect;
-    setfvar("fov", atan(tan(RAD/2.0f*(*oldfov)/aspect)*aspect)*2.0f/RAD, true);
-}
-
-COMMAND(fovcompat, "i");
-
 float dynfov()
 {
     if(player1->weaponsel->type == GUN_SNIPER && ((sniperrifle *)player1->weaponsel)->scoped) return (float)scopefov;
@@ -468,8 +482,9 @@ float dynfov()
     else return (float)fov;
 }
 
-VAR(fog, 64, 180, 1024);
-VAR(fogcolour, 0, 0x8099B3, 0xFFFFFF);
+VARF(fog, 64, DEFAULT_FOG, 1024, flagmapconfigchange());
+VARF(fogcolour, 0, DEFAULT_FOGCOLOUR, 0xFFFFFF, flagmapconfigchange());
+
 float fovy, aspect;
 int farplane;
 
@@ -506,9 +521,9 @@ void recomputecamera()
             case SM_OVERVIEW:
             {
                 // TODO : fix water rendering
-                camera1->o.x = mapdims[0] + mapdims[4]/2;
-                camera1->o.y = mapdims[1] + mapdims[5]/2;
-                camera1->o.z = mapdims[7] + 1;
+                camera1->o.x = clmapdims.xm;
+                camera1->o.y = clmapdims.ym;
+                camera1->o.z = clmapdims.maxceil + 1;
                 camera1->pitch = -90;
                 camera1->yaw = 0;
 
@@ -640,16 +655,19 @@ void drawreflection(float hf, int w, int h, float changelod, bool refract)
 
     extern float wsx1, wsx2, wsy1, wsy2;
     int sx = 0, sy = 0, sw = size, sh = size;
-    bool scissor = reflectscissor && (wsx1 > -1 || wsy1 > -1 || wsx1 < 1 || wsy1 < 1);
-    if(scissor)
+    if(reflectscissor && (wsx1 > -1 || wsy1 > -1 || wsx1 < 1 || wsy1 < 1))
     {
         sx = int(floor((wsx1+1)*0.5f*size));
         sy = int(floor((wsy1+1)*0.5f*size));
         sw = int(ceil((wsx2+1)*0.5f*size)) - sx;
         sh = int(ceil((wsy2+1)*0.5f*size)) - sy;
-        glScissor(sx, sy, sw, sh);
-        glEnable(GL_SCISSOR_TEST);
     }
+    glScissor(sx, sy, sw, sh);
+    glEnable(GL_SCISSOR_TEST);
+
+    if(!refract) glCullFace(GL_BACK);
+    glViewport(0, 0, size, size);
+    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
 
     resetcubes();
 
@@ -658,10 +676,6 @@ void drawreflection(float hf, int w, int h, float changelod, bool refract)
 
     setupstrips();
 
-    if(!refract) glCullFace(GL_BACK);
-    glViewport(0, 0, size, size);
-    glClear(GL_DEPTH_BUFFER_BIT);
-
     glEnable(GL_TEXTURE_2D);
 
     glPushMatrix();
@@ -734,7 +748,7 @@ void drawreflection(float hf, int w, int h, float changelod, bool refract)
     if(!refract) glCullFace(GL_FRONT);
     glViewport(0, 0, w, h);
 
-    if(scissor) glDisable(GL_SCISSOR_TEST);
+    glDisable(GL_SCISSOR_TEST);
 
     glBindTexture(GL_TEXTURE_2D, refract ? refracttex : reflecttex);
     glCopyTexSubImage2D(GL_TEXTURE_2D, 0, sx, sy, sx, sy, sw, sh);
@@ -798,14 +812,14 @@ void drawminimap(int w, int h)
     physent minicam;
     camera1 = &minicam;
     camera1->type = ENT_CAMERA;
-    camera1->o.x = mapdims[0] + mapdims[4]/2.0f;
-    camera1->o.y = mapdims[1] + mapdims[5]/2.0f;
+    camera1->o.x = clmapdims.xm;
+    camera1->o.y = clmapdims.ym;
 
-    //float gdim = max(mapdims[4], mapdims[5])+2.0f; //make 1 cube smaller to give it a black edge
-    float gdim = max(mapdims[4], mapdims[5]); //no border
+    //float gdim = max(mapdims.xspan, mapdims.yspan)+2.0f; //make 1 cube smaller to give it a black edge
+    float gdim = max(clmapdims.xspan, clmapdims.yspan); //no border
 
-    if(!gdim) gdim = ssize/2.0f;
-    camera1->o.z = mapdims[7] + 1;
+    if(gdim < 1) gdim = ssize/2.0f;
+    camera1->o.z = clmapdims.maxceil + 1;
     camera1->pitch = -90;
     camera1->yaw = 0;
 
@@ -820,7 +834,7 @@ void drawminimap(int w, int h)
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT); // stencil added 2010jul22
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
-    glOrtho(-orthd, orthd, -orthd, orthd, 0, (mapdims[7]-mapdims[6])+2); // depth of map +2 covered
+    glOrtho(-orthd, orthd, -orthd, orthd, 0, (clmapdims.maxceil - clmapdims.minfloor) + 2); // depth of map +2 covered
     glScalef(1, -1, 1);
     glMatrixMode(GL_MODELVIEW);
     glCullFace(GL_BACK);
@@ -836,10 +850,10 @@ void drawminimap(int w, int h)
     renderstrips();
     glDepthFunc(GL_LESS);
     rendermapmodels();
-    renderzones(mapdims[7]);
+    renderzones(clmapdims.maxceil);
     //renderentities();// IMHO better done by radar itself, if at all
     resettmu(0);
-    float hf = hdr.waterlevel-0.3f;
+    float hf = waterlevel - 0.3f;
     renderwater(hf, 0, 0);
 
     //draw a black border to prevent the minimap texture edges from bleeding in radarview
@@ -849,10 +863,10 @@ void drawminimap(int w, int h)
     glColor3f(0, 0, 0);
     glLineWidth(2.0f);
     glBegin(GL_LINE_LOOP);
-        glVertex3f(mapdims[0]+0.02f,            mapdims[1]+0.02f, mapdims[7]);
-        glVertex3f(mapdims[0]+mapdims[4]-0.02f, mapdims[1]+0.02f, mapdims[7]);
-        glVertex3f(mapdims[0]+mapdims[4]-0.02f, mapdims[1]+mapdims[5]-0.02f, mapdims[7]);
-        glVertex3f(mapdims[0]+0.02f,            mapdims[1]+mapdims[5]-0.02f, mapdims[7]);
+        glVertex3f(clmapdims.x1 + 0.02f,                   clmapdims.y1 + 0.02f,                   clmapdims.maxceil);
+        glVertex3f(clmapdims.x1 + clmapdims.xspan - 0.02f, clmapdims.y1 + 0.02f,                   clmapdims.maxceil);
+        glVertex3f(clmapdims.x1 + clmapdims.xspan - 0.02f, clmapdims.y1 + clmapdims.yspan - 0.02f, clmapdims.maxceil);
+        glVertex3f(clmapdims.x1 + 0.02f,                   clmapdims.y1 + clmapdims.yspan - 0.02f, clmapdims.maxceil);
     glEnd();
     glLineWidth(prevLineWidth);
     glEnable(GL_BLEND);
@@ -904,10 +918,10 @@ void setperspective(float fovy, float aspect, float nearplane, float farplane)
     GLdouble ydist = nearplane * tan(fovy/2*RAD), xdist = ydist * aspect;
     if(player1->isspectating() && player1->spectatemode == SM_OVERVIEW)
     {
-        int gdim = max(mapdims[4], mapdims[5]);
+        int gdim = max(clmapdims.xspan, clmapdims.yspan);
         if(!gdim) gdim = ssize/2;
         int orthd = 2 + gdim/2;
-        glOrtho(-orthd, orthd, -orthd, orthd, 0, (mapdims[7]-mapdims[6])+2); // depth of map +2 covered
+        glOrtho(-orthd, orthd, -orthd, orthd, 0, (clmapdims.maxceil - clmapdims.minfloor) + 2); // depth of map +2 covered
     }
     else
     {
@@ -928,16 +942,15 @@ void sethudgunperspective(bool on)
     glMatrixMode(GL_MODELVIEW);
 }
 
-void drawhudgun(int w, int h, float aspect, int farplane)
+void drawhudgun()
 {
     sethudgunperspective(true);
 
-    if(hudgun && (specthudgun || !player1->isspectating()) && camera1->type==ENT_PLAYER)
+    if(!rendermenumdl() && hudgun && (specthudgun || !player1->isspectating()) && camera1->type==ENT_PLAYER)
     {
         playerent *p = (playerent *)camera1;
         if(p->state==CS_ALIVE) p->weaponsel->renderhudmodel();
     }
-    rendermenumdl();
 
     sethudgunperspective(false);
 }
@@ -988,10 +1001,20 @@ void readdepth(int w, int h, vec &pos)
     vec4 world;
     invmvpmatrix.transform(screen, world);
     pos = vec(world.x, world.y, world.z).div(world.w);
+    intersectgeometry(camera1->o, pos);
 }
 
-void gl_drawframe(int w, int h, float changelod, float curfps)
+VARP(ignoreoverride_nowaterreflect, 0, 0, 1);
+VARP(ignoreoverride_nostencilshadows, 0, 0, 1);
+
+int effective_stencilshadow = 0;
+
+void gl_drawframe(int w, int h, float changelod, float curfps, int elapsed)
 {
+    extern int mapoverride_nostencilshadows, mapoverride_nowaterreflect;
+    effective_stencilshadow = mapoverride_nostencilshadows && !ignoreoverride_nostencilshadows ? 0 : stencilshadow;
+    bool effective_waterreflect = waterreflect && !editmode && (!mapoverride_nowaterreflect || ignoreoverride_nowaterreflect);
+
     dodynlights();
     drawminimap(w, h);
 
@@ -1000,7 +1023,7 @@ void gl_drawframe(int w, int h, float changelod, float curfps)
     aspect = float(w)/h;
     fovy = 2*atan2(tan(float(dynfov())/2*RAD), aspect)/RAD;
 
-    float hf = hdr.waterlevel-0.3f;
+    float hf = waterlevel - 0.3f;
     bool underwater = camera1->o.z<hf;
 
     glFogi(GL_FOG_START, (fog+64)/8);
@@ -1029,7 +1052,7 @@ void gl_drawframe(int w, int h, float changelod, float curfps)
     transplayer();
     readmatrices();
 
-    if(!underwater && waterreflect)
+    if(!underwater && effective_waterreflect)
     {
         extern int wx1;
         if(wx1>=0)
@@ -1040,8 +1063,8 @@ void gl_drawframe(int w, int h, float changelod, float curfps)
         }
     }
 
-    if(stencilshadow && hasstencil && stencilbits >= 8) glClearStencil((hasSTS || hasST2) && !hasSTW ? 128 : 0);
-    glClear((outsidemap(camera1) ? GL_COLOR_BUFFER_BIT : 0) | GL_DEPTH_BUFFER_BIT | (stencilshadow && hasstencil && stencilbits >= 8 ? GL_STENCIL_BUFFER_BIT : 0));
+    if(effective_stencilshadow && hasstencil && stencilbits >= 8) glClearStencil((hasSTS || hasST2) && !hasSTW ? 128 : 0);
+    glClear((outsidemap(camera1) ? GL_COLOR_BUFFER_BIT : 0) | GL_DEPTH_BUFFER_BIT | (effective_stencilshadow && hasstencil && stencilbits >= 8 ? GL_STENCIL_BUFFER_BIT : 0));
 
     glEnable(GL_TEXTURE_2D);
 
@@ -1079,7 +1102,7 @@ void gl_drawframe(int w, int h, float changelod, float curfps)
     rendermapmodels();
     endmodelbatches();
 
-    if(stencilshadow && hasstencil && stencilbits >= 8) drawstencilshadows();
+    if(effective_stencilshadow && hasstencil && stencilbits >= 8) drawstencilshadows();
 
     startmodelbatches();
     renderentities();
@@ -1099,7 +1122,7 @@ void gl_drawframe(int w, int h, float changelod, float curfps)
     WaypointClass.Think();
     // end add
 
-    drawhudgun(w, h, aspect, farplane);
+    drawhudgun();
 
     resettmu(0);
 
@@ -1107,7 +1130,7 @@ void gl_drawframe(int w, int h, float changelod, float curfps)
 
     render_particles(curtime, PT_DECAL_MASK);
 
-    int nquads = renderwater(hf, !waterreflect || underwater ? 0 : reflecttex, !waterreflect || !waterrefract || underwater ? 0 : refracttex);
+    int nquads = renderwater(hf, !effective_waterreflect || underwater ? 0 : reflecttex, !effective_waterreflect || !waterrefract || underwater ? 0 : refracttex);
 
     render_particles(curtime, ~PT_DECAL_MASK);
 
@@ -1125,9 +1148,11 @@ void gl_drawframe(int w, int h, float changelod, float curfps)
         glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
         disablepolygonoffset(GL_POLYGON_OFFSET_LINE, false);
     }
+    extern vector<int> tagclipcubes;
+    tagclipcubes.setsize(0);
 
     extern vector<vertex> verts;
-    gl_drawhud(w, h, (int)round(curfps), nquads, verts.length(), underwater);
+    gl_drawhud(w, h, int(curfps + 0.5f), nquads, verts.length(), underwater, elapsed);
 
     glEnable(GL_CULL_FACE);
     glEnable(GL_FOG);
