@@ -184,8 +184,6 @@ void cleanupexplosion()
     }
 }
 
-#define MAXPARTYPES 20
-
 struct particle { vec o, d; int fade, type; int millis; particle *next; };
 particle *parlist[MAXPARTYPES], *parempty = NULL;
 
@@ -248,7 +246,10 @@ void newparticle(const vec &o, const vec &d, int fade, int type)
     parlist[type] = p;
 }
 
-static struct parttype { int type; float r, g, b; int gr, tex; float sz; } parttypes[] =
+const char *particletypenames[MAXPARTYPES + 1] = { "SPARK", "SMOKE", "ECLOSEST", "BLOOD", "DEMOTRACK", "FIREBALL", "SHOTLINE", "BULLETHOLE", "BLOODSTAIN", "SCORCH",        // 0..9
+                                                   "HUDMUZZLEFLASH", "MUZZLEFLASH", "ELIGHT", "ESPAWN", "EAMMO", "EPICKUP", "EMODEL", "ECARROT", "ELADDER", "EFLAG", "" };  // 10..19
+
+static struct parttype { int type; float r, g, b; int gr, tex; float sz; } parttypes[MAXPARTYPES] =
 {
     { PT_PART,       0.4f, 0.4f, 0.4f, 2,  0, 0.06f }, // yellow: sparks
     { PT_PART,       1.0f, 1.0f, 1.0f, 20, 1, 0.15f }, // grey:   small smoke
@@ -258,6 +259,7 @@ static struct parttype { int type; float r, g, b; int gr, tex; float sz; } partt
     { PT_FIREBALL,   1.0f, 0.5f, 0.5f, 0,  2, 7.0f  }, // explosion fireball
     { PT_SHOTLINE,   1.0f, 1.0f, 0.7f, 0, -1, 0.0f  }, // yellow: shotline
     { PT_BULLETHOLE, 1.0f, 1.0f, 1.0f, 0,  3, 0.3f  }, // hole decal
+
     { PT_STAIN,      0.5f, 0.0f, 0.0f, 0,  4, 0.6f  }, // red:    blood stain
     { PT_DECAL,      1.0f, 1.0f, 1.0f, 0,  5, 1.5f  }, // scorch decal
     { PT_HUDFLASH,   1.0f, 1.0f, 1.0f, 0,  6, 0.7f  }, // hudgun muzzle flash
@@ -386,7 +388,7 @@ void render_particles(int time, int typemask)
                     bool inside = p->o.dist(camera1->o) <= sz*1.25f; //1.25 is max wobble scale
                     vec oc(p->o);
                     oc.sub(camera1->o);
-                    if(reflecting && !refracting) oc.z = p->o.z - (hdr.waterlevel-0.3f);
+                    if(reflecting && !refracting) oc.z = p->o.z - (waterlevel - 0.3f);
                     glRotatef(inside ? camera1->yaw - 180 : atan2(oc.y, oc.x)/RAD - 90, 0, 0, 1);
                     glRotatef((inside ? camera1->pitch : asin(oc.z/oc.magnitude())/RAD) - 90, 1, 0, 0);
 
@@ -480,7 +482,7 @@ void render_particles(int time, int typemask)
             }
             else
             {
-			    if(pt.gr) p->o.z -= ((lastmillis-p->millis)/3.0f)*time/(pt.gr*10000);
+                if(pt.gr) p->o.z -= ((lastmillis-p->millis)/3.0f)*time/(pt.gr*10000);
                 if(pt.type==PT_PART || pt.type==PT_BLOOD)
                 {
                     p->o.add(vec(p->d).mul(time/20000.0f));
@@ -562,6 +564,7 @@ void particle_flash(int type, float scale, float angle, const vec &p)
 void particle_splash(int type, int num, int fade, const vec &p)
 {
     if(parttypes[type].type==PT_BLOOD && !blood) return;
+    if(fade == 0) fade = 1;
     loopi(num)
     {
         const int radius = 150;
@@ -578,6 +581,19 @@ void particle_splash(int type, int num, int fade, const vec &p)
     }
 }
 
+void particle_cube(int type, int num, int fade, int x, int y)
+{
+    sqr *s = S(x, y);
+    const int fc = s->ceil - s->floor + (s->vdelta + 3) / 4;
+    const float floor = s->floor - (s->type == FHF ? float(s->vdelta) / 4 : 0);
+    if(SOLID(s) || fc < 1) return;
+    loopi(num * fc / 12)
+    {
+        vec p(float(rnd(100)) / 100.0 + x, float(rnd(100)) / 100.0 + y, float(rnd(100) * fc) / 100 + floor), d(0, 0, 0);
+        newparticle(p, d, rnd(fade*3), type);
+    }
+}
+
 VARP(maxtrail, 1, 500, 10000);
 
 void particle_trail(int type, int fade, const vec &s, const vec &e)
@@ -615,15 +631,16 @@ bool addbullethole(dynent *d, const vec &from, const vec &to, float radius, bool
     if(surface.iszero() || (radius>0 && (dist < mag-radius || dist > mag+radius))) return false;
     vec o(from);
     o.add(ray.mul(dist));
-    o.add(vec(surface).mul(0.005f));
+    o.add(vec(surface).normalize().mul(0.01f));
     newparticle(o, surface, bulletholettl, 7);
     if(noisy && bulletbouncesound && bulletbouncesoundrad && d!=player1 && o.dist(camera1->o) <= bulletbouncesoundrad)
     {
-        audiomgr.playsound(o.z<hdr.waterlevel ? S_BULLETWATERHIT : S_BULLETHIT, &o, SP_LOW);
+        audiomgr.playsound(o.z < waterlevel ? S_BULLETWATERHIT : S_BULLETHIT, &o, SP_LOW);
     }
     return true;
 }
 
+
 VARP(scorch, 0, 1, 1);
 VARP(scorchttl, 0, 10000, 30000);
 
@@ -632,7 +649,7 @@ bool addscorchmark(vec &o, float radius)
     if(!scorchttl || !scorch) return false;
     sqr *s = S((int)o.x, (int)o.y);
     if(s->type!=SPACE || o.z-s->floor>radius) return false;
-    newparticle(vec(o.x, o.y, s->floor+0.005f), vec(0, 0, 1), scorchttl, 9);
+    newparticle(vec(o.x, o.y, s->floor+0.02f), vec(0, 0, 1), scorchttl, 9);
     return true;
 }
 
@@ -646,12 +663,13 @@ VARP(bulletairsounddestrad, 0, 8, 1000);
 void addshotline(dynent *pl, const vec &from, const vec &to)
 {
     if(pl == player1 || !shotlinettl || !shotline) return;
-    bool fx = shotlinettl > 0 && (player1->isspectating() || !multiplayer(false)); // allow fx only in spect mode and locally
+    bool fx = shotlinettl > 0 && (player1->isspectating() || !multiplayer(NULL)); // allow fx only in spect mode and locally
     if(rnd(3) && !fx) return; // show all shotlines when fx enabled
 
     int start = (camera1->o.dist(to) <= 10.0f) ? 8 : 5;
     vec unitv;
     float dist = to.dist(from, unitv);
+    if(dist < 1) return;
     unitv.div(dist);
 
     // shotline visuals
@@ -671,6 +689,6 @@ void addshotline(dynent *pl, const vec &from, const vec &to)
     soundpos.mul(fd/(fd+td)*dist);
     soundpos.add(from);
     if(!bulletairsound || soundpos.dist(camera1->o) > bulletairsoundrad) return; // outside player radius
-    audiomgr.playsound(S_BULLETAIR1 + rnd(2), &soundpos, SP_LOW);
+    audiomgr.playsound(rnd(2) ? S_BULLETAIR1 : S_BULLETAIR2, &soundpos, SP_LOW);
 }
 
