@@ -10,10 +10,12 @@ static font *fontdef = NULL;
 font *curfont = NULL;
 
 VARP(allowblinkingtext, 0, 0, 1); // if you're so inclined
-VARP(__fontsetting, 0, 0, 2);
 
 void newfont(char *name, char *tex, int *defaultw, int *defaulth, int *offsetx, int *offsety, int *offsetw, int *offseth)
 {
+    if(*defaulth < 10) return;          // (becomes FONTH)
+    Texture *_tex = textureload(tex);
+    if(_tex == notexture || !_tex->xs || !_tex->ys) return;
     font *f = fonts.access(name);
     if(!f)
     {
@@ -22,7 +24,7 @@ void newfont(char *name, char *tex, int *defaultw, int *defaulth, int *offsetx,
         f->name = name;
     }
 
-    f->tex = textureload(tex);
+    f->tex = _tex;
     f->chars.shrink(0);
     f->defaultw = *defaultw;
     f->defaulth = *defaulth;
@@ -34,19 +36,21 @@ void newfont(char *name, char *tex, int *defaultw, int *defaulth, int *offsetx,
 
     fontdef = f;
 }
-
-extern GLenum texformat(int bpp);
+COMMANDN(font, newfont, "ssiiiiii");
 
 void fontchar(int *x, int *y, int *w, int *h)
 {
     if(!fontdef) return;
 
     font::charinfo &c = fontdef->chars.add();
-    c.x = *x;
-    c.y = *y;
     c.w = *w ? *w : fontdef->defaultw;
     c.h = *h ? *h : fontdef->defaulth;
+    c.left    = (*x + fontdef->offsetx) / float(fontdef->tex->xs);
+    c.top     = (*y + fontdef->offsety) / float(fontdef->tex->ys);
+    c.right   = (*x + c.w + fontdef->offsetw) / float(fontdef->tex->xs);
+    c.bottom  = (*y + c.h + fontdef->offseth) / float(fontdef->tex->ys);
 }
+COMMAND(fontchar, "iiii");
 
 void fontskip(int *n)
 {
@@ -54,33 +58,18 @@ void fontskip(int *n)
 
     fontdef->skip = *n;
 }
-
-COMMANDN(font, newfont, "ssiiiiii");
-COMMAND(fontchar, "iiii");
 COMMAND(fontskip, "i");
 
-string myfont = "default";
-void newsetfont(const char *name)
-{
-    if ( setfont(name) ) copystring(myfont,name);
-}
-
 bool setfont(const char *name)
 {
     font *f = fonts.access(name);
     if(!f) return false;
-    int v = -1;
-    if(strcmp(name, "default")==0)
-        v = 0;
-    else if(strcmp(name, "serif")==0)
-        v = 1;
-    else if(strcmp(name, "mono")==0)
-        v = 2;
-    if(v!=-1) __fontsetting = v;
     curfont = f;
     return true;
 }
-COMMANDN(setfont, newsetfont, "s");
+COMMAND(setfont, "s");
+
+COMMANDF(curfont, "", () { result(curfont && curfont->name ? curfont->name : ""); });
 
 font *getfont(const char *name)
 {
@@ -113,209 +102,58 @@ void draw_textf(const char *fstr, int left, int top, ...)
     draw_text(str, left, top);
 }
 
-
-
-extern int shdsize, outline, win32msg;
-
-
-#if SDL_BYTEORDER == SDL_BIG_ENDIAN
-#define RMASK 0xff000000
-#define GMASK 0x00ff0000
-#define BMASK 0x0000ff00
-#define AMASK 0x000000ff
-#else
-#define RMASK 0x000000ff
-#define GMASK 0x0000ff00
-#define BMASK 0x00ff0000
-#define AMASK 0xff000000
-#endif
-/*
-// ringbuf for utf8 character storage
-struct charringbuf : ringbuf<font::utf8charinfo, 32>
+inline int draw_char_contd(font &f, font::charinfo &info, int charcode, int x, int y)
 {
-    // find by character code
-    int findbycharcode(int code)
-    {
-        loopi(len)
-        {
-            if(data[i].code == code)
-                return i;
-        }
-
-        return -1;
-    }
-};
-
-
-TTF_Font *ttffont = NULL;
-font utf8font;
-charringbuf utf8chars;
-
-void initfont()
-{
-    static bool initialized = false;
-    if(!initialized)
-    {
-        TTF_Init();
-
-        int fsize = 64;
-        const char *fontname = "packages/misc/font.ttf";
-        ttffont = TTF_OpenFont(findfile(path(fontname, true), "r"), fsize);
-
-        utf8font.defaulth = 0;
-        utf8font.defaultw = 0;
-        utf8font.offsetw = 0;
-        utf8font.offseth = 10;
-        utf8font.offsetx = 0;
-        utf8font.offsety = 0;
-
-        initialized = true;
-    }
-}
-
-void createutf8charset()
-{
-    int isize = 512;
-
-    SDL_Surface *charsetsurface = SDL_CreateRGBSurface(SDL_SWSURFACE, isize, isize, 32, RMASK, GMASK, BMASK, AMASK);
-
-    if(charsetsurface)
-    {
-        SDL_Color color;
-        color.r = 255;
-        color.g = 255;
-        color.b = 255;
-
-        int posx = 0;
-        int posy = 0;
-
-        loopv(utf8chars)
-        {
-            font::utf8charinfo &charinfo = utf8chars[i];
-            int code = charinfo.code;
-
-            char u[5] = {0,0,0,0,0};
-            utf8::append(code, u);
-
-            SDL_Surface *fontsurface = TTF_RenderUTF8_Blended(ttffont, u, color);
-
-            // update row/column info
-            if(posx + fontsurface->w > charsetsurface->w)
-            {
-                posx = 0;
-                posy += fontsurface->h; // fixme
-            }
-            if(posy + fontsurface->h > charsetsurface->h)
-                break;
-
-            // blit onto charset
-            SDL_SetAlpha(fontsurface, 0, 0);
-            blitsurface(charsetsurface, fontsurface, posx, posy);
-
-            // update charinfo properties
-            charinfo.x = posx;
-            charinfo.y = posy;
-            charinfo.w = fontsurface->w;
-            charinfo.h = fontsurface->h;
-
-            posx += fontsurface->w;
-            SDL_FreeSurface(fontsurface);
-        }
-
-        utf8font.tex = createtexturefromsurface("utf8charset", charsetsurface);
-
-        //extern void savepng(SDL_Surface *s, const char *name);
-        //savepng(t, "font.png");
-
-        SDL_FreeSurface(charsetsurface);
-    }
-}
-
-void addutf8char(int code)
-{
-    // add to buf
-    font::utf8charinfo charinfo;
-    charinfo.code = code;
-    charinfo.x = charinfo.y = charinfo.w = charinfo.h = 0;
-    utf8chars.add(charinfo);
-
-    // update charset
-    createutf8charset();
-}
-
-font::charinfo *loadchar(int code)
-{
-    int idx = utf8chars.findbycharcode(code);
-    if(idx >= 0)
-        return &utf8chars[idx];
-
-    // add
-    addutf8char(code);
-
-    idx = utf8chars.findbycharcode(code);
-    return &utf8chars[idx];
-}
-*/
-int draw_char(font &f, font::charinfo &info, int charcode, int x, int y)
-{
-/*
-    // fixme
-    glEnd();
-    glBindTexture(GL_TEXTURE_2D, f.tex->id);
-    glBegin(GL_QUADS);
-*/
-    float tc_left    = (info.x + f.offsetx) / float(f.tex->xs);
-    float tc_top     = (info.y + f.offsety) / float(f.tex->ys);
-    float tc_right   = (info.x + info.w + f.offsetw) / float(f.tex->xs);
-    float tc_bottom  = (info.y + info.h + f.offseth) / float(f.tex->ys);
-
-    glTexCoord2f(tc_left,  tc_top   ); glVertex2f(x,          y);
-    glTexCoord2f(tc_right, tc_top   ); glVertex2f(x + info.w, y);
-    glTexCoord2f(tc_right, tc_bottom); glVertex2f(x + info.w, y + info.h);
-    glTexCoord2f(tc_left,  tc_bottom); glVertex2f(x,          y + info.h);
+    glTexCoord2f(info.left,  info.top   ); glVertex2f(x,          y);
+    glTexCoord2f(info.right, info.top   ); glVertex2f(x + info.w, y);
+    glTexCoord2f(info.right, info.bottom); glVertex2f(x + info.w, y + info.h);
+    glTexCoord2f(info.left,  info.bottom); glVertex2f(x,          y + info.h);
 
     xtraverts += 4;
     return info.w;
 }
 
-/*
-// fixme
-font::charinfo &getcharinfo(int c)
-{
-    if(curfont->chars.inrange(c-curfont->skip))
-    {
-        font::charinfo &info = curfont->chars[c-curfont->skip];
-        return info;
-    }
-    //else { font::charinfo &info = *loadchar(c); return info; }
-    //return NULL;
-    font::charinfo &info = curfont->chars[0]; // 0 || (FONTCHARS-1)
-    return info;
-}
-*/
-
 static int draw_char(int c, int x, int y)
 {
     if(curfont->chars.inrange(c-curfont->skip))
     {
         font::charinfo &info = curfont->chars[c-curfont->skip];
 
-        return draw_char(*curfont, info, c, x, y);
+        return draw_char_contd(*curfont, info, c, x, y);
     }
-    /*
-    else
-    {
-        // fixme
-        glEnd();
-        font::charinfo &info = *loadchar(c);
-        glBegin(GL_QUADS);
-
-        return draw_char(utf8font, info, c, x, y);
-    }
-    */
     return 0;
 }
 
+vector<threeint> igraphbatch;
+
+void queue_igraph(int c, int x, int y)
+{
+    threeint v = { c, x, y };
+    igraphbatch.add(v);
+}
+
+VARP(igraphsize, 80, 120, 300);
+VARP(igraphsizehardcoded, 80, 106, 160);
+VARP(igraphanimate, 0, 1, 1);
+
+void render_igraphs()
+{
+    igraphbatch.sort(cmpintasc);
+    int last = 0, w[2] = { (FONTH * igraphsize) / 100, (FONTH * igraphsizehardcoded) / 100 }, offs[2] = { (FONTH - w[0]) / 2, (FONTH - w[1]) / 2 }, ishc = 0;
+    igraph *ig = NULL;
+    glColor4ub(255, 255, 255, 255);
+    loopv(igraphbatch)
+    {
+        if(last != igraphbatch[i].key) ig = getusedigraph((last = igraphbatch[i].key)), ishc = last > 0 && last < 10 ? 1 : 0;
+        if(ig && ig->tex)
+        {
+            int &x = igraphbatch[i].val1, &y = igraphbatch[i].val2, fpos = ((totalmillis + x * x + VIRTW * y) % ig->frames.last() + ig->frames.last()) % ig->frames.last(), frame = 0;
+            float fw = min(1.0f, float(ig->tex->ys) / ig->tex->xs);
+            if(igraphanimate) loopvj(ig->frames) if(ig->frames[j] < fpos) frame++;
+            quad(ig->tex->id, x + offs[ishc], y + offs[ishc], w[ishc], (frame % max(1, ig->tex->xs / ig->tex->ys)) * fw, 0, fw, 1);
+        }
+    }
+}
 
 //stack[sp] is current color index
 static void text_color(char c, char *stack, int size, int &sp, bvec color, int a)
@@ -330,7 +168,7 @@ static void text_color(char c, char *stack, int size, int &sp, bvec color, int a
         if(c=='r') c = stack[(sp > 0) ? --sp : sp]; // restore color
         else if(c == 'b') { if(allowblinkingtext && !ignoreblinkingbit) stack[sp] *= -1; } // blinking text - only if allowed
         else stack[sp] = c;
-        switch(abs(stack[sp]))
+        switch(iabs(stack[sp]))
         {
             case '0': color = bvec( 2,  255,  128 ); break;   // green: player talk
             case '1': color = bvec( 96,  160, 255 ); break;   // blue: team chat
@@ -342,8 +180,7 @@ static void text_color(char c, char *stack, int size, int &sp, bvec color, int a
             case '7': color = bvec( 153,  51,  51 ); break;   // dark red: dead admin
             case '8': color = bvec( 192,  64, 192 ); break;   // magenta
             case '9': color = bvec( 255, 102,   0 ); break;   // orange
-            //extendeded color palette
-            //case 'a': case 'A':color = bvec( 0xFF, 0xCC, 0xCC); break;   // some lowercase seem to have special meaning like 'b' (flashing text) so not yet using them
+
             case 'A':color = bvec( 0xff, 0xb7, 0xb7); break;   // red set
             case 'B':color = bvec( 0xCC, 0x33, 0x33); break;   //
             case 'C':color = bvec( 0x66, 0x33, 0x33); break;   //
@@ -376,11 +213,14 @@ static void text_color(char c, char *stack, int size, int &sp, bvec color, int a
 
             case 'Y':color = bvec( 0xC7, 0xD1, 0xE2); break;   // lt gray
             case 'Z':color = bvec( 0x32, 0x32, 0x32); break;   // dark gray
-            // white (provided color): everything else
-            //default: color = bvec( 255, 255, 255 ); break;
+
+            case 'u': color = bvec(120, 240, 120); break;   // stats: green
+            case 'v': color = bvec(120, 120, 240); break;   // stats: blue
+            case 'w': color = bvec(230, 230, 110); break;   // stats: yellow
+            case 'x': color = bvec(250, 100, 100); break;   // stats: red
         }
         int b = (int) (sinf(lastmillis / 200.0f) * 115.0f);
-        b = stack[sp] > 0 ? 100 : min(abs(b), 100);
+        b = stack[sp] > 0 ? 100 : min(iabs(b), 100);
         glColor4ub(color.x, color.y, color.z, (a * b) / 100);
     }
 }
@@ -432,6 +272,7 @@ void text_endcolumns()
         else if(c==' ')  { x += curfont->defaultw; TEXTWHITE(i) }\
         else if(c=='\n') { TEXTLINE(i) x = 0; y += FONTH; }\
         else if(c=='\f') { if(str[i+1]) { i++; TEXTCOLOR(i) }}\
+        else if(c=='\1') { if(str[i+1]) { i++; TEXTIGRAPH(i) }}\
         else if(c=='\a') { if(str[i+1]) { i++; }}\
         else if(curfont->chars.inrange(c-curfont->skip))\
         {\
@@ -442,10 +283,12 @@ void text_endcolumns()
                 for(; str[i+1]; i++)\
                 {\
                     int c = str[i+1];\
+                    bool isig = c == '\1';\
                     if(c=='\f') { if(str[i+2]) i++; continue; }\
                     if(i-j > 16) break;\
-                    if(!curfont->chars.inrange(c-curfont->skip)) break;\
-                    int cw = curfont->chars[c-curfont->skip].w + 1;\
+                    if(!isig && !curfont->chars.inrange(c-curfont->skip)) break;\
+                    int cw = isig ? FONTH + 1 : curfont->chars[c-curfont->skip].w + 1;\
+                    if(isig && str[i + 2]) i++;\
                     if(w + cw >= maxwidth) break;\
                     w += cw;\
                 }\
@@ -464,6 +307,7 @@ void text_endcolumns()
                     TEXTINDEX(j)\
                     int c = str[j];\
                     if(c=='\f') { if(str[j+1]) { j++; TEXTCOLOR(j) }}\
+                    else if(c=='\1') { if(str[j+1]) { j++; TEXTIGRAPH(j) }}\
                     else { TEXTCHAR(j) }\
                 }
 
@@ -474,6 +318,7 @@ int text_visible(const char *str, int hitx, int hity, int maxwidth)
     #define TEXTWHITE(idx) if(y+FONTH > hity && x >= hitx) return idx;
     #define TEXTLINE(idx) if(y+FONTH > hity) return idx;
     #define TEXTCOLOR(idx)
+    #define TEXTIGRAPH(idx) x += FONTH; TEXTWHITE(idx)
     #define TEXTCHAR(idx) x += curfont->chars[c-curfont->skip].w+1; TEXTWHITE(idx)
     #define TEXTWORD TEXTWORDSKELETON
     TEXTSKELETON
@@ -482,6 +327,7 @@ int text_visible(const char *str, int hitx, int hity, int maxwidth)
     #undef TEXTWHITE
     #undef TEXTLINE
     #undef TEXTCOLOR
+    #undef TEXTIGRAPH
     #undef TEXTCHAR
     #undef TEXTWORD
     return i;
@@ -495,6 +341,7 @@ void text_pos(const char *str, int cursor, int &cx, int &cy, int maxwidth)
     #define TEXTWHITE(idx)
     #define TEXTLINE(idx)
     #define TEXTCOLOR(idx)
+    #define TEXTIGRAPH(idx) x += FONTH;
     #define TEXTCHAR(idx) x += curfont->chars[c-curfont->skip].w + 1;
     #define TEXTWORD TEXTWORDSKELETON if(i >= cursor) break;
     cx = INT_MIN;
@@ -506,6 +353,7 @@ void text_pos(const char *str, int cursor, int &cx, int &cy, int maxwidth)
     #undef TEXTWHITE
     #undef TEXTLINE
     #undef TEXTCOLOR
+    #undef TEXTIGRAPH
     #undef TEXTCHAR
     #undef TEXTWORD
 }
@@ -517,6 +365,7 @@ void text_bounds(const char *str, int &width, int &height, int maxwidth)
     #define TEXTWHITE(idx)
     #define TEXTLINE(idx) if(x > width) width = x;
     #define TEXTCOLOR(idx)
+    #define TEXTIGRAPH(idx) x += FONTH + 1;
     #define TEXTCHAR(idx) x += curfont->chars[c-curfont->skip].w + 1;
     #define TEXTWORD x += w + 1;
     width = 0;
@@ -528,13 +377,11 @@ void text_bounds(const char *str, int &width, int &height, int maxwidth)
     #undef TEXTWHITE
     #undef TEXTLINE
     #undef TEXTCOLOR
+    #undef TEXTIGRAPH
     #undef TEXTCHAR
     #undef TEXTWORD
 }
 
-/** This is the 1.0.4 function
-    It will substituted by draw_text_wip
-    I am putting this temporarily here because it is impossible to test without colours : Brahma */
 void draw_text(const char *str, int left, int top, int r, int g, int b, int a, int cursor, int maxwidth)
 {
 #define TEXTINDEX(idx) if(idx == cursor) { cx = x; cy = y; cc = str[idx]; }
@@ -542,18 +389,20 @@ void draw_text(const char *str, int left, int top, int r, int g, int b, int a, i
 #define TEXTWHITE(idx)
 #define TEXTLINE(idx)
 #define TEXTCOLOR(idx) text_color(str[idx], colorstack, sizeof(colorstack), colorpos, color, a);
-#define TEXTCHAR(idx) x += draw_char(c, left+x, top+y)+1;
+#define TEXTIGRAPH(idx) queue_igraph((uchar)str[idx], left+x, top+y), x += FONTH + 1;
+#define TEXTCHAR(idx) x += draw_char(c, left+x, top+y) + 1;
 #define TEXTWORD TEXTWORDSKELETON
     char colorstack[10];
     bvec color(r, g, b);
     int colorpos = 0, cx = INT_MIN, cy = 0, cc = ' ';
     colorstack[0] = 'c'; //indicate user color
+    igraphbatch.setsize(0);
     glBlendFunc(GL_SRC_ALPHA, curfont->tex->bpp==32 ? GL_ONE_MINUS_SRC_ALPHA : GL_ONE);
     glBindTexture(GL_TEXTURE_2D, curfont->tex->id);
     glBegin(GL_QUADS);
     glColor4ub(color.x, color.y, color.z, a);
     TEXTSKELETON
-            glEnd();
+    glEnd();
     if(cursor >= 0)
     {
         if(cx == INT_MIN) { cx = x; cy = y; }
@@ -561,180 +410,163 @@ void draw_text(const char *str, int left, int top, int r, int g, int b, int a, i
         int cw = curfont->chars.inrange(cc-33) ? curfont->chars[cc-33].w + 1 : curfont->defaultw;
         rendercursor(left+cx, top+cy, cw);
     }
+    render_igraphs();
 #undef TEXTINDEX
 #undef TEXTTAB
 #undef TEXTWHITE
 #undef TEXTLINE
 #undef TEXTCOLOR
+#undef TEXTIGRAPH
 #undef TEXTCHAR
 #undef TEXTWORD
 }
 
-/* WIP ALERT */
-/*
-void draw_text_wip(const char *str, int left, int top, int r, int g, int b, int a, int cursor, int maxwidth)
-{
-    char colorstack[10];
-    bvec color(r, g, b);
-    int colorpos = 0, cx = INT_MIN, cy = 0, cc = ' ';
-    colorstack[0] = 'c'; //indicate user color
-
-    glBlendFunc(GL_SRC_ALPHA, curfont->tex->bpp==32 ? GL_ONE_MINUS_SRC_ALPHA : GL_ONE);
-    glBindTexture(GL_TEXTURE_2D, curfont->tex->id);
-
-    glBegin(GL_QUADS);
-    glColor4ub(color.x, color.y, color.z, a);
-
-    std::string text(str);
-    std::string::iterator begin = text.begin();
-    std::string::iterator end = text.end();
-    std::string::iterator cursoriter = end;
-    if(cursor >= 0 && cursor < utf8::distance(begin, end))
-    {
-        cursoriter = begin;
-        utf8::advance(cursoriter, cursor, end);
-    }
-
-    int y = 0, x = 0, col = 0, colx = 0;
-
-    for(std::string::iterator iter = text.begin(); iter != text.end(); utf8::next(iter, text.end()))
-    {
-        int c = utf8::peek_next(iter, text.end());
-
-        if(iter == cursoriter)
-        {
-            cx = x;
-            cy = y;
-            cc = c;
-        }
-
-        if(c=='\t')
-        {
-            if(columns && col<columns->length())
-            {
-                colx += (*columns)[col++];
-                x = colx;
-            }
-            else x = TABALIGN(x);
-        }
-        else if(c==' ')
-        {
-            x += curfont->defaultw;
-        }
-        else if(c=='\n')
-        {
-            x = 0;
-            y += FONTH;
-        }
-        else if(c=='\f')
-        {
-            std::string::iterator test = iter;
-            test++;
-            if(test != end)
-            {
-                c = utf8::next(iter, end);
-                text_color(c, colorstack, sizeof(colorstack), colorpos, color, a);
-            }
-        }
-        else if(c=='\a')
-        {
-            std::string::iterator next = iter;
-            next++;
-            if(next != end)
-            {
-                iter++;
-            }
-
-        }
-        else if(curfont->chars.inrange(c-curfont->skip))
-        {
-            font::charinfo &cinfo = getcharinfo(c);
-
-            if(maxwidth != -1)
-            {
-                std::string::iterator next = iter;
-                int w = cinfo.w;
-
-                do
-                {
-                    std::string::iterator test = iter;
-                    int c = utf8::next(test, end);
-                    if(test == end) break;
-
-                    if(c=='\f')
-                    {
-                        std::string::iterator test = iter;
-                        utf8::advance(test, 2, end);
-                        if(test == end) break;
-                        utf8::next(iter, end);
-                        continue;
-                    }
-                    if(utf8::distance(iter, next) > 16) break;
-                    //if(!curfont->chars.inrange(c-curfont->skip)) fixme
-                    if(c < curfont->skip) // fixme
-                    {
-                        break;
-                    }
-                    int cw = getcharinfo(c).w + 1;
-                    if(w + cw >= maxwidth) break;
-                    w += cw;
-
-                    utf8::next(iter, end);
-
-                } while(true);
-
-                if(x + w >= maxwidth && next != begin) //fixme
-                {
-                    x = 0;
-                    y += FONTH;
-                }
-
-                for(; next <= iter && next != end; )
-                {
-                    int c = utf8::peek_next(next, end);
-                    if(next == cursoriter) { cx = x; cy = y; cc = c; }
-
-                    if(c=='\f')
-                    {
-                        std::string::iterator test = next;
-                        utf8::next(test, end);
-                        if(test != end)
-                        {
-                            c = utf8::next(next, end);
-                            text_color(c, colorstack, sizeof(colorstack), colorpos, color, a);
-                        }
-                    }
-                    else
-                    {
-                        x += draw_char(c, left+x, top+y)+1;
-                    }
-
-                    utf8::next(next, end);
-                }
-
-            }
-            else
-            {
-                x += draw_char(c, left+x, top+y)+1;
-            }
-        }
-    }
-
-    glEnd();
-    if(cursor >= 0)
-    {
-        if(cx == INT_MIN) { cx = x; cy = y; }
-        if(maxwidth != -1 && cx >= maxwidth) { cx = 0; cy += FONTH; }
-        int cw = curfont->chars.inrange(cc-curfont->skip) ? curfont->chars[cc-curfont->skip].w + 1 : curfont->defaultw;
-        rendercursor(left+cx, top+cy, cw);
-    }
-}
-*/
 void reloadfonts()
 {
-    //createutf8charset();
-
     enumerate(fonts, font, f,
         if(!reloadtexture(*f.tex)) fatal("failed to reload font texture");
     );
 }
+
+void cutcolorstring(char *text, int maxlen)
+{ // limit string length, ignore color codes
+    if(!curfont) return;
+    int len = 0;
+    maxlen *= curfont->defaultw;
+    while(*text)
+    {
+        if(*text == '\f' && text[1]) text++;
+        else if(*text == '\1' && text[1]) text++, len += FONTH + 1;
+        else if(*text == '\t') len = TABALIGN(len);
+        else len += curfont->chars.inrange(*text - curfont->skip) ? curfont->chars[*text - curfont->skip].w : curfont->defaultw;
+        if(len > maxlen) { *text = '\0'; break; }
+        text++;
+    }
+}
+
+bool filterunrenderables(char *s)
+{
+    bool res = false;
+    char *d = s;
+    while(*s)
+    {
+        if(!curfont->chars.inrange(*s - curfont->skip) && *s != ' ') res = true;
+        else *d++ = *s;
+        s++;
+    }
+    *d = '\0';
+    return res;
+}
+
+// animated inlined graphics
+
+VAR(igraphdefaultframetime, 5, 200, 2000);
+VARP(hideigraphs, 0, 0, 1);
+#define IGRAPHPATH "packages" PATHDIVS "misc" PATHDIVS "igraph" PATHDIVS
+hashtable<const char *, igraph> igraphs;    // keyed by shorthand
+hashtable<const char *, char> igraphsi;     // known filenames
+vector<igraph *> usedigraphs;               // char codes ('\1' + n)
+
+void addigraph(char *fname)
+{
+    char *b, *s = newstring(fname), *mnem = strtok_r(s, "_", &b), *r;
+    if(mnem && *mnem && !igraphs.access(mnem))
+    { // new mnem
+        defformatstring(filename)("%s%s.png", IGRAPHPATH, fname);
+        Texture *tex = textureload(filename);
+        if(tex != notexture && tex->xs && tex->ys)
+        {
+            igraph &ig = igraphs[newstring(mnem)];
+            ig.fname = fname;
+            ig.tex = tex;
+            ig.used = !mnem[1] && *mnem >= '1' && *mnem <= '9' ? *mnem - '0' : 0;
+            if(ig.used && usedigraphs.inrange(ig.used)) usedigraphs[ig.used] = &ig;     // hardcode "1".."9"
+            while((r = strtok_r(NULL, "_", &b))) ig.frames.add(max(5, (int)ATOI(r)));
+            if(ig.frames.empty()) ig.frames.add(igraphdefaultframetime);
+            while(ig.frames.length() < tex->xs / tex->ys) ig.frames.add(ig.frames.last());
+            loopv(ig.frames) if(i) ig.frames[i] += ig.frames[i - 1];
+#ifdef _DEBUG
+            clientlogf(" loaded igraph \"%s\", short \"%s\", %dx%d, %d frame%s", fname, mnem, tex->xs, tex->ys, ig.frames.length(), ig.frames.length() > 1 ? "s" : "");
+#endif
+        }
+    }
+    igraphsi[fname] = 0;
+    delstring(s);
+}
+
+void updateigraphs()    // read all filenames and parse new ones (also preloads the textures)
+{
+    vector<char *> files;
+    while(usedigraphs.length() <= (int)'\n') usedigraphs.add(NULL); // allocate hardcoded slots and skip slot 10 ('\n')
+    listfiles(IGRAPHPATH, "png", files);
+    loopvrev(files)
+    {
+        if(igraphsi.access(files[i])) delstring(files[i]);
+        else addigraph(files[i]);
+    }
+}
+COMMAND(updateigraphs, "");
+
+igraph *getusedigraph(int i)
+{
+    if(usedigraphs.inrange(i)) return usedigraphs[i];
+    return NULL;
+}
+
+int getigraph(const char *mnem)
+{
+    igraph *ig = igraphs.access(mnem);
+    if(ig)
+    {
+        if(!ig->used && usedigraphs.length() < 255)
+        {
+            ig->used = usedigraphs.length();
+            usedigraphs.add(ig);
+        }
+        return ig->used;
+    }
+    return 0;
+}
+
+void _getigraph(char *s)
+{
+    uchar res[3] = { '\1', (uchar)getigraph(s), '\0' };
+    result(res[1] ? (char*)res : "");
+}
+COMMANDN(getigraph, _getigraph, "s");
+
+void encodeigraphs(char *d, const char *s, int len) // find known igraphs in a string and substitute with rendercodes, only used for console messages
+{
+    if(hideigraphs) copystring(d, s, len);
+    else
+    {
+        len--;
+        string mnem;
+        loopi(len)
+        {
+            if(*s == ':' && (!i || isspace(s[-1])) && i < len - 1)
+            {
+                int l = strcspn(s + 1, " \t\n."), u;
+                if(l && l < MAXSTRLEN)
+                {
+                    copystring(mnem, s + 1, l + 1);
+                    if((u = getigraph(mnem)))
+                    { // found one: encode
+                        *d++ = '\1';
+                        *d++ = u;
+                        s += l + 1;
+                        i++;
+                        continue;
+                    }
+                }
+            }
+            *d++ = *s++;
+        }
+        *d = '\0';
+    }
+}
+
+void enumigraphs(vector<const char *> &igs, const char *s, int len)
+{
+    enumeratek(igraphs, const char *, mnem, if(!strncasecmp(mnem, s, len)) igs.add(mnem));
+}
