@@ -53,7 +53,7 @@ struct sline
 
     sline() : bgcolor(NULL), textcolor(0) { copystring(s, ""); }
 
-    void addcol(int priority, const char *format = NULL, ...)
+    void addcol(int priority, const char *format = NULL, ...) PRINTFARGS(3, 4)
     {
         if(priority < 0) return;
         coldata &col = cols.add();
@@ -109,7 +109,7 @@ struct teamscore
     }
 };
 
-void teamflagscores(int &team1, int &team2)
+void calcteamscores(int scores[4])
 {
     teamscore teamscores[2] = { teamscore(TEAM_CLA), teamscore(TEAM_RVSF) };
     loopv(players) if(players[i] && players[i]->team != TEAM_SPECT)
@@ -120,9 +120,12 @@ void teamflagscores(int &team1, int &team2)
     {
         teamscores[team_base(discscores[i].team)].addscore(discscores[i]);
     }
-    if(!watchingdemo) teamscores[team_base(player1->team)].addplayer(player1);
-    team1 = teamscores[0].flagscore;
-    team2 = teamscores[1].flagscore;
+    if(!watchingdemo && player1->team != TEAM_SPECT) teamscores[team_base(player1->team)].addplayer(player1);
+    loopi(2)
+    {
+        scores[i] = teamscores[i].flagscore;
+        scores[i + 2] = teamscores[i].frags;
+    }
 }
 
 static int teamscorecmp(const teamscore *x, const teamscore *y)
@@ -192,10 +195,10 @@ void renderdiscscores(int team)
         line.addcol(sc_frags, "%d", d.frags);
         line.addcol(sc_deaths, "%d", d.deaths);
         line.addcol(sc_ratio, "%.2f", SCORERATIO(d.frags, d.deaths));
-        if(multiplayer(false) || watchingdemo) line.addcol(sc_score, "%d", max(d.points, 0));
-        line.addcol(sc_lag, clag);
+        if(multiplayer(NULL) || watchingdemo) line.addcol(sc_score, "%d", max(d.points, 0));
+        line.addcol(sc_lag, "%s", clag);
         line.addcol(sc_clientnum, "DISC");
-        line.addcol(sc_name, d.name);
+        line.addcol(sc_name, "%s", d.name);
     }
 }
 
@@ -206,13 +209,13 @@ void renderscore(playerent *d)
     const char *status = "";
     string lagping;
     static color localplayerc(0.2f, 0.2f, 0.2f, 0.2f);
-    if(d->clientrole==CR_ADMIN) status = d->state==CS_DEAD ? "\f7" : "\f3";
-    else if(d->state==CS_DEAD) status = "\f4";
+    if(d->clientrole==CR_ADMIN) status = d->state!=CS_ALIVE ? "\f7" : "\f3";
+    else if(d->state!=CS_ALIVE) status = "\f4";
     if (team_isspect(d->team)) copystring(lagping, "SPECT");
     else if (d->state==CS_LAGGED || (d->ping > 999 && d->plag > 99)) copystring(lagping, "LAG");
     else
     {
-        if(multiplayer(false)) formatstring(lagping)("%s/%s", colorpj(d->plag), colorping(d->ping));
+        if(multiplayer(NULL)) formatstring(lagping)("%s/%s", colorpj(d->plag), colorping(d->ping));
         else formatstring(lagping)("%d/%d", d->plag, d->ping);
     }
     const char *ign = d->ignored ? " (ignored)" : (d->muted ? " (muted)" : "");
@@ -224,18 +227,26 @@ void renderscore(playerent *d)
     line.addcol(sc_frags, "%d", d->frags);
     line.addcol(sc_deaths, "%d", d->deaths);
     line.addcol(sc_ratio, "%.2f", SCORERATIO(d->frags, d->deaths));
-    if(multiplayer(false) || watchingdemo)
+    if(multiplayer(NULL) || watchingdemo)
     {
         line.addcol(sc_score, "%d", max(d->points, 0));
-        line.addcol(sc_lag, lagping);
+        line.addcol(sc_lag, "%s", lagping);
     }
     line.addcol(sc_clientnum, "\fs\f%d%d\fr", cncolumncolor, d->clientnum);
-    line.addcol(sc_name, "\fs%s%s\fr%s", status, colorname(d), ign);
+    char flagicon = '\0';
+    if(m_flags) //show flag icon at flag carrier with use radaricons font
+    {
+        loopi(2)
+        {
+            if(flaginfos[i].state == CTFF_STOLEN && flaginfos[i].actor == d) flagicon = m_ktf ? 'L' : "DH"[i];
+        }
+    }
+    line.addcol(sc_name, "\fs%s%s\fr%s%s%c ", status, colorname(d), ign, flagicon ? " \a" : "", flagicon);
 }
 
 int totalplayers = 0;
 
-int renderteamscore(teamscore *t)
+void renderteamscore(teamscore *t)
 {
     if(!scorelines.empty()) // space between teams
     {
@@ -243,31 +254,30 @@ int renderteamscore(teamscore *t)
         space.s[0] = 0;
     }
     sline &line = scorelines.add();
-    int n = t->teammembers.length();
-    defformatstring(plrs)("(%d %s)", n, n == 1 ? "player" : "players");
 
     if(m_flags) line.addcol(sc_flags, "%d", t->flagscore);
     line.addcol(sc_frags, "%d", t->frags);
     line.addcol(sc_deaths, "%d", t->deaths);
     line.addcol(sc_ratio, "%.2f", SCORERATIO(t->frags, t->deaths));
-    if(multiplayer(false) || watchingdemo)
+    if(multiplayer(NULL) || watchingdemo)
     {
         line.addcol(sc_score, "%d", max(t->points, 0));
         line.addcol(sc_lag);
     }
-    line.addcol(sc_clientnum, team_string(t->team));
-    line.addcol(sc_name, plrs);
+    line.addcol(sc_clientnum, "%s", team_string(t->team));
+    int n = t->teammembers.length();
+    line.addcol(sc_name, "(%d %s)", n, n == 1 ? "player" : "players");
 
     static color teamcolors[2] = { color(1.0f, 0, 0, 0.2f), color(0, 0, 1.0f, 0.2f) };
     line.bgcolor = &teamcolors[team_base(t->team)];
     loopv(t->teammembers) renderscore(t->teammembers[i]);
-    return n;
 }
 
 extern bool watchingdemo;
 
 void reorderscorecolumns()
 {
+    static string scoreboardtitle;
     needscoresreorder = false;
     extern void *scoremenu;
     sline sscore;
@@ -276,14 +286,15 @@ void reorderscorecolumns()
     sscore.addcol(sc_frags, "frags");
     sscore.addcol(sc_deaths, "deaths");
     sscore.addcol(sc_ratio, "ratio");
-    if(multiplayer(false) || watchingdemo)
+    if(multiplayer(NULL) || watchingdemo)
     {
         sscore.addcol(sc_score, "score");
         sscore.addcol(sc_lag, "pj/ping");
     }
     sscore.addcol(sc_clientnum, "cn");
     sscore.addcol(sc_name, "name");
-    menutitle(scoremenu, newstring(sscore.getcols()));
+    copystring(scoreboardtitle, sscore.getcols());
+    menutitlemanual(scoremenu, scoreboardtitle);
 }
 
 void renderscores(void *menu, bool init)
@@ -323,7 +334,7 @@ void renderscores(void *menu, bool init)
         winner = m_flags ?
             (teamscores[sort].flagscore > teamscores[team_opposite(sort)].flagscore ? sort : -1) :
             (teamscores[sort].frags > teamscores[team_opposite(sort)].frags ? sort : -1);
-                   
+
     }
     else
     { // ffa mode
@@ -355,13 +366,15 @@ void renderscores(void *menu, bool init)
         formatstring(modeline)("\"%s\" on map %s", modestr(gamemode, modeacronyms > 0), fldrprefix ? getclientmap()+strlen("maps/") : getclientmap());
     }
 
-    extern int minutesremaining;
-    if((gamemode>1 || (gamemode==0 && (multiplayer(false) || watchingdemo))) && minutesremaining >= 0)
+    extern int minutesremaining, gametimedisplay;
+    extern string gtime;
+
+    if((gamemode>1 || (gamemode==0 && (multiplayer(NULL) || watchingdemo))) && minutesremaining >= 0)
     {
         if(!minutesremaining)
         {
             concatstring(modeline, ", intermission");
-            
+
             if (m_teammode) // Add in the winning team
             {
                 switch(winner)
@@ -380,21 +393,30 @@ void renderscores(void *menu, bool init)
                 else concatformatstring(modeline, ", \f1%s wins!", scores[0]->name);
             }
         }
-        else concatformatstring(modeline, ", %d %s remaining", minutesremaining, minutesremaining==1 ? "minute" : "minutes");
+        else
+        {
+            concatformatstring(modeline, ", %s", gtime);
+            if(gametimedisplay == 2) concatformatstring(modeline, " / %d:00", gametimemaximum/60000);
+            else concatformatstring(modeline, " remaining");
+        }
     }
 
-    if(multiplayer(false))
+    if(multiplayer(NULL))
     {
         serverinfo *s = getconnectedserverinfo();
         if(s)
         {
-            if(servstate.mastermode > MM_OPEN) concatformatstring(serverline, servstate.mastermode == MM_MATCH ? "M%d " : "P ", servstate.matchteamsize);
+            if(servstate.mastermode > MM_OPEN)
+            {
+                if(servstate.mastermode == MM_MATCH) concatformatstring(serverline, "M%d ", servstate.matchteamsize);
+                else concatstring(serverline, "P ");
+            }
             // ft: 2010jun12: this can write over the menu boundary
             //concatformatstring(serverline, "%s:%d %s", s->name, s->port, s->sdesc);
             // for now we'll just cut it off, same as the serverbrowser
             // but we might want to consider wrapping the bottom-line to accomodate longer descriptions - to a limit.
             string text;
-            filtertext(text, s->sdesc);
+            filtertext(text, s->sdesc, FTXT__SERVDESC);
             //for(char *p = text; (p = strchr(p, '\"')); *p++ = ' ');
             //text[30] = '\0'; // serverbrowser has less room - +8 chars here - 2010AUG03 - seems it was too much, falling back to 30 (for now): TODO get real width of menu as reference-width. FIXME: cutoff
             concatformatstring(serverline, "%s:%d %s", s->name, s->port, text);
@@ -403,7 +425,7 @@ void renderscores(void *menu, bool init)
     }
 
     menureset(menu);
-    loopv(scorelines) menumanual(menu, scorelines[i].getcols(), NULL, scorelines[i].bgcolor);
+    loopv(scorelines) menuimagemanual(menu, NULL, "radaricons", scorelines[i].getcols(), NULL, scorelines[i].bgcolor);
     menuheader(menu, modeline, serverline);
 
     // update server stats
@@ -417,7 +439,7 @@ void renderscores(void *menu, bool init)
 
 #define MAXJPGCOM 65533  // maximum JPEG comment length
 
-void addstr(char *dest, const char *src) { if(strlen(dest) + strlen(src) < MAXJPGCOM) strcat(dest, src); }
+static void addstr(char *&dest, const char *end, const char *src) { size_t l = strlen(src); if(dest + l < end) copystring(dest, src, l + 1), dest += l; }
 
 const char *asciiscores(bool destjpg)
 {
@@ -433,34 +455,35 @@ const char *asciiscores(bool destjpg)
     loopv(players) if(players[i]) scores.add(players[i]);
     scores.sort(scorecmp);
 
-    buf[0] = '\0';
+    copystring(buf, "Comment", MAXJPGCOM);              // include PNG keyword and skip it (which looks like a hack because it is one)
+    char *t = buf + 8, *e = buf + MAXJPGCOM;
     if(destjpg)
     {
-        formatstring(text)("AssaultCube Screenshot (%s)\n", asctime());
-        addstr(buf, text);
+        formatstring(text)("AssaultCube Screenshot (%s)\n", asctimestr());
+        addstr(t, e, text);
     }
     if(getclientmap()[0])
     {
-        formatstring(text)("\n\"%s\" on map %s", modestr(gamemode, 0), getclientmap(), asctime());
-        addstr(buf, text);
+        formatstring(text)("\n\"%s\" on map %s", modestr(gamemode, 0), getclientmap());
+        addstr(t, e, text);
     }
-    if(multiplayer(false))
+    if(multiplayer(NULL))
     {
         serverinfo *s = getconnectedserverinfo();
         if(s)
         {
             string sdesc;
-            filtertext(sdesc, s->sdesc, 1);
+            filtertext(sdesc, s->sdesc, FTXT__SERVDESC | FTXT_NOCOLOR);
             formatstring(text)(", %s:%d %s", s->name, s->port, sdesc);
-            addstr(buf, text);
+            addstr(t, e, text);
         }
     }
     if(destjpg)
-        addstr(buf, "\n");
+        addstr(t, e, "\n");
     else
     {
         formatstring(text)("\n%sfrags deaths cn%s name\n", m_flags ? "flags " : "", m_teammode ? " team" : "");
-        addstr(buf, text);
+        addstr(t, e, text);
     }
     loopv(scores)
     {
@@ -473,7 +496,7 @@ const char *asciiscores(bool destjpg)
         else
             formatstring(text)("%s %4d   %4d %2d%s %s%s\n", m_flags ? flags : "", d->frags, d->deaths, d->clientnum,
                             m_teammode ? team : "", d->name, d->clientrole==CR_ADMIN ? " (admin)" : d==player1 ? " (you)" : "");
-        addstr(buf, text);
+        addstr(t, e, text);
     }
     discscores.sort(discscorecmp);
     loopv(discscores)
@@ -486,15 +509,15 @@ const char *asciiscores(bool destjpg)
             formatstring(text)("%s(disconnected)%s (%s%d/%d)\n", d.name, m_teammode ? team : "", m_flags ? flags : "", d.frags, d.deaths);
         else
             formatstring(text)("%s %4d   %4d --%s %s(disconnected)\n", m_flags ? flags : "", d.frags, d.deaths, m_teammode ? team : "", d.name);
-        addstr(buf, text);
+        addstr(t, e, text);
     }
     if(destjpg)
     {
         extern int minutesremaining;
         formatstring(text)("(%sfrags/deaths), %d minute%s remaining\n", m_flags ? "flags/" : "", minutesremaining, minutesremaining == 1 ? "" : "s");
-        addstr(buf, text);
+        addstr(t, e, text);
     }
-    return buf;
+    return buf + 8;
 }
 
 void consolescores()
