@@ -74,7 +74,7 @@ void resolverinit()
         resolverthread &rt = resolverthreads.add();
         rt.query = NULL;
         rt.starttime = 0;
-        rt.thread = SDL_CreateThread(resolverloop, &rt);
+        rt.thread = SDL_CreateThread(resolverloop, "ResolverThread", &rt);
     }
     SDL_UnlockMutex(resolvermutex);
 }
@@ -84,10 +84,8 @@ void resolverstop(resolverthread &rt)
     SDL_LockMutex(resolvermutex);
     if(rt.query)
     {
-#ifndef __APPLE__
-        SDL_KillThread(rt.thread);
-#endif
-        rt.thread = SDL_CreateThread(resolverloop, &rt);
+        SDL_DetachThread(rt.thread);
+        rt.thread = SDL_CreateThread(resolverloop, "ResolverThread", &rt);
     }
     rt.query = NULL;
     rt.starttime = 0;
@@ -252,7 +250,7 @@ int connectwithtimeout(ENetSocket sock, const char *hostname, ENetAddress &addre
     if(!conncond) conncond = SDL_CreateCond();
     SDL_LockMutex(connmutex);
     connectdata cd = { sock, address, -1 };
-    connthread = SDL_CreateThread(connectthread, &cd);
+    connthread = SDL_CreateThread(connectthread, "ConnectThread", &cd);
 
     int starttime = SDL_GetTicks(), timeout = 0;
     for(;;)
@@ -285,8 +283,6 @@ vector<serverinfo *> servers;
 ENetSocket pingsock = ENET_SOCKET_NULL;
 int lastinfo = 0;
 
-char *getservername(int n) { return servers[n]->name; }
-
 serverinfo *findserverinfo(ENetAddress address)
 {
     loopv(servers) if(servers[i]->address.host == address.host && servers[i]->port == address.port) return servers[i];
@@ -487,9 +483,9 @@ void checkpings()
         si->numplayers = getint(p);
         si->minremain = getint(p);
         getstring(text, p);
-        filtertext(si->map, text, 1);
+        filtertext(si->map, behindpath(text), FTXT__MAPNAME);
         getstring(text, p);
-        filterservdesc(si->sdesc, text);
+        filtertext(si->sdesc, text, FTXT__SERVDESC);
         copystring(si->description, si->sdesc);
         si->maxclients = getint(p);
         if(p.remaining())
@@ -510,7 +506,7 @@ void checkpings()
                         loopi(si->numplayers)
                         {
                             getstring(text, p);
-                            filtertext(text, text, 0);
+                            filtertext(text, text, FTXT__PLAYERNAME);
                             if(text[0] && !p.overread())
                             {
                                 si->playernames.add((const char *)si->namedata + q.length());
@@ -536,7 +532,7 @@ void checkpings()
                             if(*text && !p.overread())
                             {
                                 text[MAXINFOLINELEN] = '\0';
-                                cutcolorstring(text, 80);
+                                cutcolorstring(text, 100);
                                 ADDINFOLINE(strcmp(text, ".") ? text : "");
                             }
                             else break;
@@ -552,7 +548,7 @@ void checkpings()
                         while(p.remaining())
                         {
                             getstring(text, p);
-                            filtertext(text, text, 0);
+                            filtertext(text, behindpath(text), FTXT__MAPNAME);
                             if(*text && !p.overread())
                             {
                                 text[MAXINFOLINELEN] = '\0';
@@ -644,7 +640,8 @@ VARP(serversort, 0, 0, NUMSERVSORT-1);
 VARP(serversortdir, 0, 0, 1);
 VARP(showonlygoodservers, 0, 0, 1);
 VAR(shownamesinbrowser, 0, 0, 1);
-VARP(showminremain, 0, 0, 1);
+VARP(showminremain, 0, 1, 1);
+VARP(showallplayersnumber, 0, 1, 1);
 VARP(serversortpreferofficial, 0, 1, 1);
 
 void serversortprepare()
@@ -689,8 +686,8 @@ int sicompare(serverinfo **ap, serverinfo **bp)
         case SBS_DESC: // description
         {
             static string ad, bd;
-            filtertext(ad, a->sdesc);
-            filtertext(bd, b->sdesc);
+            filtertext(ad, a->sdesc, FTXT__SERVDESC);
+            filtertext(bd, b->sdesc, FTXT__SERVDESC);
             if(!ad[0] && bd[0]) return dir;
             if(ad[0] && !bd[0]) return -dir;
             int mdir = dir * strcasecmp(ad, bd);
@@ -782,12 +779,12 @@ void addfavcategory(const char *refdes)
     string text, val;
     char alx[FC_NUM];
     if(!refdes) { intret(0); return; }
-    filtertext(text, refdes);
+    filtertext(text, refdes, FTXT__FAVCATEGORY);
     if(!text[0]) { intret(0); return; }
     loopv(favcats) if(!strcmp(favcats[i], text)) { intret(i + 1); return; }
     favcats.add(newstring(text));
-    bool oldpersist = per_idents;
-    per_idents = false; // only keep changed values
+    bool oldpersist = persistidents;
+    persistidents = false; // only keep changed values
     loopi(FC_NUM) alx[i] = getalias(favcatargname(text, i)) ? 1 : 0;
     loopi(3)
     {
@@ -798,13 +795,13 @@ void addfavcategory(const char *refdes)
     const int defk[] = { FC_WEIGHT, FC_DESC, FC_TAG, FC_KEYS, FC_IGNORE, FC_ALPHA };
     const char *defv[] = { "0",     val,     refdes, "",      "",        "20" };
     loopi(sizeof(defk)/sizeof(defk[0])) { if(!alx[defk[i]]) alias(favcatargname(text, defk[i]), defv[i]); }
-    per_idents = oldpersist;
+    persistidents = oldpersist;
     intret(favcats.length());
 }
 
 void listfavcats()
 {
-    const char *str = conc(&favcats[0], favcats.length(), true);
+    const char *str = conc((const char **)favcats.getbuf(), favcats.length(), true);
     result(str);
     delete [] str;
 }
@@ -834,16 +831,17 @@ bool favcatcheckkey(serverinfo &si, const char *key)
         case '$':
             if(key[1])
             {
-                formatstring(text)("%s \"%s\" %d %d, %d %d %d \"%s\" %d %d", key + 1, si.map, si.mode, si.ping, si.minremain, si.numplayers, si.maxclients, si.name, si.port, si.pongflags);
-                filtertext(text, text, 1);
+                formatstring(text)("%s \"%s\" %d %d %d %d %d \"%s\" %d %d", key + 1, si.map, si.mode, si.ping, si.minremain, si.numplayers, si.maxclients, si.name, si.port, si.pongflags);
+                filtertext(text, text, FTXT_NOCOLOR|FTXT_NOWHITE|FTXT_ALLOWBLANKS);
                 int cnt = 0;
                 for(const char *p = text; (p = strchr(p, '\"')); p++) cnt++;
                 return cnt == 4 && execute(text);
             }
             break;
         default:
-            filtertext(text, si.sdesc);
-            return *key && strstr(text, key);
+            filtertext(text, si.sdesc, FTXT_NOCOLOR|FTXT_NOWHITE|FTXT_ALLOWBLANKS|(showonlyfavourites > 0 ? FTXT_TOUPPER : 0));
+            strtoupper(keyuc, key);
+            return *key && strstr(text, showonlyfavourites > 0 ? keyuc : key);
     }
     return false;
 }
@@ -853,7 +851,7 @@ const char *favcatcheck(serverinfo &si, const char *ckeys, char *autokeys = NULL
     if(!ckeys) return NULL;
     static char *nkeys = NULL;
     const char *sep = " \t\n\r";
-    char *keys = newstring(ckeys), *k = strtok(keys, sep);
+    char *keys = newstring(ckeys), *b, *k = strtok_r(keys, sep, &b);
     bool res = false;
     DELETEA(nkeys);
     nkeys = newstring(strlen(ckeys));
@@ -870,12 +868,37 @@ const char *favcatcheck(serverinfo &si, const char *ckeys, char *autokeys = NULL
             if(*nkeys) strcat(nkeys, " ");
             strcat(nkeys, k);
         }
-        k = strtok(NULL, sep);
+        k = strtok_r(NULL, sep, &b);
     }
     delete[] keys;
     return res ? nkeys : NULL;
 }
 
+void serverbrowseralternativeviews(int shiftdirection)
+{
+    if(searchlan == 2) return;
+    const char *ckeys = getalias("serverbrowseraltviews");
+    if(!ckeys) return;
+    const char *sep = " \t\n\r";
+    char *keys = newstring(ckeys), *b, *k = strtok_r(keys, sep, &b);
+    vector<int> cats;
+    cats.add(0);
+    while(k)
+    {
+        loopv(favcats) if(!strcmp(favcats[i], k)) cats.add(i + 1);
+        k = strtok_r(NULL, sep, &b);
+    }
+    delete[] keys;
+    if(cats.length())
+    {
+        loopv(cats)
+        {
+           if(cats[i] == showonlyfavourites) { showonlyfavourites = cats[(i + shiftdirection + cats.length()) % cats.length()]; return; }
+        }
+    }
+    showonlyfavourites = 0;
+}
+
 vector<const char *> favcattags;
 
 bool assignserverfavourites()
@@ -890,7 +913,7 @@ bool assignserverfavourites()
         loopi(FC_NUM) { alx[i] = getalias(favcatargname(favcats[j], i)); alxn[i] = alx[i] ? atoi(alx[i]) : 0; }
         favcattags.add(alx[FC_TAG]);
         bool showonlythiscat = j == showonlyfavourites - 1;
-        char *keys = newstring(alx[FC_KEYS]), *k = strtok(keys, sep);
+        char *keys = newstring(alx[FC_KEYS]), *b, *k = strtok_r(keys, sep, &b);
         while(k)
         {
             loopv(servers)
@@ -909,10 +932,11 @@ bool assignserverfavourites()
                             if(!si.bgcolor) si.bgcolor = new color;
                             new (si.bgcolor) color(((float)alxn[FC_RED])/100, ((float)alxn[FC_GREEN])/100, ((float)alxn[FC_BLUE])/100, ((float)alxn[FC_ALPHA])/100);
                         }
+                        else DELETEP(si.bgcolor)
                     }
                 }
             }
-            k = strtok(NULL, sep);
+            k = strtok_r(NULL, sep, &b);
         }
         delete[] keys;
     }
@@ -960,7 +984,6 @@ void refreshservers(void *menu, bool init)
             loopv(servers) if(lastselectedserver == servers[i]) { found = true; break; }
             if(!found) lastselectedserver = NULL;
         }
-        menutitle(menu, "extended server information (F5: refresh)");
         menureset(menu);
         static string infotext;
         static char dummy = '\0';
@@ -968,8 +991,8 @@ void refreshservers(void *menu, bool init)
         {
             serverinfo &si = *lastselectedserver;
             formatstring(si.full)("%s:%d  %s", si.name, si.port, si.sdesc);
-            menumanual(menu, si.full);
-            menumanual(menu, &dummy);
+            menuitemmanual(menu, si.full);
+            menuitemmanual(menu, &dummy);
             if(si.infotexts.length() && !pinglastselected)
             {
                 infotext[0] = '\0';
@@ -985,7 +1008,7 @@ void refreshservers(void *menu, bool init)
         }
         else
             copystring(infotext, "  -- no server selected --");
-        if(*infotext) menumanual(menu, infotext);
+        if(*infotext) menuitemmanual(menu, infotext);
         return;
     }
     if((init && issearch) || totalmillis - lastinfo >= (servpingrate * (issearch ? 2 : 1))/(maxservpings ? max(1, (servers.length() + maxservpings - 1) / maxservpings) : 1))
@@ -1007,29 +1030,18 @@ void refreshservers(void *menu, bool init)
     servers.sort(sicompare);
     if(menu)
     {
-        static const char *titles[NUMSERVSORT] =
-        {
-            "%s\fs\f0ping\fr\tplr\tserver%s%s",                               // 0: ping
-            "%sping\t\fs\f0plr\fr\tserver%s%s",                               // 1: player number
-            "%sping\tplr\tserver (\fs\f0max players\fr)%s%s",                 // 2: maxplayers
-            "%sping\tplr\fs\f0\fr\tserver (\fs\f0minutes remaining\fr)%s%s",  // 3: minutes remaining
-            "%sping\tplr\tserver (\fs\f0map\fr)%s%s",                         // 4: map
-            "%sping\tplr\tserver (\fs\f0game mode\fr)%s%s",                   // 5: mode
-            "%sping\tplr\tserver (\fs\f0IP\fr)%s%s",                          // 6: IP
-            "%sping\tplr\tserver (\fs\f0description\fr)%s%s"                  // 7: description
-        };
         bool showmr = showminremain || serversort == SBS_MINREM;
-        formatstring(title)(titles[serversort], showfavtag ? "fav\t" : "", issearch ? "      search results for \f3" : "     (F1: Help/Settings)", issearch ? cursearch : "");
-        menutitle(menu, title);
         menureset(menu);
+        menuheader(menu, NULL, NULL);
         string text;
         int curnl = 0;
+        int allplayers = 0;
         bool sbconnectexists = identexists("sbconnect");
         loopv(servers)
         {
             serverinfo &si = *servers[i];
             si.menuline_to = si.menuline_from = ((gmenu *)menu)->items.length();
-            if( (!showallservers && si.lastpingmillis < servermenumillis) || (si.maxclients>MAXCL && searchlan<2) ) continue; // no pong yet or forbidden
+            if( (!showallservers && si.lastpingmillis <= servermenumillis) || (si.maxclients>MAXCL && searchlan<2) ) continue; // no pong yet or forbidden
             int banned = ((si.pongflags >> PONGFLAG_BANNED) & 1) | ((si.pongflags >> (PONGFLAG_BLACKLIST - 1)) & 2);
             bool showthisone = !(banned && showonlygoodservers) && !(showonlyfavourites > 0 && si.favcat != showonlyfavourites - 1);
             bool serverfull = si.numplayers >= si.maxclients;
@@ -1047,10 +1059,11 @@ void refreshservers(void *menu, bool init)
                 }
                 else
                 {
-                    if(!hidefavicons && showfavtag && si.favcat > -1) favimage = getalias(favcatargname(favcats[si.favcat], FC_IMAGE));
-                    filterrichtext(text, si.favcat > -1 && !favimage ? favcattags[si.favcat] : "");
+                    if(!hidefavicons && !showweights && showfavtag && si.favcat > -1) favimage = getalias(favcatargname(favcats[si.favcat], FC_IMAGE));
+                    copystring(text, si.favcat > -1 && !favimage ? favcattags[si.favcat] : "");
                     if(showweights) concatformatstring(text, "(%d)", si.weight);
-                    formatstring(si.full)(showfavtag ? (favimage ? "\t" : "\fs%s\fr\t") : "", text);
+                    if(showfavtag && !favimage) formatstring(si.full)("\fs%s\fr\t", text);
+                    else copystring(si.full, showfavtag ? "\t" : "");
                     concatformatstring(si.full, "\fs\f%c%s\t\fs\f%c%d/%d\fr\t\a%c  ", basecolor, colorping(si.ping), plnumcolor, si.numplayers, si.maxclients, '0' + si.uplinkqual);
                     if(si.map[0])
                     {
@@ -1058,7 +1071,8 @@ void refreshservers(void *menu, bool init)
                         if(showmr) concatformatstring(si.full, ", (%d)", si.minremain);
                     }
                     else concatformatstring(si.full, "empty");
-                    concatformatstring(si.full, serverbrowserhideip < 2 ? ": \fs%s%s:%d\fr" : ": ", serverbrowserhideip == 1 ? "\f4" : "", si.name, si.port);
+                    concatstring(si.full, ": ");
+                    if(serverbrowserhideip < 2) concatformatstring(si.full, "\fs%s%s:%d\fr", serverbrowserhideip == 1 ? "\f4" : "", si.name, si.port);
                     concatformatstring(si.full, "\fr %s", si.sdesc);
                 }
             }
@@ -1075,14 +1089,15 @@ void refreshservers(void *menu, bool init)
             }
             if(showthisone)
             {
-                cutcolorstring(si.full, 76); // cut off too long server descriptions
-                cutcolorstring(si.description, 76);
+                cutcolorstring(si.full, 105); // cut off too long server descriptions
+                cutcolorstring(si.description, 100);
                 if(sbconnectexists)
                 {
-                    filtertext(text, si.sdesc);
+                    filtertext(text, si.sdesc, FTXT_NOCOLOR|FTXT_NOWHITE|FTXT_ALLOWBLANKS);
                     for(char *p = text; (p = strchr(p, '\"')); *p++ = ' ');
-                    text[30] = '\0';
-                    formatstring(si.cmd)("sbconnect %s %d %d %d %d %d \"%s\"", si.name, si.port, serverfull ?1:0, needspasswd ?1:0, mmode, banned, text);
+                    formatstring(si.cmd)("sbconnect %s %d %d %d %d %d", si.name, si.port, serverfull ? 1 : 0, needspasswd ? 1 : 0, mmode, banned);
+                    text[max(int(MAXSTRLEN - strlen(si.cmd) - 10), 0)] = '\0';
+                    concatformatstring(si.cmd, " \"%s\"", text);
                 }
                 else formatstring(si.cmd)("connect %s %d", si.name, si.port);
                 menuimagemanual(menu, favimage, "serverquality", si.full, si.cmd, si.bgcolor, si.description);
@@ -1108,36 +1123,52 @@ void refreshservers(void *menu, bool init)
                         cur++;
                         if(cur == 4)
                         {
-                            menumanual(menu, t, NULL, NULL, NULL);
+                            menuitemmanual(menu, t, NULL, NULL, NULL);
                             cur = 0;
                         }
                     }
-                    if(cur) menumanual(menu, t, NULL, NULL, NULL);
+                    if(cur) menuitemmanual(menu, t, NULL, NULL, NULL);
                 }
             }
             si.menuline_to = ((gmenu *)menu)->items.length();
+            if(!(showonlyfavourites > 0 && (servers[i]->favcat != showonlyfavourites - 1))) allplayers += servers[i]->numplayers;
         }
-        static string notfoundmsg;
+
+        static const char *titles[NUMSERVSORT] =
+        {
+            "%s\fs\f0ping\fr\t%s plr\tserver%s%s",                               // 0: ping
+            "%sping\t\fs\f0%s plr\fr\tserver%s%s",                               // 1: player number
+            "%sping\t%s plr\tserver (\fs\f0max players\fr)%s%s",                 // 2: maxplayers
+            "%sping\t%s plr\fs\f0\fr\tserver (\fs\f0minutes remaining\fr)%s%s",  // 3: minutes remaining
+            "%sping\t%s plr\tserver (\fs\f0map\fr)%s%s",                         // 4: map
+            "%sping\t%s plr\tserver (\fs\f0game mode\fr)%s%s",                   // 5: mode
+            "%sping\t%s plr\tserver (\fs\f0IP\fr)%s%s",                          // 6: IP
+            "%sping\t%s plr\tserver (\fs\f0description\fr)%s%s"                  // 7: description
+        };
+        defformatstring(allplrs)("%d", allplayers);
+        formatstring(title)(titles[serversort], showfavtag ? "fav\t" : "", !issearch && showallplayersnumber ? allplrs : "", issearch ? "      search results for \f3" : "     (F1: Help/Settings)", issearch ? cursearch : "");
+        menutitlemanual(menu, title);
+
+        static string notfoundmsg, headermsg;
+        notfoundmsg[0] = '\0';
         if(issearch)
         {
-            if(curnl == 0)
-            {
-                formatstring(notfoundmsg)("\t\tpattern \fs\f3%s\fr not found.", cursearch);
-                menumanual(menu, notfoundmsg, NULL, NULL, NULL);
-            }
+            if(curnl == 0) formatstring(notfoundmsg)("\t\tpattern \fs\f3%s\fr not found.", cursearch);
         }
-        else if(!((gmenu *)menu)->items.length() && showonlyfavourites && favcats.inrange(showonlyfavourites - 1))
+        if(showonlyfavourites && favcats.inrange(showonlyfavourites - 1))
         {
             const char *desc = getalias(favcatargname(favcats[showonlyfavourites - 1], FC_DESC));
-            formatstring(notfoundmsg)("no servers in category \f2%s", desc ? desc : favcattags[showonlyfavourites - 1]);
-            menumanual(menu, notfoundmsg, NULL, NULL, NULL);
+            formatstring(headermsg)("Servers in category \f2%s", desc ? desc : favcattags[showonlyfavourites - 1]);
+            menuheader(menu, headermsg, NULL);
+            if(!((gmenu *)menu)->items.length() && !issearch) formatstring(notfoundmsg)("\t(no servers in this category)");
         }
+        if(*notfoundmsg) menuitemmanual(menu, notfoundmsg, NULL, NULL, NULL);
     }
 }
 
-bool serverskey(void *menu, int code, bool isdown, int unicode)
+bool serverskey(void *menu, int code, bool isdown)
 {
-    const int fk[] = { SDLK_1, SDLK_2, SDLK_3, SDLK_4, SDLK_5, SDLK_6, SDLK_7, SDLK_8, SDLK_9, SDLK_0 };
+    int fk[] = { SDLK_1, SDLK_2, SDLK_3, SDLK_4, SDLK_5, SDLK_6, SDLK_7, SDLK_8, SDLK_9, SDLK_0 }, gogetem = 1;
     if(!isdown) return false;
     loopi(sizeof(fk)/sizeof(fk[0])) if(code == fk[i] && favcats.inrange(i))
     {
@@ -1149,11 +1180,11 @@ bool serverskey(void *menu, int code, bool isdown, int unicode)
             if(!desc) desc = "";
             if(*ak)
             { // server was automatically added to this favourite group, don't remove
-                conoutf(_("server \"%cs%s%cr\" is in category '%cs%s%cr' because of key '%s', please remove manually"), CC, servers[j]->sdesc, CC, CC, desc, CC, ak);
+                conoutf("server \"\fs%s\fr\" is in category '\fs%s\fr' because of key '%s', please remove manually", servers[j]->sdesc, desc, ak);
             }
             else if(rest)
             { // remove from favourite group
-                conoutf(_("removing server \"%cs%s%cr\" from favourites category '%cs%s%cr' (rest '%s')"), CC, servers[j]->sdesc, CC, CC, desc, CC, rest);
+                conoutf("removing server \"\fs%s\fr\" from favourites category '\fs%s\fr' (rest '%s')", servers[j]->sdesc, desc, rest);
                 alias(keyalias, rest);
             }
             else
@@ -1168,17 +1199,13 @@ bool serverskey(void *menu, int code, bool isdown, int unicode)
                     delete[] newkey;
                 }
                 else alias(keyalias, text);
-                conoutf(_("adding server \"%cs%s%cr\" to favourites category '%cs%s%cr' (new '%s')"), CC, servers[j]->sdesc, CC, CC, desc, CC, getalias(keyalias));
+                conoutf("adding server \"\fs%s\fr\" to favourites category '\fs%s\fr' (new '%s')", servers[j]->sdesc, desc, getalias(keyalias));
             }
             return true;
         }
     }
     switch(code)
     {
-        case SDLK_HOME:
-            if(menu) ((gmenu *)menu)->menusel = 0;
-            return true;
-
         case SDLK_LEFT:
             serversort = (serversort+NUMSERVSORT-1) % NUMSERVSORT;
             return true;
@@ -1187,8 +1214,16 @@ bool serverskey(void *menu, int code, bool isdown, int unicode)
             serversort = (serversort+1) % NUMSERVSORT;
             return true;
 
+        case SDLK_LEFTBRACKET:
+            serverbrowseralternativeviews(-1);
+            return true;
+
+        case SDLK_RIGHTBRACKET:
+            serverbrowseralternativeviews(1);
+            return true;
+
         case SDLK_F5:
-            updatefrommaster(1);
+            updatefrommaster(&gogetem);
             return true;
 
         case SDLK_F6:
@@ -1229,15 +1264,11 @@ bool serverskey(void *menu, int code, bool isdown, int unicode)
     return false;
 }
 
-bool serverinfokey(void *menu, int code, bool isdown, int unicode)
+bool serverinfokey(void *menu, int code, bool isdown)
 {
     if(!isdown) return false;
     switch(code)
     {
-        case SDLK_HOME:
-            if(menu) ((gmenu *)menu)->menusel = 0;
-            return true;
-
         case SDLK_F5:
             if(lastselectedserver) lastselectedserver->getinfo = EXTPING_SERVERINFO;
             pinglastselected = true;
@@ -1256,6 +1287,11 @@ bool serverinfokey(void *menu, int code, bool isdown, int unicode)
             if(lastselectedserver) lastselectedserver->getinfo = EXTPING_MAPROT;
             pinglastselected = true;
             break;
+
+        case SDLK_c:
+            lastselectedserver = getconnectedserverinfo();
+            pinglastselected = true;
+            break;
     }
     return false;
 }
@@ -1267,87 +1303,45 @@ void clearservers()
 }
 
 #define RETRIEVELIMIT 5000
+
 extern char *global_name;
 bool cllock = false, clfail = false;
 
-struct resolver_data
+int progress_callback_retrieveservers(void *data, float progress)
 {
-    int timeout, starttime;
-    string text;
-};
-
-static int progress_callback(void *clientp, double dltotal, double dlnow, double ultotal, double ulnow)
-{
-    resolver_data *rd = (resolver_data *)clientp;
-    rd->timeout = SDL_GetTicks() - rd->starttime;
-    show_out_of_renderloop_progress(min(float(rd->timeout)/RETRIEVELIMIT, 1.0f), rd->text);
-    if(interceptkey(SDLK_ESCAPE))
-    {
-        loadingscreen();
-        return 1;
-    }
-    return 0;
+    if(progress < 0) show_out_of_renderloop_progress(progress + 1.0f, "waiting for response (esc to abort)");
+    else show_out_of_renderloop_progress(min(progress, 1.0f), (const char *)data);
+    return interceptkey(SDLK_ESCAPE) ? 1 : 0;
 }
 
-static size_t write_callback(void *ptr, size_t size, size_t nmemb, FILE *stream)
-{
-    return fwrite(ptr, size, nmemb, stream);
-}
+VARP(mastertype, 0, 1, 1); // 0: TCP direct, 1: HTTP proxy
 
 void retrieveservers(vector<char> &data)
 {
-    if(mastertype == AC_MASTER_HTTP)
+    if(mastertype)
     {
-        string request;
-        sprintf(request, "http://%s/retrieve.do?action=list&name=%s&version=%d&build=%d", mastername, global_name, AC_VERSION, getbuildtype()|(1<<16));
-
-        const char *tmpname = findfile(path("config/servers.cfg", true), "wb");
-        FILE *outfile = fopen(tmpname, "w+");
-        if(!outfile)
+        httpget h;
+        defformatstring(progresstext)("resolving %s", mastername);
+        h.callbackfunc = progress_callback_retrieveservers;
+        h.callbackdata = progresstext;
+        show_out_of_renderloop_progress(0.01f, progresstext);
+        if(h.set_host(mastername))
         {
-            conoutf("\f3cannot write server list");
-            return;
+            formatstring(progresstext)("retrieving servers from %s:%d... (esc to abort)", mastername, masterport);
+            defformatstring(url)("/retrieve.do?action=list&name=%s&version=%d&build=%d", urlencode(global_name, true), AC_VERSION, getbuildtype()|(1<<16));
+            h.outvec = (vector<uchar> *) &data; // ouch...
+            show_out_of_renderloop_progress(0, progresstext);
+            int got = h.get(url, RETRIEVELIMIT, RETRIEVELIMIT);
+            if(got < 0 || h.response != 200) data.setsize(0);
+            h.outvec = NULL; // must not be cleaned up by httpget
+            if(data.length()) data.add('\0');
+            clfail = false;
         }
-
-        resolver_data *rd = new resolver_data();
-        formatstring(rd->text)("retrieving servers from %s:%d... (esc to abort)", mastername, masterport);
-        show_out_of_renderloop_progress(0, rd->text);
-
-        rd->starttime = SDL_GetTicks();
-        rd->timeout = 0;
-
-        CURL *curl = curl_easy_init();
-        int result = 0, httpresult = 0;
-
-        curl_easy_setopt(curl, CURLOPT_URL, request);
-	curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);	// Fixes crashbug for some buggy libcurl versions (Linux)
-        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_callback);
-        curl_easy_setopt(curl, CURLOPT_WRITEDATA, outfile);
-        curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0);
-        curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, progress_callback);
-        curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, rd);
-        curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, RETRIEVELIMIT/1000);
-        result = curl_easy_perform(curl);
-        curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpresult);
-        curl_easy_cleanup(curl);
-        curl = NULL;
-        if(outfile) fclose(outfile);
-
-        if(result == CURLE_OPERATION_TIMEDOUT || result == CURLE_COULDNT_RESOLVE_HOST)
+        else
         {
+            conoutf("failed to resolve host %s", mastername);
             clfail = true;
         }
-        else clfail = false;
-
-        if(!result && httpresult == 200)
-        {
-            int size = 0;
-            char *content = loadfile(path("config/servers.cfg", true), &size);
-            data.shrink(0);
-            data.insert(0, content, size);
-            if(data.length()) data.add('\0');
-        }
-        DELETEP(rd);
     }
     else
     {
@@ -1362,8 +1356,7 @@ void retrieveservers(vector<char> &data)
         defformatstring(text)("retrieving servers from %s:%d... (esc to abort)", mastername, masterport);
         show_out_of_renderloop_progress(0, text);
         int starttime = SDL_GetTicks(), timeout = 0;
-        string request;
-        sprintf(request, "list %s %d %d\n",global_name,AC_VERSION,getbuildtype());
+        defformatstring(request)("list %s %d %d\n", global_name, AC_VERSION, getbuildtype());
         const char *req = request;
         int reqlen = strlen(req);
         ENetBuffer buf;
@@ -1403,17 +1396,18 @@ void retrieveservers(vector<char> &data)
             if(timeout > RETRIEVELIMIT) break;
         }
         if(data.length()) data.add('\0');
-        enet_socket_destroy(sock); 
+        enet_socket_destroy(sock);
     }
 }
 
 VARP(masterupdatefrequency, 1, 60*60, 24*60*60);
+VAR(msctrl, 1, 1, INT_MAX);
 
-void updatefrommaster(int force)
+void updatefrommaster(int *force)
 {
     static int lastupdate = 0;
     if(lastupdate==0) cllock = true;
-    if(!force && lastupdate && totalmillis-lastupdate<masterupdatefrequency*1000) return;
+    if(!*force && lastupdate && totalmillis-lastupdate<masterupdatefrequency*1000) return;
 
     vector<char> data;
     retrieveservers(data);
@@ -1443,7 +1437,7 @@ void updatefrommaster(int force)
 
 COMMANDF(addserver, "sii", (char *n, int *p, int *w) { addserver(n, *p, *w); });
 COMMAND(clearservers, "");
-COMMANDF(updatefrommaster, "i", (int *f) { updatefrommaster(*f); });
+COMMAND(updatefrommaster, "i");
 
 void writeservercfg()
 {
