@@ -9,6 +9,8 @@
 #include <dirent.h>
 #endif
 
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+
 string homedir = "";
 vector<char *> packagedirs;
 
@@ -33,44 +35,52 @@ char *makerelpath(const char *dir, const char *file, const char *prefix, const c
     return tmp;
 }
 
-
 char *path(char *s)
 {
-    for(char *curpart = s;;)
+    char *c = s;
+    // skip "<decal>"
+    if(c[0] == '<')
     {
-        char *endpart = strchr(curpart, '&');
-        if(endpart) *endpart = '\0';
-        if(curpart[0]=='<')
-        {
-            char *file = strrchr(curpart, '>');
-            if(!file) return s;
-            curpart = file+1;
-        }
-        for(char *t = curpart; (t = strpbrk(t, "/\\")); *t++ = PATHDIV);
-        for(char *prevdir = NULL, *curdir = s;;)
-        {
-            prevdir = curdir[0]==PATHDIV ? curdir+1 : curdir;
-            curdir = strchr(prevdir, PATHDIV);
-            if(!curdir) break;
-            if(prevdir+1==curdir && prevdir[0]=='.')
-            {
-                memmove(prevdir, curdir+1, strlen(curdir+1)+1);
-                curdir = prevdir;
-            }
-            else if(curdir[1]=='.' && curdir[2]=='.' && curdir[3]==PATHDIV)
-            {
-                if(prevdir+2==curdir && prevdir[0]=='.' && prevdir[1]=='.') continue;
-                memmove(prevdir, curdir+4, strlen(curdir+4)+1);
-                curdir = prevdir;
-            }
-        }
-        if(endpart)
-        {
-            *endpart = '&';
-            curpart = endpart+1;
-        }
-        else break;
+        char *enddecal = strrchr(c, '>');
+        if(!enddecal) return s;
+        c = enddecal + 1;
     }
+    // substitute with single, proper path delimiters
+    for(char *t = c; (t = strpbrk(t, "/\\")); )
+    {
+        *t++ = PATHDIV;
+        size_t d = strspn(t, "/\\");
+        if(d) memmove(t, t + d, strlen(t + d) + 1); // remove multiple path delimiters
+    }
+    // collapse ".."-parts
+    for(char *prevdir = NULL, *curdir = s;;)
+    {
+        prevdir = curdir[0] == PATHDIV ? curdir + 1 : curdir;
+        curdir = strchr(prevdir, PATHDIV);
+        if(!curdir) break;
+        if(prevdir + 1 == curdir && prevdir[0]=='.')
+        { // simply remove "./"
+            memmove(prevdir, curdir + 1, strlen(curdir));
+            curdir = prevdir;
+        }
+        else if(curdir[1] == '.' && curdir[2] == '.' && curdir[3] == PATHDIV)
+        { // collapse "/foo/../" to "/"
+            if(prevdir + 2 == curdir && prevdir[0] == '.' && prevdir[1] == '.') continue; // foo is also ".." -> skip
+            memmove(prevdir, curdir + 4, strlen(curdir + 3));
+            if(prevdir >= c + 2 && prevdir[-1] == PATHDIV)
+            {
+                prevdir -= 2;
+                while(prevdir > c && prevdir[-1] != PATHDIV) --prevdir;
+            }
+            curdir = prevdir;
+        }
+    }
+    return s;
+}
+
+char *unixpath(char *s)
+{
+    for(char *t = s; (t = strchr(t, '\\')); *t++ = '/');
     return s;
 }
 
@@ -186,59 +196,148 @@ void sethomedir(const char *dir)
     }
 #endif
 
-#ifndef STANDALONE
-    clientlogf("Using home directory: %s", tmpdir);
-#endif
-
     if(fixpackagedir(tmpdir) > 0)
     {
+#ifndef STANDALONE
+        clientlogf("Using home directory: %s", tmpdir);
+#endif
         copystring(homedir, tmpdir);
         createdir(homedir);
     }
 }
 
+bool havehomedir()
+{
+    return homedir[0] != '\0';
+}
+
 void addpackagedir(const char *dir)
 {
-#ifndef STANDALONE
-    clientlogf("Adding package directory: %s", dir);
-#endif
-
     string pdir;
     copystring(pdir, dir);
-    if(fixpackagedir(pdir) > 0) packagedirs.add(newstring(pdir));
+    if(fixpackagedir(pdir) > 0)
+    {
+#ifndef STANDALONE
+        clientlogf("Adding package directory: %s", pdir);
+#endif
+        packagedirs.add(newstring(pdir));
+    }
 }
 
+int findfilelocation;
+
 const char *findfile(const char *filename, const char *mode)
 {
+    while(filename[0] == PATHDIV) filename++; // skip leading pathdiv
+    while(!strncmp(".." PATHDIVS, filename, 3)) filename += 3; // skip leading "../" (don't allow access to files below "AC root dir")
     static string s;
-    if(homedir[0])
-    {
-        formatstring(s)("%s%s", homedir, filename);
-        if(fileexists(s, mode)) return s;
-        if(mode[0]=='w' || mode[0]=='a')
+    formatstring(s)("%s%s", homedir, filename);         // homedir may be ""
+    findfilelocation = FFL_HOME;
+    if(homedir[0] && fileexists(s, mode)) return s;
+    if(mode[0]=='w' || mode[0]=='a')
+    { // create missing directories, if necessary
+        string dirs;
+        copystring(dirs, s);
+        char *dir = strchr(dirs[0]==PATHDIV ? dirs+1 : dirs, PATHDIV);
+        while(dir)
         {
-            string dirs;
-            copystring(dirs, s);
-            char *dir = strchr(dirs[0]==PATHDIV ? dirs+1 : dirs, PATHDIV);
-            while(dir)
-            {
-                *dir = '\0';
-                if(!fileexists(dirs, "r") && !createdir(dirs)) return s;
-                *dir = PATHDIV;
-                dir = strchr(dir+1, PATHDIV);
-            }
-            return s;
+            *dir = '\0';
+            if(!fileexists(dirs, "r") && !createdir(dirs)) return s;
+            *dir = PATHDIV;
+            dir = strchr(dir+1, PATHDIV);
         }
+        return s;
     }
-    if(mode[0]=='w' || mode[0]=='a') return filename;
+    findfilelocation = FFL_ZIP;
+#ifndef STANDALONE
+    formatstring(s)("zip://%s", filename);
+    if(findzipfile(filename)) return s;
+#endif
     loopv(packagedirs)
     {
+        findfilelocation++;
         formatstring(s)("%s%s", packagedirs[i], filename);
         if(fileexists(s, mode)) return s;
     }
+    findfilelocation = FFL_WORKDIR;
     return filename;
 }
 
+const char *stream_capabilities()
+{
+    #if !defined(WIN32) && !defined(_DIRENT_HAVE_D_TYPE)
+    return "no support for d_type, listing directories may be slow";
+    #else
+    return "";
+    #endif
+}
+
+bool listsubdir(const char *dir, vector<char *> &subdirs)
+{
+    #if defined(WIN32)
+    defformatstring(pathname)("%s\\*", dir);
+    WIN32_FIND_DATA FindFileData;
+    HANDLE Find = FindFirstFile(path(pathname), &FindFileData);
+    if(Find != INVALID_HANDLE_VALUE)
+    {
+        do {
+            if((FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) && FindFileData.cFileName[0] != '.') subdirs.add(newstring(FindFileData.cFileName));
+        } while(FindNextFile(Find, &FindFileData));
+        FindClose(Find);
+        return true;
+    }
+    #else
+    string pathname;
+    copystring(pathname, dir);
+    DIR *d = opendir(path(pathname));
+    if(d)
+    {
+        struct dirent *de, b;
+        while(!readdir_r(d, &b, &de) && de != NULL)
+        {
+        #ifdef _DIRENT_HAVE_D_TYPE
+            if(de->d_type == DT_DIR && de->d_name[0] != '.') subdirs.add(newstring(de->d_name));
+            else if(de->d_type == DT_UNKNOWN && de->d_name[0] != '.')
+        #endif
+            {
+                struct stat s;
+                int dl = (int)strlen(pathname);
+                concatformatstring(pathname, "/%s", de->d_name);
+                if(!lstat(pathname, &s) && S_ISDIR(s.st_mode) && de->d_name[0] != '.') subdirs.add(newstring(de->d_name));
+                pathname[dl] = '\0';
+            }
+        }
+        closedir(d);
+        return true;
+    }
+    #endif
+    else return false;
+}
+
+void listsubdirs(const char *dir, vector<char *> &subdirs, int (__cdecl *sf)(const char **, const char **))
+{
+    listsubdir(dir, subdirs);
+    string s;
+    if(homedir[0])
+    {
+        formatstring(s)("%s%s", homedir, dir);
+        listsubdir(s, subdirs);
+    }
+    loopv(packagedirs)
+    {
+        formatstring(s)("%s%s", packagedirs[i], dir);
+        listsubdir(s, subdirs);
+    }
+#ifndef STANDALONE
+    listzipdirs(dir, subdirs);
+#endif
+    subdirs.sort(sf);
+    for(int i = subdirs.length() - 1; i > 0; i--)
+    { // remove doubles
+        if(!strcmp(subdirs[i], subdirs[i - 1])) delstring(subdirs.remove(i));
+    }
+}
+
 bool listdir(const char *dir, const char *ext, vector<char *> &files)
 {
     int extsize = ext ? (int)strlen(ext)+1 : 0;
@@ -249,8 +348,10 @@ bool listdir(const char *dir, const char *ext, vector<char *> &files)
     if(Find != INVALID_HANDLE_VALUE)
     {
         do {
-            files.add(newstring(FindFileData.cFileName, (int)strlen(FindFileData.cFileName) - extsize));
+            if(!(FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
+                files.add(newstring(FindFileData.cFileName, (int)strlen(FindFileData.cFileName) - extsize));
         } while(FindNextFile(Find, &FindFileData));
+        FindClose(Find);
         return true;
     }
     #else
@@ -259,15 +360,30 @@ bool listdir(const char *dir, const char *ext, vector<char *> &files)
     DIR *d = opendir(path(pathname));
     if(d)
     {
-        struct dirent *de;
-        while((de = readdir(d)) != NULL)
+        struct dirent *de, b;
+        while(!readdir_r(d, &b, &de) && de != NULL)
         {
-            if(!ext) files.add(newstring(de->d_name));
-            else
+            bool isreg = false;
+        #ifdef _DIRENT_HAVE_D_TYPE
+            if(de->d_type == DT_REG) isreg = true;
+            else if(de->d_type == DT_UNKNOWN)
+        #endif
             {
-                int namelength = (int)strlen(de->d_name) - extsize;
-                if(namelength > 0 && de->d_name[namelength] == '.' && strncmp(de->d_name+namelength+1, ext, extsize-1)==0)
-                    files.add(newstring(de->d_name, namelength));
+                struct stat s;
+                int dl = (int)strlen(pathname);
+                concatformatstring(pathname, "/%s", de->d_name);
+                isreg = !lstat(pathname, &s) && S_ISREG(s.st_mode);
+                pathname[dl] = '\0';
+            }
+            if(isreg)
+            {
+                if(!ext) files.add(newstring(de->d_name));
+                else
+                {
+                    int namelength = (int)strlen(de->d_name) - extsize;
+                    if(namelength > 0 && de->d_name[namelength] == '.' && strncmp(de->d_name+namelength+1, ext, extsize-1)==0)
+                        files.add(newstring(de->d_name, namelength));
+                }
             }
         }
         closedir(d);
@@ -277,47 +393,103 @@ bool listdir(const char *dir, const char *ext, vector<char *> &files)
     else return false;
 }
 
-int listfiles(const char *dir, const char *ext, vector<char *> &files)
+void listfiles(const char *dir, const char *ext, vector<char *> &files, int (__cdecl *sf)(const char **, const char **))
 {
-    int dirs = 0;
-    if(listdir(dir, ext, files)) dirs++;
+    listdir(dir, ext, files);
     string s;
     if(homedir[0])
     {
         formatstring(s)("%s%s", homedir, dir);
-        if(listdir(s, ext, files)) dirs++;
+        listdir(s, ext, files);
     }
     loopv(packagedirs)
     {
         formatstring(s)("%s%s", packagedirs[i], dir);
-        if(listdir(s, ext, files)) dirs++;
+        listdir(s, ext, files);
     }
 #ifndef STANDALONE
-    dirs += listzipfiles(dir, ext, files);
+    listzipfiles(dir, ext, files);
 #endif
-    return dirs;
+    if(sf)
+    { // sort and remove doubles
+        files.sort(sf);
+        for(int i = files.length() - 1; i > 0; i--)
+        {
+            if(!strcmp(files[i], files[i - 1])) delstring(files.remove(i));
+        }
+    }
 }
 
+#ifndef STANDALONE
+void listfilesrecursive(const char *dir, vector<char *> &files, int level)
+{
+    if(level > 8) return; // 8 levels is insane enough...
+    vector<char *> dirs, thisdir;
+    listsubdirs(dir, dirs, stringsort);
+    loopv(dirs)
+    {
+        if(dirs[i][0] != '.')  // ignore "." and ".." (and also other directories starting with '.', like it is unix-convention - and doesn't hurt on windows)
+        {
+            defformatstring(name)("%s/%s", dir, dirs[i]);
+            listfilesrecursive(name, files, level + 1);
+        }
+        delstring(dirs[i]);
+    }
+    listfiles(dir, NULL, thisdir);
+    loopv(thisdir)
+    {
+        defformatstring(name)("%s/%s", dir, thisdir[i]);
+        files.add(newstring(name));
+        delstring(thisdir[i]);
+    }
+}
+
+void listdirsrecursive(const char *dir, vector<char *> &subdirs, int level)
+{
+    if(level > 8) return; // 8 levels is insane enough...
+    vector<char *> dirs;
+    listsubdirs(dir, dirs, stringsort);
+    loopv(dirs)
+    {
+        if(dirs[i][0] != '.')  // ignore "." and ".." (and also other directories starting with '.', like it is unix-convention - and doesn't hurt on windows)
+        {
+            defformatstring(name)("%s/%s", dir, dirs[i]);
+            subdirs.add(newstring(name));
+            listdirsrecursive(name, subdirs, level + 1);
+        }
+        delstring(dirs[i]);
+    }
+}
+#endif
+
 bool delfile(const char *path)
 {
-    return !remove(path);
+    return strncmp(path, "zip://", 6) ? !remove(path) : false;
+}
+
+void backup(char *name, char *backupname)
+{
+    string backupfile;
+    copystring(backupfile, findfile(backupname, "wb"));
+    remove(backupfile);
+    rename(findfile(name, "wb"), backupfile);
 }
 
 #ifndef STANDALONE
-static int rwopsseek(SDL_RWops *rw, int offset, int whence)
+static Sint64 rwopsseek(SDL_RWops *rw, Sint64 offset, int whence)
 {
     stream *f = (stream *)rw->hidden.unknown.data1;
     if((!offset && whence==SEEK_CUR) || f->seek(offset, whence)) return f->tell();
     return -1;
 }
 
-static int rwopsread(SDL_RWops *rw, void *buf, int size, int nmemb)
+static size_t rwopsread(SDL_RWops *rw, void *buf, size_t size, size_t nmemb)
 {
     stream *f = (stream *)rw->hidden.unknown.data1;
     return f->read(buf, size*nmemb)/size;
 }
 
-static int rwopswrite(SDL_RWops *rw, const void *buf, int size, int nmemb)
+static size_t rwopswrite(SDL_RWops *rw, const void *buf, size_t size, size_t nmemb)
 {
     stream *f = (stream *)rw->hidden.unknown.data1;
     return f->write(buf, size*nmemb)/size;
@@ -360,9 +532,9 @@ bool stream::getline(char *str, int len)
     return true;
 }
 
-#ifdef __linux__
+#ifndef WIN32
 #include <sys/statvfs.h>
-#define MINFSSIZE 50000000              // 50MB
+const int64_t MINFSSIZE = 50000000;         // 50MB
 #endif
 
 struct filestream : stream
@@ -376,12 +548,12 @@ struct filestream : stream
     {
         if(file) return false;
         file = fopen(name, mode);
-#ifdef __linux__
+#ifndef WIN32
         struct statvfs buf;
         if(file && strchr(mode,'w'))
         {
             int fail = fstatvfs(fileno(file), &buf);
-            if (fail || buf.f_frsize * buf.f_bavail < MINFSSIZE)
+            if (fail || (int64_t)buf.f_frsize * (int64_t)buf.f_bavail < MINFSSIZE)
             {
                 close();
                 return false;
@@ -410,6 +582,7 @@ struct filestream : stream
     bool end() { return feof(file)!=0; }
     long tell() { return ftell(file); }
     bool seek(long offset, int whence) { return fseek(file, offset, whence) >= 0; }
+    void fflush() { if(file) ::fflush(file); }
     int read(void *buf, int len) { return (int)fread(buf, 1, len, file); }
     int write(const void *buf, int len) { return (int)fwrite(buf, 1, len, file); }
     int getchar() { return fgetc(file); }
@@ -428,7 +601,8 @@ struct filestream : stream
 };
 
 #ifndef STANDALONE
-VAR(dbggz, 0, 0, 1);
+//VAR(dbggz, 0, 0, 1);
+const int dbggz = 0;
 #endif
 
 struct gzstream : stream
@@ -549,7 +723,7 @@ struct gzstream : stream
 
         if(reading)
         {
-            if(!checkheader()) { stopreading(); return false; }
+            if(!checkheader()) { stopreading(); file = NULL; return false; }
         }
         else if(writing) writeheader();
         return true;
@@ -569,7 +743,7 @@ struct gzstream : stream
             if(checkcrc != crc)
                 conoutf("gzip crc check failed: read %X, calculated %X", checkcrc, crc);
             if(checksize != zfile.total_out)
-                conoutf("gzip size check failed: read %d, calculated %d", checksize, zfile.total_out);
+                conoutf("gzip size check failed: read %u, calculated %u", checksize, (uint) zfile.total_out);
         }
 #endif
     }
@@ -593,8 +767,8 @@ struct gzstream : stream
         }
         uchar trailer[8] =
         {
-            crc&0xFF, (crc>>8)&0xFF, (crc>>16)&0xFF, (crc>>24)&0xFF,
-            zfile.total_in&0xFF, (zfile.total_in>>8)&0xFF, (zfile.total_in>>16)&0xFF, (zfile.total_in>>24)&0xFF
+            uchar(crc&0xFF), uchar((crc>>8)&0xFF), uchar((crc>>16)&0xFF), uchar((crc>>24)&0xFF),
+            uchar(zfile.total_in&0xFF), uchar((zfile.total_in>>8)&0xFF), uchar((zfile.total_in>>16)&0xFF), uchar((zfile.total_in>>24)&0xFF)
         };
         file->write(trailer, sizeof(trailer));
     }
@@ -621,9 +795,15 @@ struct gzstream : stream
 
     bool seek(long offset, int whence)
     {
-        if(writing || !reading || whence == SEEK_END) return false;
+        if(writing || !reading) return false;
 
-        if(whence == SEEK_CUR) offset += zfile.total_out;
+        if(whence == SEEK_END)
+        {
+            uchar skip[512];
+            while(read(skip, sizeof(skip)) == sizeof(skip));
+            return !offset;
+        }
+        else if(whence == SEEK_CUR) offset += zfile.total_out;
 
         if(offset >= (int)zfile.total_out) offset -= zfile.total_out;
         else if(offset < 0 || !file->seek(headersize, SEEK_SET)) return false;
@@ -702,16 +882,148 @@ struct gzstream : stream
     }
 };
 
+struct vecstream : stream
+{
+    vector<uchar> *data;
+    int pointer;
+    bool autodelete;
+
+    vecstream(vector<uchar> *s, bool autodelete) : data(s), pointer(0), autodelete(autodelete) {}
+    ~vecstream() { if(autodelete)  { DELETEP(data); } }
+
+    void close() { DELETEP(data); }
+    bool end() { return data ? pointer >= data->length() : true; }
+    long tell() { return data ? pointer : -1; }
+    long size() { return data ? data->length() : -1; }
+
+    bool seek(long offset, int whence)
+    {
+        int newpointer = -1;
+        if(data) switch(whence)
+        {
+            case SEEK_SET: newpointer = 0; break;
+            case SEEK_CUR: newpointer = pointer; break;
+            case SEEK_END: newpointer = data->length(); break;
+        }
+        if(newpointer >= 0) newpointer += offset;
+        if(newpointer >= 0 && data && newpointer <= data->length())
+        {
+            pointer = newpointer;
+            return true;
+        }
+        return false;
+    }
+
+    int read(void *buf, int len)
+    {
+        int got = 0;
+        if(data && data->inrange(pointer))
+        {
+            got = min(len, data->length() - pointer);
+            memcpy(buf, data->getbuf() + pointer, got);
+            pointer += got;
+        }
+        return got;
+    }
+
+    int write(const void *buf, int len)
+    {
+        if(data)
+        {
+            while(data->length() < pointer + len) data->add(0);
+            memcpy(data->getbuf() + pointer, buf, len);
+            pointer += len;
+        }
+        else len = 0;
+        return len;
+    }
+
+    int printf(const char *fmt, ...) // limited to MAXSTRLEN
+    {
+        int len = 0;
+        if(data)
+        {
+            defvformatstring(temp, fmt, fmt);
+            len = strlen(temp);
+            if(len) data->put((uchar *)temp, len);
+        }
+        return len;
+    }
+};
+
+struct memstream : stream
+{
+    const uchar *data;
+    int memsize;
+    int pointer;
+    int *refcount;
+
+    memstream(const uchar *s, int size, int *refcnt) : data(s), memsize(size), pointer(0), refcount(refcnt) { if(refcnt) (*refcnt)++; }
+    ~memstream() { close(); }
+
+    void close()
+    {
+        if(data && refcount)
+        {
+            (*refcount)--;
+            data = NULL;
+        }
+        else DELETEA(data);
+        memsize = -1;
+    }
+    bool end() { return data ? pointer >= memsize : true; }
+    long tell() { return data ? pointer : -1; }
+    long size() { return data ? memsize : -1; }
+
+    bool seek(long offset, int whence)
+    {
+        int newpointer = -1;
+        if(data) switch(whence)
+        {
+            case SEEK_SET: newpointer = 0; break;
+            case SEEK_CUR: newpointer = pointer; break;
+            case SEEK_END: newpointer = memsize; break;
+        }
+        if(newpointer >= 0) newpointer += offset;
+        if(newpointer >= 0 && newpointer <= memsize)
+        {
+            pointer = newpointer;
+            return true;
+        }
+        return false;
+    }
+
+    int read(void *buf, int len)
+    {
+        int got = 0;
+        if(data && pointer >= 0 && pointer < memsize)
+        {
+            got = min(len, memsize - pointer);
+            memcpy(buf, data + pointer, got);
+            pointer += got;
+        }
+        return got;
+    }
+};
+
+stream *openvecfile(vector<uchar> *s, bool autodelete)
+{
+    return new vecstream(s ? s : new vector<uchar>, autodelete);
+}
+
+stream *openmemfile(const uchar *buf, int size, int *refcnt)
+{
+    return new memstream(buf, size, refcnt);
+}
 
 stream *openrawfile(const char *filename, const char *mode)
 {
-    const char *found = findfile(filename, mode);
 #ifndef STANDALONE
-    if(mode && (mode[0]=='w' || mode[0]=='a')) conoutf("writing to file: %s", found);
+    if(mode && (mode[0]=='w' || mode[0]=='a')) conoutf("writing to file: %s", filename);
 #endif
-    if(!found) return NULL;
+    if(!strncmp(filename, "zip://", 6)) return NULL;
     filestream *file = new filestream;
-    if(!file->open(found, mode))
+    if(!file->open(filename, mode))
     {
 #ifndef STANDALONE
 //         conoutf("file failure! %s",filename);
@@ -723,11 +1035,11 @@ stream *openrawfile(const char *filename, const char *mode)
 
 stream *openfile(const char *filename, const char *mode)
 {
+    const char *found = findfile(filename, mode);
 #ifndef STANDALONE
-    stream *s = openzipfile(filename, mode);
-    if(s) return s;
+    if(!strncmp(found, "zip://", 6)) return openzipfile(found + 6, mode);
 #endif
-    return openrawfile(filename, mode);
+    return openrawfile(found, mode);
 }
 
 int getfilesize(const char *filename)
@@ -752,7 +1064,7 @@ stream *opengzfile(const char *filename, const char *mode, stream *file, int lev
     stream *source = file ? file : openfile(filename, mode);
     if(!source) return NULL;
     gzstream *gz = new gzstream;
-    if(!gz->open(source, mode, !file, level)) { if(!file) delete source; return NULL; }
+    if(!gz->open(source, mode, !file, level)) { if(!file) delete source; delete gz; return NULL; }
     return gz;
 }
 
@@ -776,3 +1088,28 @@ char *loadfile(const char *fn, int *size, const char *mode)
     return buf;
 }
 
+int streamcopy(stream *dest, stream *source, int maxlen)
+{
+    int got = 0, len;
+    uchar copybuf[1024];
+    while(got < maxlen && (len = source->read(copybuf, 1024))) got += dest->write(copybuf, len);
+    return got;
+}
+
+#ifndef STANDALONE
+void filerotate(const char *basename, const char *ext, int keepold, const char *oldformat)  // rotate old logfiles
+{
+    char fname1[MAXSTRLEN * 2], fname2[MAXSTRLEN] = "";
+    copystring(fname1, findfile(basename, "w"));
+    int mid = strlen(fname1);
+    if(!oldformat) oldformat = "-old-%d";
+    for(; keepold >= 0; keepold--)
+    {
+        formatstring(fname1 + mid)(keepold ? oldformat : "", keepold);
+        concatformatstring(fname1 + mid, ".%s", ext);
+        if(fname2[0]) rename(fname1, fname2);
+        else remove(fname1);
+        copystring(fname2, fname1);
+    }
+}
+#endif
