@@ -4,7 +4,15 @@ int wx1, wy1, wx2, wy2;
 float wsx1, wsy1, wsx2, wsy2;
 
 VARP(watersubdiv, 1, 4, 64);
-VARF(waterlevel, -128, -128, 127, if(!noteditmode("waterlevel")) hdr.waterlevel = waterlevel);
+FVARF(waterlevel, -10000, -10000, 127,
+{
+    if(!noteditmode("waterlevel"))
+    {
+        hdr.waterlevel = waterlevel * WATERLEVELSCALING;
+        unsavededits++;
+    }
+    waterlevel = float(hdr.waterlevel) / WATERLEVELSCALING; // waterlevel and hdr.waterlevel need to be kept in sync
+});
 
 void setwatercolor(const char *r, const char *g, const char *b, const char *a)
 {
@@ -22,10 +30,34 @@ void setwatercolor(const char *r, const char *g, const char *b, const char *a)
         hdr.watercolor[2] = 20;
         hdr.watercolor[3] = 178;
     }
+    if(editmode) unsavededits++;
 }
 
-COMMANDN(watercolour, setwatercolor, "ssss");
+COMMANDF(watercolour, "ssss", (char *r, char *g, char *b, char *a) { if(editmode) setwatercolor(r, g, b, a); });
+
 FVAR(waveheight, 0, 0.3f, 1.0f);
+VARP(ignoreoverride_limitwaveheight, 0, 0, 1);
+float effective_waveheight = 0.3f;
+
+void getwatercolour()
+{
+    defformatstring(res)("%d %d %d %d", hdr.watercolor[0], hdr.watercolor[1], hdr.watercolor[2], hdr.watercolor[3]);
+    result(res);
+}
+COMMAND(getwatercolour, "");
+
+const char *componentnames[] = { "red", "green", "blue", "alpha", "" };
+
+void setwatercolour(char *what, int *v)
+{
+    int t = getlistindex(what, componentnames, true, -1);
+    if(t >= 0 && t < 4 && editmode)
+    {
+        hdr.watercolor[t] = *v;
+        unsavededits++;
+    }
+}
+COMMAND(setwatercolour, "si");
 
 // renders water for bounding rect area that contains water... simple but very inefficient
 
@@ -37,10 +69,10 @@ FVAR(waveheight, 0, 0.3f, 1.0f);
     } \
     inline void vertw(float v1, float v2, float v3, float t) \
     { \
-        float angle = v1*v2*0.1f/(2*M_PI) + t; \
+        float angle = v1*v2*0.1f/PI2 + t; \
         float s = angle - int(angle) - 0.5f; \
         s *= 8 - fabs(s)*16; \
-        float h = waveheight*0.5f*s; \
+        float h = effective_waveheight*0.5f*s; \
         varray::attrib<float>(v1, v2, v3+h); \
         body; \
     }
@@ -52,7 +84,7 @@ FVAR(waveheight, 0, 0.3f, 1.0f);
     })
 #define VERTWT(vertwt, defbody, body) \
     VERTWC(vertwt, defbody, { \
-        float duv = waveheight*0.5f*v; \
+        float duv = effective_waveheight*0.5f*v; \
         body; \
     })
 
@@ -163,10 +195,13 @@ int renderwater(float hf, GLuint reflecttex, GLuint refracttex)
 {
     if(wx1<0) return nquads;
 
+    extern int mapoverride_limitwaveheight;
+    effective_waveheight = mapoverride_limitwaveheight && !ignoreoverride_limitwaveheight ? 0.1f : waveheight;
+
     wx1 -= wx1%watersubdiv;
     wy1 -= wy1%watersubdiv;
 
-    float t = fmod(float(lastmillis/300.0f/(2*M_PI)), 1.0f);
+    float t = fmod(float(lastmillis/300.0f/PI2), 1.0f);
 
     if(mtwater && maxtmus>=2 && reflecttex)
     {
@@ -273,7 +308,7 @@ void calcwaterscissor()
     loopi(4)
     {
         vec4 &p = v[i];
-        mvpmatrix.transform(vec(i&1 ? wx2 : wx1, i&2 ? wy2 : wy1, hdr.waterlevel-0.3f), p);
+        mvpmatrix.transform(vec(i&1 ? wx2 : wx1, i&2 ? wy2 : wy1, waterlevel - 0.3f), p);
         if(p.z >= 0)
         {
             float x = p.x / p.w, y = p.y / p.w;
