@@ -45,8 +45,6 @@ struct weapon
     bool modelattacking();
     void renderhudmodel(int lastaction, int index = 0);
 
-    static bool valid(int id);
-
     virtual int flashtime() const;
 };
 
@@ -59,6 +57,7 @@ struct grenades : weapon
     grenadeent *inhandnade;
     const int throwwait;
     int throwmillis;
+    int cookingmillis;
     int state;
 
     grenades(playerent *owner);
@@ -87,6 +86,7 @@ struct gun : weapon
     virtual void attackfx(const vec &from, const vec &to, int millis);
     int modelanim();
     void checkautoreload();
+    void onownerdies();
 };
 
 
